#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/App/RegisterPackages.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $
# $Change: 124736 $
#
# Desc: application to call register method of each installed MaxDB package
#		this is useful if registrations has been deleted


package DummyLog;

sub new{
	my $self = bless({},shift);
	my $file = shift;
	if($file =~ /\S/){
		open(LD,'>'.$file) or return undef;
		$self->{'file'} = $file;
	}
	return $self;
}

sub SetMsg{
	my ($self,$text) = @_;
	if(exists $self->{'file'}){
		print LD $text;
	}
	return 1;
}

sub DESTROY{
	my ($self) = @_;
	if(exists $self->{'file'}){
		close(LD);
		delete $self->{'file'};
	}	
}

1;


package SAPDB::Install::App::RegisterPackages;

use SDB::Common::Getopt::Long;
use SAPDB::Install::Registry;
use SAPDB::Install::PackageCollector;
use SAPDB::Install::InstallRegistry;
use SAPDB::Install::Values;
use SAPDB::Install::System;
use SAPDB::Install::StdIO;


sub mydiesoft{
	die($SAPDB::Install::StdIO::err);
}


sub cleanGlobals{
	
	#
	# killing global references for clean destruction
	#
	
	@SAPDB::Install::InstRegistry::Package::SELFOBJ = ();
	$SAPDB::Install::StartInstall::instReg = undef;	
	$SAPDB::Install::StartInstall::log = undef;
	$SAPDB::Install::Values::log = undef;		
}

sub main{
	local @ARGV = @_;
	my $rc = 0;
	my $opt_indep_data;
	my $opt_logfile;
	my $opt_database;
	my $opt_start_xserver;
	my $start_db;
	my $opt_dbroot;
	my ($opt_sw_all,$opt_sw_kernel,$opt_sw_client);
	
	unless(IsAdmin()){
		print2stderr ("please restart as ".($^O =~ /mswin/i ? 'local administrator' : 'root')."!\n");
		return -1;
	}
	
	my %opt_ctrl = (
		'indep_data=s' => \$opt_indep_data,
		'start_xserver' => \$opt_start_xserver,
		'start_db' => \$opt_start_db,
		'logfile=s' => \$opt_logfile,
		'd=s' => \$opt_database,
		'R=s' => \$opt_dbroot,
		'sw_all' => \$opt_sw_all,
		'sw_kernel' => \$opt_sw_kernel,
		'sw_client' => \$opt_sw_client  
	);		

	my ($data,$prog) = readIndepPath();	

	sub OptCheck{
		my $rc = 1;
		
		if ($data =~ /\S/ and defined $opt_indep_data){
			if (normalizePath($data) ne normalizePath($opt_indep_data)){
				print ("-indep_data $opt_indep_data: independent data path already registerd as $data\n");
				print ("It cannot be modified by command line option!\n");
				$rc = 0;
			}
		}
		elsif ($data !~ /\S/ and not defined $opt_indep_data){
			print "need -indep_data <path>: indepedent data path unknown\n";
			$rc = 0;
		}
		
		unless ($opt_database || $opt_sw_all || $opt_sw_kernel ||$opt_sw_client){
			print "need one of the following options: -d <db name>, -sw_all, -sw_kernel, -sw_client\n";
			$rc = 0;
		}
		
		if ((defined $opt_database or defined $opt_sw_kernel) and $opt_dbroot !~ /\S/){
			print "need -R <path>: dependent path unknown\n";	
			$rc = 0;
		}
		
		if($opt_start_db){
			unless (defined $opt_database){
				print "-start_db needs -d <db name>: database unknown\n";	
				$rc = 0;
			}
		}
		return $rc;
	}
	
			
	SDB::Common::Getopt::Long::Configure ('no_pass_through','no_ignore_case');
	if (!GetOptions(%opt_ctrl) or !OptCheck ()){
		return 42;
	}
	
	$opt_indep_data = $data unless(defined $opt_indep_data);

	unless(-f $opt_indep_data.'/config/install/INSTREG'){
		print2stderr ("InstallRegistry not found\n");
		return -1;
	}	
	my $log = DummyLog->new($opt_logfile);
	
	if ($opt_database){
		my %instances = getInstances();
		if (exists $instances{$opt_database}){
			print2stderr ("database $opt_database is already registered\n");
			return -1;
		}
	}
	
	
	my $instreg = SAPDB::Install::InstallRegistry->new($opt_indep_data.'/config/install',0,$log);
	
	$SAPDB::Install::StartInstall::instReg = $instreg;	
	$SAPDB::Install::Values::log = $log;
	$SAPDB::Install::StartInstall::log = $log;	
	
	
	#
	# overload diesoft in SAPDB::Install::System	
	# to avoid call of exit()
	#	
	#	
	
	if(exists ${'SAPDB::Install::System::'}{'diesoft'}){
		*SAPDB::Install::System::diesoft = 
			\&mydiesoft;
	}	


	#
	#	make sure that there are needed service ports
	#
	
	checkServices();



	#
	#  getting independent paths from InstallRegistry 
	#

	my %releases = $instreg->getInstallPathes('Base');
	my ($indep_prog) = keys(%releases);
	
	unless(-d $indep_prog){
		print2stderr ("Indepedent Programs in $indep_prog not found\n");
		cleanGlobals();
		return -1;
	}

	my %regdata = $instreg->getPackageData('Base',$indep_prog);
	
	#foreach my $key (keys(%$regdata))
	
	my $indep_data = $regdata{'Pathes'}->[0]->{'value'};
	
	unless(normalizePath($indep_data) eq normalizePath($opt_indep_data)){
		print2stderr ("Independent data not unique: $indep_data,$opt_indep_data\n");
		cleanGlobals();
		return -1;
	}
		
	
	$SAPDB::Install::Values::indep_data = $indep_data;	
	$SAPDB::Install::Values::indep_prog = $indep_prog;
	my $collector = SAPDB::Install::PackageCollector->new();
	

	$collector->log($log);
	
	unless(defined $collector->GenRegPackageTree($instreg)){
		print2stderr ("cannot create package tree: ".$collector->last_err."\n");
		cleanGlobals();
		return -1;
	}

	sub select_tree{
		my ($node) = @_;
		$node->{'selected'} = 1; 
		return 1 unless defined $node->{'children'};
		foreach my $child (@{$node->{'children'}}){
			select_tree($child);
		}
	}
	
	sub find_tree{
		my ($node,$packagename,$path) = @_;
		if ($node->{'name'} eq $packagename){
			if (defined $path){
				if (normalizePath($path) eq $node->{'instkey'}){
					return $node;
				}
			}
			else{
				return $node;	
			}
		
		}
		else{
			foreach my $child (@{$node->{'children'}}){
				my $found = find_tree($child,$packagename,$path);
				return $found if (defined $found);
			}
		}
		return undef;	
	}
	
	
	sub unselect_tree{
		my ($node) = @_;
		$node->{'selected'} = 0; 
		return 1 unless defined $node->{'children'};
		foreach my $child (@{$node->{'children'}}){
			unselect_tree($child);
		}
	}


	if ($opt_sw_kernel || $opt_sw_client || $opt_sw_all){
		
			#
			#	select all packages	
			#
		
			select_tree($collector->regpackage_tree);
	}

	my $utils_node;
	
	if ($opt_sw_client || $opt_sw_kernel){
		
		#
		#	unselect all server packages
		#
		
		my $unselect_root = find_tree($collector->regpackage_tree,'Server Utilities');
		$utils_node = $unselect_root;
		unselect_tree($unselect_root) if defined $unselect_root;	
	}
	
	if ($opt_sw_kernel){
		
		#
		#	select Server Utilities
		#
	
		$utils_node->{'selected'} = 1;
				
		#
		# select Database Kernel in <dependent path>
		#
				
		my $kernel_node = find_tree ($collector->regpackage_tree,'Database Kernel',$opt_dbroot);
		if (defined $kernel_node){
			select_tree($kernel_node);
		}
		else{
			print2stderr ("no database kernel found in $opt_dbroot\n");
			cleanGlobals();
			return -1;
		}
	}
	
	#
	#	getting packages
	#		
	
	my $packages = $collector->GenRegPackages($instreg);
	unless(defined $packages){
		print2stderr ("cannot get packages: ".$collector->last_error."\n");
		cleanGlobals();
		return -1;		
	}
	


	while(1){
		last if $#{@$packages} == -1;
		my $package = pop @$packages;
		my $name = $package->DispName;
		if($package->Name =~ /^PCR/){
			
			#
			#  modify PCR scripts to workaround
			#  needed preinstall() initialization
			#		
	
			local *buff = $package->Script;
			$buff =~ s/my\s\$prog\;/my\ \$prog\ =\ \$SAPDB::Install::Values::indep_prog\;/m;	
		}
		
		$package->evalScript;		
		
		if($package->Name eq 'Database Kernel'){	
			
			#
			# initialize $indep_prog in kernel package to workaround 
			# needed preinstall() initialization
			#				

			unless(${'SAPDB::Install::Uninst'.$package->ID.'::indep_prog'} =~ /\S/){
				${'SAPDB::Install::Uninst'.$package->ID.'::indep_prog'} =
				$SAPDB::Install::Values::indep_prog;
	
			}	
		}
		elsif($package->Name eq 'XML Indexing Engine'){
			
			#
			# overload register() to workaround
			# loading module Hostname with unsupported repository mechanism 
			#

			$package->register( 
				sub{
					require SAPDB::Install::Hostname;
					*{'SAPDB::Install::Uninst'.$package->ID.'::hostname'} = 
						\&SAPDB::Install::Hostname::hostname;
					if($^O =~ /mswin/i){
						&{'SAPDB::Install::Uninst'.$package->ID.'::registerWin'}();	
					}
					else{
						&{'SAPDB::Install::Uninst'.$package->ID.'::registerUX'}();	
					}
				});				
		}
	
		#
		# overload diesoft in package script		
		# to avoid call of exit()	
		#	

		if(exists ${'SAPDB::Install::Uninst'.$package->ID.'::'}{'diesoft'}){
			*{'SAPDB::Install::Uninst'.$package->ID.'::diesoft'} =
				\&mydiesoft;
		}
		
		
		#
		# call register()
		#

		if(defined $package->register){
			print "registering package $name [".$package->Path."]\n";
			eval{
				&{$package->register}();
			};
			if($@){
				$rc = -1;
				print2stderr ("registering $name [".$package->Path."] failed\n");
			};	
		}
	}
	
	if ($opt_start_xserver){
		require SAPDB::Install::Service::Communication;
		my $xserver = SAPDB::Install::Service::Communication->new();
		if ($xserver){
			my $rc = $xserver->start();
			if (defined $rc){
				print2stdout ("communication service (x_server) started successfully\n");
			}
			else{
				print2stderr ("cannot start communication service (x_server): ".$xserver->last_error."\n");
				$rc = -1;
			}
		}
	}
	
	if ($opt_database){
		require SAPDB::Install::Service::DBM;
		my $dbm = SAPDB::Install::Service::DBM->new($opt_database);
		
		if(defined $dbm->db_reg($opt_dbroot)){
			print2stdout ("database $opt_database registered successfully\n");
			if ($opt_start_db){
				if (defined $dbm->db_online()){
					print2stdout ("database $opt_database started successfully\n");
				}
				else{
					print2stderr ("cannot start database \"$opt_database\": ".$dbm->{'last_error'}."\n");
					$rc	= -1;
				}
			}
		}
		else{
			print2stderr ("cannot register database \"$opt_database\": ".$dbm->{'last_error'}."\n");
			$rc = -1;
		}
	}
	
	cleanGlobals();
	return $rc;
}
