#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/Control/PathCtrl.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $
# $Change: 124736 $
#
# Desc: 

package SAPDB::Install::Control::PathCtrl;

use Wx::Event qw(EVT_BUTTON EVT_SIZE EVT_KEY_DOWN
				 EVT_SET_FOCUS EVT_KILL_FOCUS
				EVT_TEXT_ENTER);
use Wx qw (wxHORIZONTAL wxEXPAND wxDD_NEW_DIR_BUTTON
			wxTE_PROCESS_TAB wxTE_PROCESS_ENTER
			wxWANTS_CHARS WXK_TAB wxEVT_COMMAND_TEXT_ENTER
			wxALIGN_CENTER_VERTICAL);
use SAPDB::Install::Resources;
use SAPDB::Install::Tools;


our @ISA = ($^O =~ /mswin/i ? qw (Wx::Control) : qw (Wx::Panel));

our $bitmap_folder = Wx::Bitmap->new(SAPDB::Install::Resources->new('RESOURCES.TGZ')->
						getImages('icon/folder.png')->{'icon/folder.png'});

sub new{
	my ($this ) = shift->SUPER::new(@_);
	$this->SetWindowStyleFlag(wxWANTS_CHARS);

	if ($^O =~ /mswin/i){
		EVT_SIZE($this,\&OnSize);
		EVT_SET_FOCUS($this,\&OnSetFocus);
	}
	my $sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	$this->{'edit'} = Wx::TextCtrl->new($this,-1,'bitmap_folder',[0,0],[-1,-1],wxTE_PROCESS_TAB | wxTE_PROCESS_ENTER);
	$this->{'button'} = Wx::BitmapButton->new($this,-1,$bitmap_folder,[0,0],[-1,-1]);
	$sizer->Add($this->{'edit'},1,wxALIGN_CENTER_VERTICAL);
	$sizer->Add($this->{'button'});
	EVT_BUTTON($this,$this->{'button'}->GetId,
		sub {
			my ($self,$evt) = @_;
			my $dirdlg = Wx::DirDialog->new($self,'Choose a directory',$self->{'edit'}->GetValue,wxDD_NEW_DIR_BUTTON);
			my $dir = $self->{'edit'}->GetValue,
			$dir = getRealPathName($dir) if $^O =~ /mswin/i;
			$dirdlg->SetPath($dir);
			$dirdlg->ShowModal;
			my $result = $dirdlg->GetPath;
			$result =~ s/\\/\//g  unless $^O =~ /mswin/i;
			$result = getRealPathName($result) if $^O =~ /mswin/i;
			$self->{'edit'}->SetValue($result) if $result =~ /\S/;
			$self->GenerateTextEnterEvent;
		}
	);
	
	EVT_TEXT_ENTER($this,$this->{'edit'}->GetId,
		sub{
			my ($self,$evt) = @_;
			$self->GenerateTextEnterEvent;
		}
	);
	
	
	EVT_KEY_DOWN($this->{'edit'},	
			sub{
				my ($self,$evt) = @_;
				if($evt->GetKeyCode == WXK_TAB){
					$this->{'button'}->SetFocus;				
				}
				else{
					$evt->Skip(1);
				}
			}
	);

	#EVT_KEY_DOWN($this->{'button'},	
	#		sub{
	#			my ($self,$evt) = @_;
	#			if($evt->GetKeyCode == WXK_TAB){
	#				print ">>>>>>> catch TAB\n";
	#			}
	#			else{
	#				$evt->Skip(1);
	##			}
	#		}
	#);


	$this->SetSizerAndFit($sizer);
	$this->SetAutoLayout(1);
	$this->Layout;
	return $this;
}

sub OnSize{
	my ($self,$evt) = @_;
	$self->Layout();
	$evt->Skip(1);
}

sub GenerateTextEnterEvent{
	my ($self) =  @_;
	$self->GetParent->AddPendingEvent(Wx::CommandEvent->new(wxEVT_COMMAND_TEXT_ENTER,$self->GetId));		 
}

sub OnSetFocus{
	my ($self,$evt) = @_;
	if(defined $self->{'edit'} and defined $self->{'button'}){
		$self->{'edit'}->SetFocus();
		$self->{'edit'}->SetSelection(-1,-1);
	}else{
		$evt->Skip(1);
	}
}

sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	if(defined $self->{'edit'}){
		$self->{'edit'}->Destroy;
		delete $self->{'edit'};
	}
	if(defined $self->{'button'}){
		$self->{'button'}->Destroy;
		delete $self->{'button'};
	}
	$self->SUPER::Destroy;
}

sub SetValue{
	my ($self,$value) = @_	;
	$value = getRealPathName($value) if $^O =~ /mswin/i;
	$self->{'edit'}->SetValue($value);
	return 1;
}

sub Show{
	my ($self,$show) = @_;
	$self->{'edit'}->Show($show);
	$self->{'button'}->Show($show);	
	$self->SUPER::Show($show);
}
	

sub GetValue{
	my ($self) = @_	;
	my $value = $self->{'edit'}->GetValue();
		
	#
	#	remove trailing slashes / backslashes
	#
			
	$value =~ s/[\\\/]*$//;
	
	#
	#	remove doubled slashes / backslashes (not at the begin -> unc path)
	#
			
	$value =~ s/(.)\\{2,}/$1\\/g;
	$value =~ s/(.)\/{2,}/$1\//g;	
	
	$value = getRealPathName($value) if $^O =~ /mswin/i;
	return $value;
}

sub DESTROY{
	my ($self) =  @_;
	print ">>>> DESTROY $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;