#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/Control/TreeCtrl.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $
# $Change: 124736 $
#
# Desc: 

package SAPDB::Install::Control::TreeCtrl;

use Wx::Event qw (EVT_LEFT_DOWN EVT_KEY_DOWN);
use Wx qw (wxTreeItemIcon_Selected wxTREE_HITTEST_ONITEMICON);

our @ISA = qw (Wx::TreeCtrl);

sub new{
	my($this ) = shift->SUPER::new(@_);
	EVT_LEFT_DOWN($this,\&OnItemSelect);
	EVT_KEY_DOWN($this,\&OnItemSelect);
	return $this;
}

sub Destroy{
	my ($self) = @_;
	$self->SUPER::Destroy;
}

sub OnItemSelect{
	my ($self,$evt) = @_;
	my $item;
	if ($evt->isa('Wx::KeyEvent')) {
      	return $evt->Skip (1) unless $evt->GetKeyCode() == 32;
        $item = $self->GetSelection;
    }
	else{
		($item, my $flags) = $self->HitTest($evt->GetPosition);
		return $evt->Skip(1) unless $flags & wxTREE_HITTEST_ONITEMICON;
		$evt->Skip (0) if $evt->ButtonDClick;
		
	}
	$self->CheckItem($item);
}

sub UpdateItems{
	my ($self,$item,$text) = @_;
	return unless $self->ItemHasChildren($item);
	my @children;
	my ($child_id, $cookie) = $self->GetFirstChild ($item);
	push @children,$child_id;
	foreach (2 .. $self->GetChildrenCount($item,0)){
		($child_id, $cookie) = $self->GetNextChild ($item,$cookie);
		last unless defined $child_id;
		push @children,$child_id;
	}
	foreach my $child (@children){
		if($self->GetItemText($child) eq $text){
			next if $self->GetPlData($child)->{'type'} == 1;
			if($self->GetPlData($child)->{'selected'}){
				$self->SetItemImage($child,1);
				$self->SetItemImage($child,1,wxTreeItemIcon_Selected);
				unless($self->GetPlData($item)->{'selected'}){
					$self->CheckParent($child);
				}
			}
			else{
				$self->SetItemImage($child,2);
				$self->SetItemImage($child,2,wxTreeItemIcon_Selected);
			}	
		}
		$self->UpdateItems($child,$text);
	}
	
}



sub CheckItem{
	my ($self,$item) = @_;
	my $text = $self->GetItemText($item);
	if(defined $self->GetPlData($item)){
		return undef if($self->GetPlData($item)->{'type'} == 1);
		if($self->GetPlData($item)->{'selected'}){
			$self->GetPlData($item)->{'selected'} = 0;
			$self->SetItemImage($item,2);
			$self->SetItemImage($item,2,wxTreeItemIcon_Selected);
			$self->UncheckChildren($item);
		}
		else{
			$self->GetPlData($item)->{'selected'} = 1;
			$self->SetItemImage($item,1);
			$self->SetItemImage($item,1,wxTreeItemIcon_Selected);
			$self->CheckParent($item);
		}
		if(exists $self->{'dependent_nodes'}->{$text}){
			foreach my $ditem (@{$self->{'dependent_nodes'}->{$text}}){
				next if($ditem !~ /\S/);
				$self->UpdateItems($self->GetRootItem(),$ditem);
			}		
		}
		
	 
	}	
}


sub CheckParent{
	my ($self,$item) = @_;
	my $parent = $self->GetItemParent($item);
	return unless defined $parent;
	return unless defined $self->GetPlData($parent);
	return if $self->GetPlData($parent)->{'selected'};
	$self->CheckItem($parent);	
}

sub UncheckChildren{
	my ($self,$item) = @_;
	return unless $self->ItemHasChildren($item);
	my @children;
	my ($child_id, $cookie) = $self->GetFirstChild ($item);
	push @children,$child_id;
	foreach (2 .. $self->GetChildrenCount($item,0)){
		($child_id, $cookie) = $self->GetNextChild ($item,$cookie);
		last unless defined $child_id;
		push @children,$child_id;
	}
	foreach my $child (@children){
		if($self->GetPlData($child)->{'selected'}){
			$self->CheckItem($child);
		}
	}
}


sub CheckChildren{
	my ($self,$item) = @_;
	return undef unless $self->ItemHasChildren($item);
	my @children;
	my ($child_id, $cookie) = $self->GetFirstChild ($item);
	push @children,$child_id;
	foreach (2 .. $self->GetChildrenCount($item,0)){
		($child_id, $cookie) = $self->GetNextChild ($item,$cookie);
		last unless defined $child_id;
		push @children,$child_id;
	}
	foreach my $child (@children){
		unless($self->GetPlData($child)->{'selected'}){
			$self->CheckItem($child);
		}
		$self->CheckChildren($child);
	}
}


sub UncheckAllChildren{
	my ($self,$item) = @_;
	return undef unless $self->ItemHasChildren($item);
	my @children;
	my ($child_id, $cookie) = $self->GetFirstChild ($item);
	push @children,$child_id;
	foreach (2 .. $self->GetChildrenCount($item,0)){
		($child_id, $cookie) = $self->GetNextChild ($item,$cookie);
		last unless defined $child_id;
		push @children,$child_id;
	}
	foreach my $child (@children){
		if($self->GetPlData($child)->{'selected'}){
			$self->CheckItem($child);
		}
		$self->UncheckAllChildren($child);
	}
}




sub DESTROY{
	my ($self) =  @_;
	print ">>>> DESTROY $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;