#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/Instance/Check/DataCons/OLTP.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $
# $Change: 124736 $
#

package SAPDB::Install::Instance::Check::DataCons::OLTP;

use SAPDB::Install::Instance::Base;
use SAPDB::Install::Instance::Check::Common;
use SAPDB::Install::Instance::Check::DataCons::ConData;

push @ISA, 
	'SAPDB::Install::Instance::Base',
	'SAPDB::Install::Instance::Check::Common';

#
# datacons ()
# check if data are consistent for migration
# if instance type is 'OLTP' instance must be restartable without log
#
sub datacons {
	my ($self) = @_;
	my $dbm = $self->{dbm};

	$self->set_errorstate ('ERROR');
	$self->msgbegin ("checking data consistency");
	$self->msg0 ("checking data consistency for database migration...\n");

	$self->{instancetypename} = 'database';

	#
	# make sure that we know our mirgation strategy
	#
	my $migration_strategy = $self->get_migrationstrategy ();
	unless (defined $migration_strategy) {
		$self->msgend ();
		return -1;
	}

	if ($migration_strategy =~ /^COMPATIBLE_LOG/) {

		$self->set_errorstate ('OK');
		$self->msgend ();
		return 0;

	} elsif ($migration_strategy =~ /^CONSISTENT_DATA/) {

		# save prev class before setting new class 
		my ($prev) = ref ($self);

		my $class =
		'SAPDB::Install::Instance::Check::DataCons::ConData';

		bless ($self, $class);

		my $rc = $self->datacons ();

		# restore prev class before returning
		bless ($self, $prev);
		return $rc;
	}

	$self->msgend ();
	return -1;
}

1;
