#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/PackageCollector.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $
# $Change: 124736 $
#
# Desc: 

package SAPDB::Install::PackageCollector;


require SDB::Common::Exporter;

use SAPDB::Install::Base;
use SAPDB::Install::Untgz;
use SAPDB::Install::StdIO;
use SAPDB::Install::Tools;
use SAPDB::Install::Trace;
use SAPDB::Install::PackageObj;
use SAPDB::Install::SysInfo;
use SAPDB::Install::Values;
use SAPDB::Install::Registry;
use SAPDB::Install::System;

our @ISA = qw (SDB::Common::Exporter SAPDB::Install::Base);
our @EXPORT = qw (collect checkRequired);

our $DEBUG=0;


our $sysinfo = GetSystemInfo();

sub parseRequire{
	my ($reqString)=@_;
	my @ops=('>=','<=','=','>','<'); # operators in search order
	my $delimiter = ',';
	my @packages = split($delimiter,$reqString);
	my @returnvalue;
	my $first = 1;
	foreach my $package (@packages){
		$_ = $package;
		my %required_package;
		foreach my $operator (@ops){
			if(/$operator/){
				if(/^\s*(\S.*\S)\s*$operator\s*([0-9]{1,}\.[0-9]{1,}\.[0-9]{1,}\.[0-9]{1,})\s*(\d*)\s*/){
					# found valid operator and release -> set hash 
					$required_package{'name'} = $1;
					$required_package{'operator'} = $operator;
					$required_package{'release'} = $2;
					$required_package{'bit'} = $3 if ($3 == 32 or $3 == 64); 		
					last;
				}

				# found operand without valid release -> ignore operator and wrong release
				s/\s*$operator.*$//; 
				s/^\s*//;
				$required_package{'name'} = $_;
				last;
			}
		}
		$required_package{'first'} = $first;
		$first = 0;
		unless(exists $required_package{'name'}){
			s/^\s*//;
			s/\s*$//;
			$required_package{'name'} = $_;	
		}
		push(@returnvalue,\%required_package);
	}
	return @returnvalue; # return an array of hash references; hashes with keys: name (required), operator (optional) and release (optional) 
}


sub checkRequired{
	my (%args) = @_;
	my %required = %{$args{'required'}};
	my $pack_ok = 0;
	if($args{'package_name'} eq $required{'name'}){
		if(exists $required{'operator'}){
			TraceMsg("TEST WITH OTHER PACKAGE ".release2num($args{'version'})." $required{'operator'} ".release2num($required{'release'})."\n",4,\$DEBUG); 
			$_=$required{'operator'};
			SWITCH:{
				/^=$/ && do{
								release2num($args{'version'}) == release2num($required{'release'}) or last SWITCH;
								if($required{'bit'}){
									$required{'bit'} eq $args{'mode'} or TraceMsg("object mode dont match\n",4,\$DEBUG) and last SWITCH;
								}
								$pack_ok=1;
								last SWITCH;
							};	
				/^>$/ && do{
								release2num($args{'version'}) > release2num($required{'release'}) or last SWITCH;
								if($required{'bit'}){
									$required{'bit'} eq $args{'mode'} or TraceMsg("object mode dont match\n",4,\$DEBUG) and last SWITCH;
								}
								$pack_ok=1;
								last SWITCH;
							};
				/^<$/ && do{
								release2num($args{'version'}) < release2num($required{'release'}) or last SWITCH;
								if($required{'bit'}){
									$required{'bit'} eq $args{'mode'} or TraceMsg("object mode dont match\n",4,\$DEBUG) and last SWITCH;
								}
								$pack_ok=1;
								last SWITCH;
							};
				/^>=$/ && do{
								release2num($args{'version'}) >= release2num($required{'release'}) or last SWITCH;
								if($required{'bit'}){
									$required{'bit'} eq $args{'mode'} or TraceMsg("object mode dont match\n",4,\$DEBUG) and last SWITCH;
								}
								$pack_ok=1;
								last SWITCH;
							};
				/^<=$/ && do{
								release2num($args{'version'}) <= release2num($required{'release'}) or last SWITCH;
								if($required{'bit'}){
									$required{'bit'} eq $args{'mode'} or TraceMsg("object mode dont match\n",4,\$DEBUG) and last SWITCH;
								}
								$pack_ok=1;
								last SWITCH;
							};
			}
		}
		else{
			
			$pack_ok = 1;
		}
	}
	return $pack_ok;
}





sub new{
	my ($class,@args) = @_;
	my $this = $class->SUPER::new();

	#define properties
	$this->{'archive_dir'} = undef;
	$this->{'packages'} = undef;
	$this->{'package_index'} = undef;
	$this->{'regpackage_index'} = undef;
	$this->{'last_error'} = undef;
	$this->{'package_index'} = undef;
	$this->{'instance_index'} = undef;		
	$this->{'log'} = undef;	
	$this->{'package_tree'} = undef;	
	
	
	return $this;
}


sub SetErrMsg{
	my ($self,$msg) = @_;
	if (defined $self->log){
		$self->log->SetMsg("ERR: Collector: $msg\n");
	}
	$self->last_error($msg);
	return 1;
}


sub GenPackageIndex{
	my ($self) = @_;
	unless(-d $self->archive_dir){
		$self->SetErrMsg("cannot find archive directory ".$self->archive_dir);	
		return undef; 
	}
	unless(opendir(DH,$self->archive_dir)){
		$self->SetErrMsg("cannot open archive directory ".$self->archive_dir.": $!");	
		return undef;
	}
	my @archives = grep {-f $self->archive_dir.'/'.$_ && 
						/\.tgz$/i && not /^sdbinst.tgz$|^wxperl.tgz$|^resources.tgz$/i }
						 readdir(DH);
	closedir(DH);
	if($#archives == -1){
		$self->SetErrMsg("no MaxDB software package found in directory ".
						   $self->archive_dir);	
		return undef;
	}
	if(defined $self->log){
		$self->log->SetMsg('MSG: collector found '.
							($#archives + 1)." archives\n");
	}
	my %infos;	
	foreach my $archive (@archives){
		#
		#	refresh gui
		#
		if(defined $SAPDB::Install::Values::RefreshGui){
			&{$SAPDB::Install::Values::RefreshGui}();
		}
		my $untgz = SAPDB::Install::Untgz::new();
		$untgz->Open($self->archive_dir.'/'.$archive);
		my $header = $untgz->Next();
		unless($header){
			if(defined $self->log){
				$self->log->SetMsg("WRN: cannot read archive $archive: ".
									join(': ',$untgz->GetErr)."\n");
			}	
			next;
		}
		my ($filename) = ($header =~ /(\S+)$/);	
		$filename =~ s/^\.\///;
		unless($filename eq 'PACKAGEDATA'){
			if(defined $self->log){
				$self->log->SetMsg(
					"WRN: archive $archive has no ".
					"PACKAGEDATA: skipping it\n");
			}
			next;
		}
		my $data;
		my %packagedata;
		$untgz->ExtractScalar($data);
		foreach my $line (split("\n",$data)){
			next if $line =~ /^\s*#/;
			my ($key,$value) = ($line =~ /^\s*(\S+)\s*=\s*"(.*)"\s*$/);
			next unless $key =~ /\S/; 
			unless (exists $SAPDB::Install::Values::package_interface{$key}){
				$self->log->SetMsg("WRN: ignore unkown key \"$key\" in package \"".$packagedata{'PACKAGE_NAME'}.
							  "\" ($archive)\n");
				next;
			}
			$packagedata{$key} = $value;
		}
		unless($packagedata{'INTERFACE_VERSION'} eq '0.1'){
			if(defined $self->log){
				$self->log->SetMsg("WRN: archive $archive: hasn\'t ".
									"the right interface version: skipping it\n");
			}	
			next;
		}
		
		if(exists $infos{$packagedata{'PACKAGE_NAME'}}){
			$self->SetErrMsg("Found package \"".$packagedata{'PACKAGE_NAME'}.
							  "\" in archive \"$archive\", but it's already ".
							  "defined in archive \"".
							  $infos{$packagedata{'PACKAGE_NAME'}}{'ARCHIVE_NAME'}.'"');
			return undef;
		} 
		
		
		
		my $try_num = 0;
		
		if($packagedata{'SCRIPT'} =~ /\S/){
			$try_num++;	
		}
		
		if($packagedata{'FILELIST'} =~ /\S/){
			$try_num++;	
		}

		foreach (1..$try_num){		
			$header = $untgz->Next;		
			unless($header){
				if(defined $self->log){
					$self->log->SetMsg("WRN: cannot read archive $archive: ".
									join(': ',$untgz->GetErr)."\n");
				}	
				next;
			}
			($filename) = ($header =~ /(\S+)$/);
			if($filename eq $packagedata{'SCRIPT'}){
				my $script;
				$untgz->ExtractScalar($script);
				$packagedata{'SCRIPT_VALUE'} = $script;
			}
			elsif($filename eq $packagedata{'FILELIST'}){
				my $filelist;
				$untgz->ExtractScalar($filelist);
				my %file_list;
				foreach my $line (split("\n",$filelist)){
						my ($file,$md5sum) = 
								($line =~ /^"(.+)"\s+([a-f0-9]{32})\s*$/);
						$file_list{$file} = $md5sum;
				}
				$packagedata{'FILELIST_VALUE'} = \%file_list;
			}
			else{
				next;
			}
		}
		unless(exists $packagedata{'FILELIST_VALUE'}){
			if(defined $self->log){
				$self->log->SetMsg("WRN: no filelist found in archive $archive: ".
								join(': ',$untgz->GetErr)."\n");
			}
		}
		unless(exists $packagedata{'SCRIPT_VALUE'}){
			if(defined $self->log){
				$self->log->SetMsg("WRN: no script found in archive $archive: ".
								join(': ',$untgz->GetErr)."\n");
			}
		}
		$packagedata{'ARCHIVE_NAME'} = $self->{'archive_dir'}.'/'.$archive;
		$packagedata{'UNTGZ'} = $untgz;
		if($packagedata{'BUILD_STRING'} =~ /\S/){
			
			my $build_info = getBuildStringInfo($packagedata{'BUILD_STRING'});
			$packagedata{'PURPOSE'} = $build_info->{'purpose'};
			$packagedata{'CHANGELIST'} = $build_info->{'changelist'};
			$packagedata{'QA_LEVEL'} = $build_info->{'qa_level'};
			$packagedata{'MAKE_TYPE'} = $build_info->{'make_type'};		
		}
		
		if($packagedata{'TEST_FILE'} =~ /\S/){
		
			my $make_sysinfo = {
				'system' => undef,
				'architecture' => undef,
				'version' => undef,
				'subversion' => undef,
				'c_runtime' => undef,
				'cpp_runtime' => undef,
				'cpp_runtime64' => undef
			};
			
			my $sysinfo_in_packagedata = 0;

			foreach my $sysinfokey (keys(%$make_sysinfo)){
				my $package_data_key = $sysinfokey;
				$package_data_key =~ tr/[a-z]/[A-Z]/;
				$package_data_key = 'SYSINFO.'.$package_data_key;
				if(exists $packagedata{$package_data_key}){
					$make_sysinfo->{$sysinfokey} = $packagedata{$package_data_key};
					$sysinfo_in_packagedata = 1;
				}
			}
		
		
		
			my $magic = $packagedata{'MAGIC'};
			
			my @make_sysinfo_keys;
			
			my $pack_buildinfo;
			
			if (not ($magic =~ /\S/)){
				$pack_buildinfo = getBuildInfoTGZ($untgz,$packagedata{'TEST_FILE'});
				$magic = $pack_buildinfo->{'magic'};
				$packagedata{'MAGIC'} = $magic;
				unless($sysinfo_in_packagedata){
					$make_sysinfo = $pack_buildinfo->{'sysinfo'};
					@make_sysinfo_keys = keys(%$make_sysinfo);	
				}
			}	
		
			if(not $sysinfo_in_packagedata and $#make_sysinfo_keys == -1 and $^O !~ /mswin/i){
				$self->log->SetMsg("WRN: software in package \"".$packagedata{'PACKAGE_NAME'}."\" ($archive) has no sysinfo structure\n");				
				$self->log->SetMsg("WRN: runnable check is impossible!\n");
			}
			else{
		
				#
				#	let PCR 7301 and PCR 7301 linux i386 installable on linux opteron
				#
				#	fake sysinfo.architecture to pass IsRunnable()
				#
		
				local $sysinfo = $sysinfo;
				
				unless($^O =~ /mswin/i){
					unless(defined $pack_buildinfo){
						$pack_buildinfo = getBuildInfoTGZ($untgz,$packagedata{'TEST_FILE'});
						unless($sysinfo_in_packagedata){
							$make_sysinfo = $pack_buildinfo->{'sysinfo'};
						}
					}
					if(	$packagedata{'PACKAGE_NAME'} =~ /^PCR\s73\d\d$/ &&
						$make_sysinfo->{'architecture'} eq 'I386' &&
						$make_sysinfo->{'system'} eq 'Linux' &&
						$sysinfo->{'system'} eq 'Linux' && $sysinfo->{'architecture'} eq 'X86-64'){
						my %hSysinfo = %$sysinfo;
						$hSysinfo{'architecture'} = 'I386';						
						$sysinfo = \%hSysinfo;
					}
				}
						
				unless(IsRunnable($sysinfo,$make_sysinfo,$magic)){		
						$self->log->SetMsg("WRN: software in package \"".$packagedata{'PACKAGE_NAME'}."\" ($archive) isn\'t runnable on your system\n");	
						$self->log->SetMsg("magic string: $magic\n");
						foreach my $key (sort keys(%$sysinfo)){
							$self->log->SetMsg("YOUR SYSTEM: $key = ".$sysinfo->{$key}."\n");
						}
						foreach my $key (sort keys(%$make_sysinfo)){
							next unless $make_sysinfo->{$key} =~ /\S/;
							$self->log->SetMsg("MAKE SYSTEM: $key = ".$make_sysinfo->{$key}."\n");
						}
						next;
				}
			}
		}
		
		if (exists $packagedata{'DBG_PACKAGE'} && -f $self->archive_dir.'/'.$packagedata{'DBG_PACKAGE'}){
			my $dbg_untgz = SAPDB::Install::Untgz::new();
			my %dbg_data;
			$dbg_untgz->Open($self->archive_dir.'/'.$packagedata{'DBG_PACKAGE'});
			if(!$dbg_untgz->GetErr){
				my $header = $dbg_untgz->Next();
				if($header){
					my ($filename) = ($header =~ /(\S+)$/);	
					$filename =~ s/^\.\///;
					if($filename eq 'PACKAGEDATA'){
						my $data;
						$dbg_untgz->ExtractScalar($data);
						foreach my $line (split("\n",$data)){
							next if $line =~ /^\s*#/;
							my ($key,$value) = ($line =~ /^\s*(\S+)\s*=\s*"(.*)"\s*$/);
							next unless $key =~ /\S/; 
							unless (exists $SAPDB::Install::Values::package_interface{$key}){
								$self->log->SetMsg("WRN: ignore unkown key \"$key\" in debug package of \"".$packagedata{'PACKAGE_NAME'}.
												  "\" ($archive)\n");
						
							}
							$dbg_data{$key} = $value;
						}
			
					
					}
					else{
						if(defined $self->log){
							$self->log->SetMsg(
								"WRN: debug archive ".$packagedata{'DBG_PACKAGE'}." has no ".
								"PACKAGEDATA: skipping it\n");
						}
					}
					$header = $dbg_untgz->Next();
					if($header){
						my ($filename) = ($header =~ /(\S+)$/);	
						$filename =~ s/^\.\///;
						if($filename eq 'FILELIST'){
							my $dbgfilelist;
							$dbg_untgz->ExtractScalar($dbgfilelist);
							my %dbgfile_list;
							foreach my $line (split("\n",$dbgfilelist)){
								my ($file,$md5sum) = 
									($line =~ /^"(.+)"\s+([a-f0-9]{32})\s*$/);
								$dbgfile_list{$file} = $md5sum;
							}
							$packagedata{'DBG_FILELIST_VALUE'} = \%dbgfile_list;
						}
						else{
							if(defined $self->log){
								$self->log->SetMsg(
									"WRN: debug archive ".$packagedata{'DBG_PACKAGE'}." has no ".
									"FILELIST: skipping it\n");
							}
							delete $packagedata{'DBG_PACKAGE'};	
						}
					}
					else{
						if(defined $self->log){
							$self->log->SetMsg("WRN: cannot read debug package ".$packagedata{'DBG_PACKAGE'}.": ".
											join(': ',$dbg_untgz->GetErr)."\n");
						}
						delete $packagedata{'DBG_PACKAGE'};	
					}
				}
				else{
					if(defined $self->log){
						$self->log->SetMsg("WRN: cannot read debug ".$packagedata{'DBG_PACKAGE'}.": ".
										join(': ',$dbg_untgz->GetErr)."\n");
					}
					delete $packagedata{'DBG_PACKAGE'};	
				}
			}
			else{
				if(defined $self->log){
						$self->log->SetMsg("cannot open debug package ".
							$packagedata{'DBG_PACKAGE'}.": ".join(': ',$dbg_untgz->GetErr)."\n");
				}
				delete $packagedata{'DBG_PACKAGE'};
			}
			
			if (not defined $dbg_data{'PACKAGE_NAME'}){
				if(defined $self->log){
					$self->log->SetMsg("WRN: debug package ".$packagedata{'DBG_PACKAGE'}." isn\'t valid: no package name\n");
				}
				delete  $packagedata{'DBG_PACKAGE'};
			}
			elsif ($dbg_data{'PACKAGE_NAME'} ne $packagedata{'PACKAGE_NAME'}){
				if(defined $self->log){
					$self->log->SetMsg("WRN: debug package ".$packagedata{'DBG_PACKAGE'}." isn't valid: package name doesn\'t match\n");
					$self->log->SetMsg("WRN: ".$packagedata{'PACKAGE_NAME'}." vs. ".$dbg_data{'PACKAGE_NAME'}."\n");
				}
				delete  $packagedata{'DBG_PACKAGE'};
			}
			elsif ($packagedata{'BUILD_STRING'} ne $dbg_data{'BUILD_STRING'}){
				if(defined $self->log){
					$self->log->SetMsg("WRN: debug package ".$packagedata{'DBG_PACKAGE'}." isn't valid: build string doesn\'t match\n");
					$self->log->SetMsg("WRN: ".$packagedata{'BUILD_STRING'}." vs. ".$dbg_data{'BUILD_STRING'}."\n");
				}
				delete  $packagedata{'DBG_PACKAGE'};
			}
			elsif ($packagedata{'MAKE_ID'} ne $dbg_data{'MAKE_ID'}){
				if(defined $self->log){
					$self->log->SetMsg("WRN: debug package ".$packagedata{'DBG_PACKAGE'}." isn't valid: make id doesn\'t match\n");
					$self->log->SetMsg("WRN: ".$packagedata{'MAKE_ID'}." vs. ".$dbg_data{'MAKE_ID'}."\n");
				}
				delete  $packagedata{'DBG_PACKAGE'};
			}
			else{
				if(defined $self->log){
					$self->log->SetMsg("MSG: debug package ".$packagedata{'DBG_PACKAGE'}." is valid\n");
				}
				$packagedata{'DBG_MD5SUM'} = $dbg_data{'PACKAGE_MD5SUM'};
				$packagedata{'DBG_SIZE'} = $dbg_data{'SIZE'};
				$packagedata{'DBG_BUILDSTRING'} = $dbg_data{'BUILD_STRING'};
				$packagedata{'DBG_MAKEID'} = $dbg_data{'MAKE_ID'};
				$packagedata{'DBG_TGZHANDLE'} = $dbg_untgz;
			}
		}
		else{
			if(exists $packagedata{'DBG_PACKAGE'} and defined $self->log){
					$self->log->SetMsg("debug package ".$packagedata{'DBG_PACKAGE'}." not found\n");
					delete $packagedata{'DBG_PACKAGE'};
			}
		}
		
		$infos{$packagedata{'PACKAGE_NAME'}} = \%packagedata; 
	}
	$self->package_index(\%infos);
	return 1;
}


sub GenRegPackageIndex{
	my ($self,$registry) = @_;
	my $infos = {};
	my $regdata = ${%{$registry->RegData}}{'HashRef_Packages'};
	foreach my $package_name (keys(%$regdata)){
		foreach my $instpath (keys(%{$regdata->{$package_name}})){
			$infos->{$package_name}->{$instpath}->{'PACKAGE_NAME'} = $package_name;
			$infos->{$package_name}->{$instpath}->{'MODE'} = 
				$regdata->{$package_name}->{$instpath}->{'Mode'};					
			$infos->{$package_name}->{$instpath}->{'SOFTWARE_VERSION'} =
				$regdata->{$package_name}->{$instpath}->{'Version'};
			$infos->{$package_name}->{$instpath}->{'VALID'} =
				$regdata->{$package_name}->{$instpath}->{'Valid'};
			$infos->{$package_name}->{$instpath}->{'DATE'} =
				$regdata->{$package_name}->{$instpath}->{'Date'};
			$infos->{$package_name}->{$instpath}->{'OBJFILE_MAGIC'} =
				$regdata->{$package_name}->{$instpath}->{'ObjMagic'};
			$infos->{$package_name}->{$instpath}->{'DISPLAY_NAME'} =
				$regdata->{$package_name}->{$instpath}->{'DispName'};
			$infos->{$package_name}->{$instpath}->{'DISPLAY_NAME'} = $package_name
					unless ($infos->{$package_name}->{$instpath}->{'DISPLAY_NAME'} =~ /\S/); 

			$infos->{$package_name}->{$instpath}->{'SIZE'} =
				$regdata->{$package_name}->{$instpath}->{'Size'};
			$infos->{$package_name}->{$instpath}->{'DESC'} = 
					$regdata->{$package_name}->{$instpath}->{'Desc'};
			$infos->{$package_name}->{$instpath}->{'REQUIRE'} =
				join(',',@{$regdata->{$package_name}->{$instpath}->{'Require'}});
			
			$infos->{$package_name}->{$instpath}->{'BUILD_STRING'} = 
					$regdata->{$package_name}->{$instpath}->{'BuildString'};
					
			$infos->{$package_name}->{$instpath}->{'MAKE_ID'} = 
					$regdata->{$package_name}->{$instpath}->{'MakeId'};
			
			$infos->{$package_name}->{$instpath}->{'PARENT_PACKAGE'} = 
					$regdata->{$package_name}->{$instpath}->{'ParentPackage'};		
							
		
			if($regdata->{$package_name}->{$instpath}->{'BuildString'} =~ /\S/){
				my $build_info = getBuildStringInfo(
							$regdata->{$package_name}->{$instpath}->{'BuildString'});
				$infos->{$package_name}->{$instpath}->{'PURPOSE'} = 
									$build_info->{'purpose'};
				$infos->{$package_name}->{$instpath}->{'CHANGELIST'} = 
									$build_info->{'changelist'};
				$infos->{$package_name}->{$instpath}->{'QA_LEVEL'} = 
									$build_info->{'qa_level'};
				$infos->{$package_name}->{$instpath}->{'MAKE_TYPE'} = 
									$build_info->{'make_type'};		
			}
		} 
	}
	
	$self->regpackage_index($infos);
	return 1;
}


sub GenInstanceIndex{
	my ($self,$registry,$reset) = @_;
	
	if(defined $self->instance_index and not $reset){
		return 1;
	}
	
	my $kernel_version;
	if(	defined $self->{'package_index'} and
		exists $self->{'package_index'}->{'Database Kernel'}){
		
		$kernel_version = $self->{'package_index'}->
			{'Database Kernel'}->{'SOFTWARE_VERSION'};
	}
	my %instances = getInstances();
	
	my @instances = keys(%instances);
	
	if($#instances == -1){
		$self->last_error('No database found.');
		return undef; 
	}
	

	my @databases;
	
	my @not_ready_databases;
	
	#
	#	get number of db instances per installation path
	#
	
	my $count_instances_per_path;
	
	if(defined $kernel_version){
		foreach my $db_name (@instances){
			my $path = normalizePath($instances{$db_name});
			if(exists $count_instances_per_path->{$path}){
				$count_instances_per_path->{$path}++;
			}
			else{
				$count_instances_per_path->{$path} = 1;
			}
		}
	}
	
	my $index;

	foreach my $db_name (@instances){
		$index->{$db_name};
		my $path = normalizePath($instances{$db_name});
		$index->{$db_name}->{'instroot'} = $path;
		my $db_version;		
		if(	defined $self->regpackage_index and
			exists $self->regpackage_index->{'Database Kernel'} and
			exists $self->regpackage_index->{'Database Kernel'}->{$path}){
		
			$db_version = $self->regpackage_index->{'Database Kernel'}->
							{$path}->{'SOFTWARE_VERSION'};
		
			$index->{$db_name}->{'version'} = $db_version;
		}

		my $instancedata;
		if(defined $registry && $registry->existInstanceData($db_name)){
			my %instancedata = $registry->getInstanceData($db_name);
			$instancedata = \%instancedata;
			$index->{$db_name}->{'registry_instancedata'} = $instancedata;
		}
		
			
		if(defined $kernel_version){
			
			#
			#	assuming we are currenty in installation process
			#	and need information about update capabilities
			#	of databases
			#
			
			$index->{$db_name}->{'ready_to_update'} = 1;
			
			if($count_instances_per_path->{$path} != 1){
				#
				#	there is more than one database
				#	for this software installation    
				#
			
				$index->{$db_name}->{'ready_to_update'} = 0;
				$index->{$db_name}->{'not_ready_reason'} = 
					'there is more than one database per software installation';
				
			}	
			elsif(	defined $kernel_version and 
					defined $db_version and 
					release2num($db_version) > release2num($kernel_version)){
				
				#	
				#	database instance has a newer
				#	software version
				#
			
				$index->{$db_name}->{'ready_to_update'} = 0;
				$index->{$db_name}->{'not_ready_reason'} = 
					'database instance has already a newer software';
			}			
			elsif(defined $instancedata){
							
				#
				#	there is a instance update pending
				#
			
				unless(release2num($instancedata->{'TARGET_RELEASE'}) ==
					   release2num($kernel_version)){
				
					#
					#	instance update has to be resumed
					#	with same kernel version
					#
			
					$index->{$db_name}->{'ready_to_update'} = 0;
					
					$index->{$db_name}->{'not_ready_reason'} = 
						'instance update to version '.
						$instancedata->{'TARGET_RELEASE'}.' is pending';
				}
			}
		}
	}
	
	foreach my $db (keys(%$index)){
		foreach my $param (keys(%{$index->{$db}})){
			$self->log->SetMsg("MSG: DB INDEX: $db: $param = ".
					$index->{$db}->{$param}."\n");
		}
	}
	$self->instance_index($index);
	
	return 1;	
}


sub AddPackage{
	my ($self,$name,$req) = @_;
	my $require;
	my $type;
	my $instkey;
	
		
	#
	#	check if package exists &&
 	#	test given requirments
	#
	
	if(defined $self->package_index and exists $self->package_index->{$name}){
		return {'tree_ref' => $self->package_index->{$name}->{'tree_ref'}}
			 if defined $self->package_index->{$name}->{'tree_ref'};
		$type = 0;
		if($req){
			my %check_struct = (
				'required' => $req,
				'package_name' => $name, 
				'version' => $self->package_index->{$name}->{'SOFTWARE_VERSION'},
				'mode' => $self->package_index->{$name}->{'MODE'} 	
			);
			unless(checkRequired(%check_struct)){
				$self->last_error("package \"$name\" doesn\'t pass the dependency check (".
									$req->{'operator'}." ".$req->{'release'}." ".$req->{'bit'}.")");
				return undef;		
			}
		}
		$require = $self->package_index->{$name}->{'REQUIRE'};
	}
	elsif(defined $self->regpackage_index and
			exists $self->regpackage_index->{$name}){
		foreach my $instpath (keys(%{$self->regpackage_index->{$name}})){
			my %check_struct = (
				'required' => $req,
				'package_name' => $name, 
				'version' => $self->regpackage_index->{$name}->{$instpath}->{'SOFTWARE_VERSION'},
				'mode' => $self->regpackage_index->{$name}->{$instpath}->{'MODE'} 	
			);
			if(checkRequired(%check_struct)){
				$instkey = $instpath;
				last;
			}
		}
		unless(defined $instkey){
			$self->last_error("registered package \"$name\" doesn\'t pass the dependency check (".
									$req->{'operator'}." ".$req->{'release'}." ".$req->{'bit'}.")");
			return undef;		
		}
		return {'tree_ref' => $self->regpackage_index->{$name}->{$instkey}->{'tree_ref'}}
			if defined $self->regpackage_index->{$name}->{$instkey}->{'tree_ref'};
		$type = 1;
		$require = $self->regpackage_index->{$name}->{$instkey}->{'REQUIRE'};
	}
	else{
		$self->last_error("package \"$name\" not found");
		return undef;
	}
	#
	#	if package has no dependencies 
	#	a root package is found
	#

	if($require !~ /\S/){
		# found root
		if(exists $self->package_index->{$name}){
			$self->package_tree({'name' => $name,'type' => $type});	
			$self->package_index->{$name}->{'tree_ref'} = $self->package_tree;  
		}
		else{
			return 1 if defined $self->regpackage_index->{$name}->{$instkey}->{'tree_ref'}; 
			$self->package_tree({'name' => $name,'type' => $type,'instkey' => $instkey});	
			$self->regpackage_index->{$name}->{$instkey}->{'tree_ref'} = $self->package_tree;  
		}
		return {'tree_ref' => $self->package_tree,'state' => 1};
	}
	

	#
	#	try to find needed packages
	#	and add them to the package tree
	#

	my @req = parseRequire($require);
	my $node;
	foreach my $req (@req){
		my $rc = $self->AddPackage($req->{'name'},$req);
		unless(defined $rc){
			$self->last_error("unresolved dependency: ".$self->last_error);	
			return undef;
		}
		if(exists $self->package_index->{$name}
			and exists	$self->package_index->{$name}->{'tree_ref'}){
			$node = $self->package_index->{$name}->{'tree_ref'};	
		}
		else{
			$node = {'name' => $name, 'type' => $type,'instkey' => $instkey};
		}
		
		if($type eq 0){
			$self->package_index->{$name}->{'tree_ref'} = $node;
		}
		else{
			$self->regpackage_index->{$name}->{$instkey}->{'tree_ref'} = $node;
		}

		if(exists $rc->{'tree_ref'}){
			if(defined $rc->{'tree_ref'}->{'children'}){
				$rc->{'tree_ref'}->{'children'} = 
					[$node,@{$rc->{'tree_ref'}->{'children'}}];
	
			}
			else{
				$rc->{'tree_ref'}->{'children'} = 
					[$node];
			}
		}
		else{
		
		}
	}
	return {'state' => 1,'tree_ref' => $node};
}



sub GenPackageTree{
	my ($self,$instreg) = @_;
	unless(defined $self->package_index){
		my $rc = $self->GenPackageIndex();
		unless(defined $rc){
			$self->SetErrMsg("Cannot create package tree: ".$self->last_error);
			return undef;
		}
	}
	if(defined $instreg and not defined $self->regpackage_index){
		my $rc = $self->GenRegPackageIndex($instreg);
	}

	foreach my $package (keys (%{$self->package_index})){
		unless(defined $self->AddPackage($package)){
			$self->SetErrMsg("Cannot create package tree: ".$self->last_error);
			return undef;
		}

	}	

	return 1;
}

sub GenRegPackageTree{
	my ($self,$instreg) = @_;
	if(defined $instreg and not defined $self->regpackage_index){
		my $rc = $self->GenRegPackageIndex($instreg);
	}
	else{
		return undef;
	}
	
	$self->{'regpackage_tree'} = {};
	
	foreach my $package (keys (%{$self->regpackage_index})){
		foreach my $instkey (keys(%{$self->regpackage_index->{$package}})){
			unless(defined $self->AddRegPackage($package,$instkey)){
				$self->SetErrMsg("Cannot create package tree: ".$self->last_error);
				return undef;
			}
		}
	}	
	return 1;
}


sub AddRegPackage{
	my ($self,$name,$instkey) = @_;
	
	my $node;
	
	if(exists $self->regpackage_index->{$name}->{$instkey}->{'tree_ref'}){
		# node already added
		return $self->regpackage_index->{$name}->{$instkey}->{'tree_ref'};		
	}
	else{
		# create a new node
		$node = {'name' => $name,
				'instkey' => $instkey,
				'type' =>1};
		$self->regpackage_index->{$name}->{$instkey}->{'tree_ref'} = $node;
	}
	my $requires = $self->regpackage_index->{$name}->{$instkey}->{'REQUIRE'};

	if($name eq 'Base'){
		#root package
		$self->{'regpackage_tree'} = $node;
	}
	elsif(defined $self->regpackage_index->{$name}->{$instkey}->{'PARENT_PACKAGE'}){
		my $parent_id = $self->regpackage_index->{$name}->{$instkey}->{'PARENT_PACKAGE'};
		my $tree_ref = $self->AddRegPackage(@$parent_id);
		if(defined $tree_ref->{'children'}){
			$tree_ref->{'children'} = [$node,@{$tree_ref->{'children'}}];
		}
		else{
				$tree_ref->{'children'} = [$node];
		}	
	}
	else{
		if($requires !~ /\S/ ){
			$requires = 'Base';
		}
		foreach my $req (parseRequire($requires)){
			if(exists $self->regpackage_index->{$req->{'name'}}){
				my @instkeys = keys(%{$self->regpackage_index->{$req->{'name'}}});
				if($# > 0){
				
					# more than one possible parent
					# -> select one which matches the requirements
					
					
				}
				else{
					my $tree_ref = $self->AddRegPackage($req->{'name'},$instkeys[0]);
					if(defined $tree_ref->{'children'}){
						$tree_ref->{'children'} = [$node,@{$tree_ref->{'children'}}];
					}
					else{
							$tree_ref->{'children'} = [$node];
					}
				}
			}
			else{
				$self->last_error("cannot resolve dependency ".$req->{'name'}.
					' '.$req->{'operator'}.' '.$req->{'version'}.' '.
					$req->{'bit'}." of package $name [$instkey]");
			}	
		}
	}


	return $node;

}	
	
	
sub GetRegPackageList{
	my ($self,$node,$reset_tmp_index) = @_;
	my @list;
	if($reset_tmp_index){
		delete $self->{'tmp_regindex'};
	}
	unless( defined $node){
		$node = $self->regpackage_tree;
		delete $self->{'tmp_regindex'}; 
	}
	foreach my $child (sort {$a->{'name'} cmp $b->{'name'}} @{$node->{'children'}}){
		my @child_list = @{$self->GetRegPackageList($child)};
		if($#child_list > -1){
			push @list,@child_list;
		}
	}
	if($node->{'type'} == 1 and ($node->{'selected'})){
		unless(exists $self->{'tmp_regindex'}->{$node}){
			$self->{'tmp_regindex'}->{$node} = 1;
			push @list,[$node->{'name'},$node->{'instkey'}];
		}
	}
	return \@list;
}	
	

sub GetPackageList{
	my ($self,$node,$reset_tmp_index) = @_;
	my @list;
	if($reset_tmp_index){
		delete $self->{'tmp_index'};
	}
	unless( defined $node){
		$node = $self->package_tree;
		delete $self->{'tmp_index'}; 
	}
	my $child_selected = 0;
	foreach my $child (sort {$a->{'name'} cmp $b->{'name'}} @{$node->{'children'}}){
		my @child_list = @{$self->GetPackageList($child)};
		
		if($#child_list > -1){
			$child_selected = 1;
			unshift @list,@child_list;
		}
	}
	if($node->{'type'} == 0 and ($node->{'selected'} or 
		($child_selected and not $self->{'tree_checker_mode'}))){
		unless(exists $self->{'tmp_index'}->{$node}){
			$self->{'tmp_index'}->{$node} = 1;
			unshift @list,$node->{'name'};
		}
	}
	return \@list;
}

sub GenPackageInfos{
	my ($self,$package_name) = @_;
	my $packagedata = $self->package_index->{$package_name};
	if($packagedata->{'TEST_FILE'} =~ /\S/){
		unless($packagedata->{'MAGIC'}){
			my $pack_buildinfo = getBuildInfoTGZ($packagedata->{'UNTGZ'},$packagedata->{'TEST_FILE'});
			$packagedata->{'MAGIC'} = ${%$pack_buildinfo}{'magic'};
		}
	}
}


sub initDbgPackage{
	my ($self,$package) = @_;
	my $packagedata = $self->package_index->{$package};
	
	return 1;
}

sub GenPackages{
	my ($self) = @_;
	my @packages;
	foreach my $package (@{$self->GetPackageList(undef,1)}){
		#
		#	refresh gui
		#
		if(defined $SAPDB::Install::Values::RefreshGui){
			&{$SAPDB::Install::Values::RefreshGui}();
		}
		my $newpackage = SAPDB::Install::PackageObj->new();
		#$self->GenPackageInfos($package);
		my $packagedata = $self->package_index->{$package};
		
		
		$newpackage->initData($packagedata);
		if(${%$packagedata}{'PACKAGE_NAME'} eq 'APO COM'){
			$SAPDB::Install::Values::APO_release = ${%$packagedata}{'SOFTWARE_VERSION'};
		}	
		push @packages,$newpackage;
	}
	return \@packages;	
}

sub GenRegPackages{
	my ($self,$registry) = @_;
	my @packages;
	foreach my $package (@{$self->GetRegPackageList(undef,1)}){
			push @packages,$registry->getPackage(@$package);
	}
	return \@packages;	
}


sub DESTROY{
	my ($self) = @_;
	$self->SUPER::DESTROY;
}


1;