#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/Prepare.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $
# $Change: 124736 $
#
# Desc: 

package SAPDB::Install::Prepare;

require SDB::Common::Exporter;

our @ISA = qw (SDB::Common::Exporter);
our @EXPORT = qw (checkUpdate collectUpdateInfos checkPackage);

use SAPDB::Install::Tools;
use SAPDB::Install::StdIO;
use SAPDB::Install::Misc;
use SAPDB::Install::System;
use SAPDB::Install::MD5Sum;
use SAPDB::Install::PackageCollector;
use SAPDB::Install::InstallRegistry::Verify;

sub collectUpdateInfos{
	my ($packobj,$package_list,$registry) = @_;
	my %result;
	if($packobj->MainPath->{'value'} =~ /\S/){ 
		return undef;
	}
	
	if($packobj->IsSubPackage){
		my ($req) = SAPDB::Install::Collector::parseRequire(${@{$packobj->Require}}[0]);
		my $parent_found = 0;
		foreach my $pack (@$package_list){
			$packobj->Name eq $pack->Name and last; # no possible parent package found
			if(checkRequired('package_name' => $pack->Name,
							'version' => $pack->Version,
							'mode' => $pack->Mode,
							'required' => $req)){
				# treffer
				$parent_found = 1;
				$packobj->WithParent(1);
				local *subpackobjs = $pack->SubPackObjs;
				push @subpackobjs, \$packobj;
				local *main = $packobj->MainPath;
				$main{'value'} = $pack->MainPath->{'value'};
				defined $packobj->SubDir and $packobj->SubDir =~ /\S/ and $main{'value'} .= '/'.$packobj->SubDir;
				my $parent_path = $pack->MainPath->{'value'};
				normalizePath(\$parent_path);
				$packobj->ParentPackage([$pack->Name,$parent_path]);
				
			}
		}
		unless ($parent_found){
			if(defined $$registry){
				my @possible_parents = ();
				my @impossible_parents = ();
				my %parent_rels = $$registry->getInstallPathes($req->{'name'});
				if(%parent_rels){
					foreach my $package_path (keys(%parent_rels)){
						my %package_data = $$registry->getPackageData($req->{'name'},$package_path);
						if($package_data{'Valid'} and checkRequired('required' => $req,
										 'package_name' => $req->{'name'},
										 'version' => $parent_rels{$package_path},
										 'mode' => $package_data{'Mode'}	
										)){
							push @possible_parents,[$req->{'name'},$package_path,\%package_data];
						}
						else{
							push @impossible_parents, [$req->{'name'},$package_path,\%package_data];
						}
						unless($#impossible_parents == -1 ){	
							$result{'impossible_parents'} = \@impossible_parents;
						}
						unless($#possible_parents == -1 ){
							$parent_found = 1;
							$result{'possible_parents'} = \@possible_parents;
						}
					}	
				}
			}
		}
		unless($parent_found){
			print2stderr("no valid parent package ".$req->{'name'}." $version".($mode =~ /\S/ ? " $mode bit " : ' ')."found\n");
			diesoft($diemsg);
		}
	}
	else{
		my %installations = $$registry->getInstallPathes($packobj->Name);
		my (@toupdate,@nottoupdate);
		foreach my $path (keys(%installations)){
			my %data = $$registry->getPackageData($packobj->Name,$path);
			if(1){
				push @toupdate,[$path,$installations{$path},$data{'Mode'},$data{'Valid'}];
			}
			else{	
				push @nottoupdate,[$path,$installations{$path},$data{'Mode'},$data{'Valid'}];
			}
		} 
		$result{'toupdate'} = \@toupdate;
		$result{'nottoupdate'} = \@nottoupdate;			
	}
	return \%result;
}


sub checkUpdate{
	my ($packobj,$packages,$registry,$ext_args) = @_;

	return 1 if(${$packobj->Skip});
	
	my $opt_g = undef;
	
	if($ext_args->{'opt_g'} =~ /\S/){
		$opt_g = $ext_args->{'opt_g'};
	}

	my $opt_o = undef;	
	
	if($ext_args->{'opt_o'} =~ /\S/){
		$opt_o = $ext_args->{'opt_o'};
	}
	
	my $opt_b = undef;	
	
	if($ext_args->{'opt_b'} =~ /\S/){
		$opt_b = $ext_args->{'opt_b'};
	}


	unless(defined $packobj->RegData){
		$packobj->RegData($$registry->getPackage($packobj->Name,${%{$packobj->MainPath}}{'value'}));
	}
	
	unless($packobj->RegData->Valid){
		$$registry->Log->SetMsg("MSG: installed package is flaged as invalid => skipping update check\n");
		return 1;		
	}
	if(ref ($ext_args->{'callback_initprogress'}) eq 'CODE'){
		&{$ext_args->{'callback_initprogress'}}(${$ext_args->{'callback_dlg'}},0,5,'');
	}
	
	if(defined $ext_args->{'callback_progress'}){
						&{$ext_args->{'callback_progress'}}(${$ext_args->{'callback_dlg'}},1,"Initializing...");
	}
	
	
	if($packobj->RegData->Valid){
			#check if same package already installed
			if(release2num($packobj->Version) == release2num($packobj->RegData->Version) and 
							defined $packobj->RegData->CheckSum){
				unless(defined $packobj->CheckSum){
					$packobj->CheckSum(MD5Sum($packobj->Archive));
				}				
			}				
			
			if(release2num($packobj->Version) == release2num($packobj->RegData->Version) and 
							defined $packobj->CheckSum and defined $packobj->RegData->CheckSum){
				if($packobj->CheckSum eq $packobj->RegData->CheckSum){
					
					if(defined $ext_args->{'callback_progress'}){
						&{$ext_args->{'callback_progress'}}(${$ext_args->{'callback_dlg'}},3,'Same package already installd - testing it now ...');
					}		


					print2stdout("same package already installed\n");
					print2stdout("checking consistence of package data in install registry...\n");
					
					my %result = SAPDB::Install::InstallRegistry::Verify::checkPackage(\$packobj->RegData,1,$ext_args);
					if($result{'status'}){
						$packobj->RegData($packobj->RegData);
						$packobj->{'skip_reason'} = 'same package already installed';
						$packobj->Skip(\1);
						return 1;
					}
					else{
						print2stdout("installed package is corrupted and has to be installed again\n");
					}
				}	
			}
	}	
	
	
	if($packobj->TestFile && $packobj->RegData->Valid){

		if(defined $ext_args->{'callback_progress'}){
						&{$ext_args->{'callback_progress'}}(${$ext_args->{'callback_dlg'}},2,"Testing installed valid package ...");
		}


		#check update
		my $fullpath;
		my $testfile;
		if(exists ${%{$packobj->RegData}}{'TestFile'} and $packobj->RegData->TestFile =~ /\S/){
			$fullpath=$packobj->MainPath->{'value'}.'/'.$packobj->RegData->TestFile;	
			$testfile = $packobj->RegData->TestFile;
		}
		else{
			$fullpath=$packobj->MainPath->{'value'}.'/'.$packobj->TestFile;
			$testfile = $packobj->TestFile;
			unless($^O =~ /mswin/i or -f $fullpath){
				if($packobj->TestFile =~ /^lib\//){
					if($packobj->TestFile !~ /\/lib64\//){
						my $new_file = $packobj->TestFile;
						$new_file =~ s/^lib\//lib\/lib64\//;
						my $new_path = $packobj->MainPath->{'value'}.'/'.$new_file;
						if(-f $new_path){
							$fullpath = $new_path;
							$testfile = $new_file;  
						}	
					}
					else{
						my $new_file = $packobj->TestFile;
						$new_file =~ s/lib64\///;
						my $new_path = $packobj->MainPath->{'value'}.'/'.$new_file;
						if(-f $new_path){
							$fullpath = $new_path;
							$testfile = $new_file;  
						} 
					}
				}
			}
		}
			
		if(-f $fullpath){
			unless(${$packobj->Skip}){
		
				if(defined $ext_args->{'callback_progress'}){
						&{$ext_args->{'callback_progress'}}(${$ext_args->{'callback_dlg'}},4,"Checking version and binary object modes ...");
				}
		
				# 1. version
					
				my $rc = compareReleases($fullpath,$packobj->Version,$packobj->MinVersion);
		
				# min release to update not reached -> fatal error because of possible dependencies
				defined $rc or print2stderr("release file check failed - cannot update\n") and diesoft($SAPDB::Install::Values::diemsg);
		
				# newer package installed -> skip package, dependencies should be solved
				if($rc == 0){
					print2stdout("cannot downgrade package\n");
					if($packobj->AllowSkip){
						#$packobj->{'skip_reason'} = 'installed package doesn\'t reach min version';
						#$packobj->{'skip_detail'} = {'Needed Minimal Version' => $packobj->MinVersion,
						#							 'Installed Version' => defined $packobj->RegData->Version ? $packobj->RegData->Version : '<unkown>',	
						#							 'Version' => $packobj->Version};
						
						$packobj->{'skip_reason'} = 'newer version already installed';
						$packobj->{'skip_detail'} = {'Installed Version' => $packobj->RegData->Version,
												'Version' => $packobj->Version};
						$packobj->Skip(\1);
					}
					else{
						diesoft($SAPDB::Install::Values::diemsg);	
					}
				}
				else{
					print2stdout("update release check... ok\n");
				}			
			
				
				# 2. what string
				if($packobj->MagicMode and !${$packobj->Skip}){
					$packobj->MagicString or print2stderr("no magic to test\n") and diesoft($diemsg);
					my $old_magic;
					unless(compareMagics($packobj->MagicString,${%{$packobj->MainPath}}{'value'},$testfile,$packobj->MagicMode,\$old_magic)){
						print2stderr("magic check failed\n");
						if($packobj->AllowSkip){
							$packobj->{'skip_reason'} = 'installed package has incompatible object mode';
							$packobj->{'skip_detail'} = {'Object Mode of Installed Package' => $old_magic,
													 'Object Mode' => $packobj->MagicString,
													 'Magic Mode' => $packobj->MagicMode,
													 'Installed Version' => $packobj->RegData->Version,	
													 'Version' => $packobj->Version};
							$packobj->Skip(\1);
						}
						else{
							diesoft($SAPDB::Install::Values::diemsg);	
						}
					}
					print2stdout("update binary magic check... ok\n");
				}
			}
		}
		else{
			print2stderr("cannot do update check - test file \"$fullpath\" not found \n");
			diesoft($SAPDB::Install::Values::diemsg);
		}
		
	}
	elsif($packobj->RegData->Valid){
		
		#
		# no testfile defined - do version check with already known values
		#		
		
		if(defined $ext_args->{'callback_progress'}){
						&{$ext_args->{'callback_progress'}}(${$ext_args->{'callback_dlg'}},4,"Checking version and binary mode...");
		}
		
		if(release2num($packobj->MinVersion) <= release2num($packobj->RegData->Version)){
			if(release2num($packobj->Version) < release2num($packobj->RegData->Version)){
				print2stderr("cannot downgrade package\n");
				if($packobj->AllowSkip){
					$packobj->{'skip_reason'} = 'newer version already installed';
					$packobj->{'skip_detail'} = {'Installed Version' => $packobj->RegData->Version,
												'Version' => $packobj->Version};
					$packobj->Skip(\1);
				}
				else{
					diesoft($SAPDB::Install::Values::diemsg);	
				}
			}
			else{
				print2stdout("update release check... ok\n");
			}				
		}
		else{
			print2stderr("needed minimal version (".$packobj->MinVersion.") of installed package (".$packobj->RegData->Version.") not reached\n");
				diesoft($SAPDB::Install::Values::diemsg);	
			}
			#
			# object mode check 
			#
			
			if(not ${$packobj->Skip} and defined $packobj->MagicMode  and defined $packobj->Mode and
			   defined $packobj->RegData->Mode and $packobj->MagicMode !~ /NOBIT/){
				if($packobj->MagicMode =~ /32to64/i){
					if($packobj->Mode == 32 && $packobj->RegData->Mode == 64){
						print2stderr("object mode check... failed\n");
						diesoft($SAPDB::Install::Values::diemsg);
					}
					else{
						print2stdout("object mode check... ok\n");
					}
				}
				elsif($packobj->MagicMode =~ /64to32/i){
					if($packobj->Mode == 64 && $packobj->RegData->Mode == 32){
						print2stderr("object mode check... failed\n");
						diesoft($SAPDB::Install::Values::diemsg);
					}
					else{
						print2stdout("object mode check... ok\n");
					}
				}
				elsif($packobj->MagicMode =~ /equal/i){
					if($packobj->Mode != $packobj->RegData->Mode){
						print2stderr("object mode check... failed\n");
						diesoft($SAPDB::Install::Values::diemsg);
					}
					else{
						print2stdout("object mode check... ok\n");
					}
				}
				else{
					print2stdout("object mode check... not specified\n");
				}			
			}
		}
	
		if(${$packobj->Skip}){ 
			if(defined $ext_args->{'callback_progress'}){
				&{$ext_args->{'callback_progress'}}(${$ext_args->{'callback_dlg'}},5,"Skipping package ...");
			}
			print2stdout("skipping package\n") and return 1;
		}	
						
		# get user / group info
		unless($^O=~/mswin/i || $packobj->IsSubPackage){
			if($packobj->Type eq 'PATCH'){
				unless($^O=~/mswin/i){
					$packobj->User($packobj->RegData->User);
					my $uid = (getpwnam($packobj->RegData->User))[2];
					$uid or print2stderr("cannot get uid of user \"$packobj->RegData->User\"\n") and diesoft($SAPDB::Install::Values::diemsg);
					$packobj->UID($uid);
					$packobj->Group($packobj->RegData->Group);
					my $gid = (getgrnam($packobj->RegData->Group))[2];
					$gid or print2stderr("cannot get gid of group \"$packobj->RegData->Group\"\n") and diesoft($SAPDB::Install::Values::diemsg);
					$packobj->GID($gid);
				}
			}
			else{
				# reset user and group while updating from < 7.5 to >= 7.5
				if ($packobj->Name eq 'Base' &&
 					release2num($packobj->Version) >= release2num('7.5.0.0') &&
  					release2num($packobj->RegData->Version) < release2num('7.5.0.0')) {
  						$packobj->RegData->Group('');
  						$packobj->RegData->User('');
  					}

				# if no user/group info in registry set package default
				$packobj->RegData->Group =~ /\S/ or $packobj->RegData->Group($packobj->Group); 
				$packobj->RegData->User =~ /\S/ or $packobj->RegData->User($packobj->User);
				unless($packobj->IsSubPackage){
					checkGroup($packobj->RegData->Group,$packobj,$opt_g,$opt_b || $opt_batch);
					checkOwner($packobj->RegData->User,$packobj,$opt_o,$opt_b || $opt_batch);
				}
				# ask only once for owner/group - for first package (now all packages get same owner/group)
				unless($opt_diff_owner_group){
					$opt_g=$packobj->Group;
					$opt_o=$packobj->User;
				}
			}
							
		}
		foreach my $hrPath (@{$packobj->Paths}){
			local *hPath=$hrPath;
			my $opt=$hPath{'opt'};
               $opt=~s/=.*$//g;
			if($hPath{'value'}){
				if($opt){
					if(expandPath(${"opt_$opt"}) ne $hPath{'value'} and ${"opt_$opt"} =~ /\S/){
						$packobj->RegData->Registry->Log->SetMsg("WRN: ignore commandline option \"$opt\" for \"".$hPath{'name'}.'" = "'.${"opt_$opt"}."\" - not changeable\n");
					}
				}
				next;
			}
		}

	if(defined $ext_args->{'callback_progress'}){
				&{$ext_args->{'callback_progress'}}(${$ext_args->{'callback_dlg'}},5,"Finishing update check ...");
	}
	return 1;
}


sub testExtract{
	my ($rpackobj,$ext_args) = @_;


	local *packobj = $rpackobj; 

	# create directories
	my @paths = @{$packobj->Paths};	
	
	foreach my $pathref (@paths){
		my %path=%$pathref;
		-d $path{'value'} && next;
		my $path_value = $path{'value'};
		if ($^O =~ /mswin/i and exists $path{'realname'}){
			$path_value = $path{'realname'}; 			
		}
		makedir($path_value,($path{'mode'} > 0 and $path{'mode'} < 0777) ? $path{'mode'} : 0555, $packobj->UID,$packobj->GID) or print2stderr("cannot create directory $path{'name'} \"$path{'value'}\"") and diesoft($SAPDB::Install::Values::diemsg);
	}
	
	if($^O=~/.*aix.*/i){
		#on aix shared libs may stay in memory after ending related processes
		#slibclean remove all unused shared objects 
		sub ignore_all_errors{
			my ($text)=@_;
			$packobj->RegData->Registry->Log->SetMsg("MSG: SYS: slibclean: $text\n");
			return 1;
		}
		callsystem("slibclean",\&ignore_all_errors,1);
	}
	
	#tgz extraction testrun
	(my $fsinfos, my $mapped_files, my $busy_files ) = unpackTGZ($packobj,$packobj->MainPath->{'value'},1,[keys(%{$packobj->FileList})],$ext_args);
   		
	if($#{@$busy_files} > -1){
		my $ps = SAPDB::Install::ProcState->new();
		my %processes;
		my $found_one = 0;
		foreach my $file (@$busy_files){
			my $pids = $ps->WhoUsesModule($file);
			if(defined $pids){
				foreach my $pid (@$pids){
					foreach my $proc (@{$ps->{'procs'}}){
						if($proc->{'pid'} == $pid){
							$found_one = 1;
							$processes{$pid} = $proc->{'cmd'}; 	
						}
					}
				}
			}
		}
		my $details;
		my $process_names = {};
		my $info = {};
		if($found_one){
 			print2stdout("following programs are running: \n");
			$details = "Following programs are running:\n";
 			foreach my $pid (keys(%processes)){
 				$process_names->{$processes{$pid}} = 1;
 				print2stdout($processes{$pid}." [pid = $pid]\n");		
 				$details .= $processes{$pid}." [pid = $pid]\n";
			}
			$details .= "\nPlease stop all these processes!";
		 	print2stderr("please stop all!\n");
 		}
 		else{
			$details = "Please stop all running \nSAP DB / MaxDB Software and resume!\n";
 			print2stderr("test extraction failed\n");
 		}
 		$info->{'process_names'}  = [keys(%$process_names)];
 		return (0,{'errmsg' => 'Test extraction failed!','info' => $info, 'errdetails' => $details });			
			
	}
	
	if($^O =~ /mswin/i and defined $mapped_files and $#{@$mapped_files} > -1){
		$packobj->MappedFiles($mapped_files);
	}
	unless(defined $packobj->CheckSum){
		$packobj->CheckSum(MD5Sum($packobj->Archive));
	}
	return (1,$fsinfos);
}




sub checkPackage{
	my ($packobj,$ext_args) = @_;
	my %result;
	my @new_files = ();
	my @remove_later; 
	
	if($packobj->Update and defined $packobj->RegData->FileList){
		if($^O =~ /mswin/i){
			#
			# normalize file names in case of win os
			#
	
			my @norm_new_list;
			my @norm_old_list;
	
			my @dbg_files = ();
					
			if (defined $packobj->DbgFiles){
				@dbg_files = keys (%{$packobj->DbgFiles});
			}
			
			foreach my $file (keys(%{$packobj->FileList}),@dbg_files){
				push @norm_new_list,normalizePath($file);
			}
			foreach my $file (keys(%{$packobj->RegData->FileList})){
				push @norm_old_list,normalizePath($file);
			}
			@new_files = minus(\@norm_new_list,\@norm_old_list);		
			unless($packobj->Type eq 'PATCH'){
				@remove_later = minus(\@norm_old_list,\@norm_new_list);
			}
		}
		else{
			my @files = keys(%{$packobj->FileList});
			if (defined $packobj->DbgFiles){
				push @files, keys (%{$packobj->DbgFiles});
			}	
			@new_files = minus(\@files,[keys(%{$packobj->RegData->FileList})]);
			unless($packobj->Type eq 'PATCH'){
				@remove_later = minus([keys(%{$packobj->RegData->FileList})],\@files);
			}
		}
	}
	else{
		@new_files = keys(%{$packobj->FileList});
		if (defined $packobj->DbgFiles){
			push @new_files, keys (%{$packobj->DbgFiles});
		}
	}
	
	my %check_result = $packobj->RegData->Registry->CheckFilesOfNewPackage($packobj->Name,${%{$packobj->MainPath}}{'value'},\@new_files,
															{'callback_initprogress' => $ext_args->{'callback_initprogress'},
															'callback_progress' => $ext_args->{'callback_progress'}}	);	
	if($check_result{'state'} == 0){
		my $details;
		foreach my $file (keys(%{$check_result{'conflict_files'}})){
			$details .= "$file is used by package ".$check_result{'conflict_files'}->{$file}->[0]." in ".
						$check_result{'conflict_files'}->{$file}->[1]."\n";
		}
		return (0,{'errmsg' => 'Files are used by other packages!',
				'errdetails' => $details,'package_file_conflict' => 1});		
	}

	my ($rc,$details) = testExtract(\$packobj,$ext_args);
	
	if($rc == 0){
		$details->{'test_extract_failed'} = 1;
		return($rc,$details);
	}
	if($#remove_later > -1){
		$packobj->RegData->Registry->RemoveFromAllFiles($packobj->MainPath->{'value'},\@remove_later);
	}
	return (1,{'fs_infos' => $details});
}




1;