package SAPDB::Install::Templates::InstTemplate;
use strict;
use SAPDB::Install::Templates::Properties;
use SAPDB::Install::Templates::Util;
use SAPDB::Install::Registry;
use SAPDB::Install::System;


require Exporter;
our @ISA = qw( Exporter SAPDB::Install::Templates::Properties);

sub new{
  my $id = shift;
  my $this = { 'KEYS' =>{
                         'packages'  =>
                         {
                          'VALIDATOR' => 0,
                          'PARSER' => \&packageParser,
                          'FORMATTER' =>\&packageFormatter 
                         },
                         'removeList'  =>
                         {
                          'VALIDATOR' => 0,
                          'PARSER' => \&removeListParser,
                          'FORMATTER' =>\&removeListFormatter 
                         },
                         'dBAUser'  =>
                         {
                          'DEFAULT' => 'DBA,DBA',
                          'INITIAL' => 'DBA,DBA'
                         },
                         'dBMUser'  =>
                         {
                          'DEFAULT' => 'DBM,DBM',
                          'INITIAL' => 'DBM,DBM'
                         },
                         'lCAUser'  =>
                         {
                          'DEFAULT' => 'lcaadm,sap'
                         },
                         'lCAEncoding'  =>
                         {
                          'DEFAULT' => 'ascii'
                         },
                         'archiveDir'  => { 'DEFAULT' => '.' ,
                                            'TYPE' => 'PATH'
                                          },
                         'indepData'  => {
                                          'DEFAULT' =>  '${applicationDataPath}\sdb\data',
                                          'TYPE' =>  'PATH'
                                         },
                         'indepProg'  => {
                                          'DEFAULT' =>  '${systemProgPath}\sdb\programs',
                                          'TYPE' =>  'PATH'
                                         },
                         'dependendPath'  => {
                                          'DEFAULT' =>  '${systemProgPath}\sdb\${dBName}',
                                          'TYPE' =>  'PATH'
                                         },
                         'registryFile'  => {
                                             'DEFAULT' =>  '${indepData}\config\install\INSTREG',
                                             'TYPE' =>  'PATH'
                                            },
                         'ownerName'  => {
                                          'DEFAULT' =>  'sdb'
                                         },
                         'ownerGroup'  => {
                                           'DEFAULT' =>  'sdba'
                                          },
                         'systemProgPath'  => {
                                             'TYPE' =>  'PATH'
                                           },
                         'applicationDataPath'  => {
                                             'TYPE' =>  'PATH'
                                           },
                         'dBName'  => {
                                       'DEFAULT' =>  'MAXDB1'
                                      },
                         'dBParams'  => {
                                         'FORMATTER' => \&dbParamsFormatter
                                        } ,
                         'dBVolumes'  => {
                                          'FORMATTER' => \&dbVolumesFormatter
                                         } ,
                         'instance'  => {
                                        },
                         'upgradeFile'  => {
                                            'TYPE' =>  'PATH',
                                            'DEFAULT' =>  'upgrade_${dbname}.config',
                                            'INITIAL' =>  'upgrade_${dbname}.config'
                                           },
                         'logFile'  => {
                                        'TYPE' =>  'PATH'
                                       },
                         'templateID'  => { }
                        }};
  bless $this ;
  $this->SUPER::initData();
  $this->setTemplateID($id);
  return  $this;
}

my $xmlDataRaw = << "XMLDATA";
<Template id="templ_desktop">
 <FrameList >
   <Frame id="DBNamePasswdDesktop"/>
 </FrameList>
</Template>
<Template id="templ_server_advanced">
</Template>
XMLDATA

my @xmlData = split(/\n/,$xmlDataRaw);


sub nextFrame{
  my ($this,$frameid) = @_;
  my $retVal = 
  my $templateId = $this->getTemplateID();
  my @red = filterXMLTag(\@xmlData,'Template', "id=\"$templateId\"");

  my @list ;
  $retVal = parseXML(\@red,$this,
           { 'beginFrame'=>
             sub {
               my  ($this,$atts,$stack) = @_;
             }
           });
  $retVal and print $retVal;
}


sub createObject{
  my ($file) =@_;
  my $obj = new();
  my $retVal = 0;
  $file and
    $retVal = $obj->parseFile($file)
      and return ("Configuration file '$file' could not be read:\n".$retVal,undef);
  my ($aindep_data,$aindep_prog)=readIndepPath(); 

  my $sysProgPath=getSysProgPath(); 
  my $appDataPath=getAppDataPath(); 
  $obj->getSystemProgPath() or $obj->setSystemProgPath($sysProgPath);
  $obj->getApplicationDataPath() or $obj->setApplicationDataPath($appDataPath);

  
  if (defined $aindep_data and not $aindep_data =~ /^\s*$/) {
    $obj->setIndepData($aindep_data);
  }
  if (defined $aindep_prog and not $aindep_prog =~ /^\s*$/) {
    $obj->setIndepProg($aindep_prog);
  }
  return ($retVal,$obj);
}

sub packageParser{
  my ($this,$key,$value ) = @_;
  my @packages = split/,/,$value;
  return \@packages;
}

sub packageFormatter{
  my ($this,$key,$value ) = @_;
  my $packages = join ',',@$value;
  return (uc $key) . " " . $packages . "\n";
}

sub removeListParser{
  my ($this,$key,$value ) = @_;
  my @retVal;
  if ($value) {
    my @packages = split(/,/,$value);
    foreach (@packages) {
      if (/^(.*)\s*\[(.*)\]$/) {
        my $name = $1;
        my $path = $2;
        $name =~ s/\s+$//;
        $name =~ s/^\s+//;
        $path =~ s/\s+$//;
        $path =~ s/^\s+//;
        push @retVal,([$name,$path]);
      }
    }
  }
  return \@retVal;
}





sub getRemoveList{
  my $this = shift;
  my $l = $this->Get('removeList');
  my @outlist;
  foreach(@$l){
    my $name = $_->[0];
    my $path = $_->[1];
    $path = $this->recursiveSubstitute($path);
    push @outlist, ([$name,$path]);
  }
  return \@outlist;
}

sub removeListFormatter{
  my ($this,$key,$value ) = @_;
  my $retVal = '';
  foreach(@$value){
    $retVal .= ',' . $_->[0] . '[' . $_->[1] . ']';
  }
  $retVal =~ s/^,//;
  $retVal = (uc $key) . " " . $retVal. "\n";
#  print  "retVal: $retVal\n";
  return $retVal;
}


sub dbParamsFormatter{
  my ($this,$key,$value ) = @_;
  my $retVal = '';
  if($value){
    foreach(keys %$value){
      $retVal .=  'DBPARAM ' . $_ . ' ' . $value->{$_}  . "\n";
    }
  }
  return  $retVal;
}

sub dbVolumesFormatter{
  my ($this,$key,$value ) = @_;
  my $retVal = '';
  if($value){
    foreach(@$value){
      $retVal .=  'DBVOLUME '. $_ . "\n";
    }
  }
  return  $retVal;
}

sub getDBVolumes{
  my $this = shift;
  my $l = $this->Get('dBVolumes');
  my @outlist;
  foreach(@$l){
    my $val = $this->recursiveSubstitute($_);
    push @outlist, ($val);
  }
  return \@outlist;
}
sub parseLine{
  my ($this,$aLine) = @_;
  if ( $aLine =~ /^\s*DBPARAM\s+(.*)$/) {
    my $rest = $1;
    if ( $rest =~ m/^\s*([\w\d]+)\s+(.+)\s*$/) {
      my $val = $2;
      my $key = $1;
      my $dbps = $this->getDBParams();
      $dbps = $dbps ?  $dbps : {};
      $dbps->{$key}=$val;
      $this->setDBParams($dbps);
    }
  } elsif ( $aLine =~ /^\s*DBVOLUME\s+(.*)$/) {
    my $vol = $1;
    my $vols = $this->Get('dBVolumes');
    $vols = $vols ? $vols : [];
    push @$vols,($vol);
    $this->setDBVolumes($vols);
  } else {
    $this->SUPER::parseLine($aLine);
  }
}

package SAPDB::Install::Templates::TemplateList;
use SAPDB::Install::Templates::Util;

my ($templateList,$templateSection,$template,$description);
sub beginTemplateList{
  my  ($atts,$stack) = @_;
  $templateList = [];
}
sub endTemplateList{
  my  ($name,$stack) = @_;
}
sub beginTemplateSection{
  my  ($atts,$stack) = @_;
  $templateSection = {'label' => $atts->{'label'},'data'=>[]};
}
sub endTemplateSection{
  push @$templateList,($templateSection);
  undef $templateSection;
}
sub beginTemplate{
  my  ($atts,$stack) = @_;
  $template = {'TemplateCode'=>$atts->{'id'},
               'ShortDescription'=>$atts->{'label'}
              };
}
sub endTemplate{
  push @{$templateSection->{'data'}},($template);
  undef $template;
}
sub beginDescription{
  $description='';
}
sub endDescription{
  $template->{'LongDescription'}=$description;
  undef $description;
}
sub cdata{
  my ($line,$stack) = @_;
  defined $description or return;
  chomp $line;
  $line =~ s/^\s+//;
  $line =~ s/\s+$/ /;
  $description .= $line;
}



my $listData = << "LISTDATA";
<TemplateList>
  <TemplateSection label="General">
    <Template id="templ_desktop" label="Desktop PC / Laptop" >
      <Description>
       Typical installation for a single user environment. 
       Can be used during development of simple database applications, for 
       the MaxDB tutorial or for testing purposes.
      </Description>
    </Template>
  </TemplateSection>
  <TemplateSection label="Custom">
    <Template id="templ_server_advanced" label="Custom" >
      <Description>
       Customizable installation of a databases instance. 
       Default values for memory usage, cpu usage and other parameters are calculated and can be adapted.
       Any number of data volumes on multiple hard disks can be specified. 
      </Description>
    </Template>
  </TemplateSection>
</TemplateList>
LISTDATA


sub createTemplateList{
  my @lines = split(/\n/,$listData);
  my $retVal = parseXML(\@lines);
  bless $templateList;
  return ($retVal,$templateList);
}


sub descById{
  my ($this,$id) = @_;
  foreach(@$this){
    my $ts = $_->{'data'};
    foreach (@$ts) {
      $_->{'TemplateCode'} eq $id and return $_;
    }
  }
}




sub testit{
  my @aData = <DATA>;
  #my @red = filterXMLTag(\@aData,'Template', "id=\"templ_desktop\"");
  #foreach(@red){
  #  print;
  #}
  my ($retVal,$aList) = createTemplateList();
  $retVal and return $retVal; 
  my $template = SAPDB::Install::Templates::InstTemplate::new('templ_desktop');
  $template->nextFrame();
  return $retVal;

#  dumpList($aList);
}
sub dumpList{
  my $aList = shift;
  foreach ( @$aList) {
    my $sec = $_->{'label'};
    my $ts = $_->{'data'};
    print "Section '$sec':\n";
    foreach (@$ts) {
      print join ("\n ",values(%$_));
    }
    print "\n\n";
  }
}
#my $retVal = testit();
#$retVal and print $retVal;


1;
__DATA__
<TemplateList>
  <TemplateSection label="General">
    <Template id="templ_desktop" label="Desktop PC / Laptop" >
      <Description>
       Typical installation for a single user environment. 
       Can be used during development of simple database applications, for 
       the MaxDB tutorial or for testing purposes.
      </Description>
    </Template>
  </TemplateSection>
  <TemplateSection label="Custom">
    <Template id="templ_server_advanced" label="Custom" >
      <Description>
       Customizable installation of a databases instance. 
       Default values for memory usage, cpu usage and other parameters are calculated and can be adapted.
       Any number of data volumes on multiple hard disks can be specified. 
      </Description>
    </Template>
  </TemplateSection>
</TemplateList>



# Local Variables: ***
# perl-dbg-flags:"-w " ***
# sdbinst-defines: "-define SCRIPTING=YES" ***
# sdbinst-testfile: "InstTemplate.pmp" ***
# sdbinst-script-directory:  "sys/wrk/script/SAPDB/Install/Templates" ***
# End: ***

