#!/usr/bin/perl

package SAPDB::Install::Templates::OpensourceOltpWorkstation;

#ifdef SCRIPTING
#use strict;
#use SAPDB::Install::Templates::Template;
#use SAPDB::Install::Templates::Simple;
#use SAPDB::Install::Templates::Util;
#use SAPDB::Install::Registry;
#use SAPDB::Install::MachInfo;
#use SAPDB::Install::System::Win32::Registry ;
#require Exporter;
#our @ISA = qw( Exporter  SAPDB::Install::Templates::Template );
#our @EXPORT = qw( );
#endif

#because SDBINST
our @ISA = qw (SDB::Common::Exporter SAPDB::Install::Templates::Template);
use SAPDB::Install::Templates::Template;
use SAPDB::Install::Templates::Simple;
use SAPDB::Install::Templates::Util;
use SAPDB::Install::Registry;
use SAPDB::Install::MachInfo;
use SAPDB::Install::Registry;
use SAPDB::Install::System::Win32::Registry ;
#endbecause

#ifdef doc
#<h1>SAPDB::Install::Templates::OpensourceOltpWorkstation</h1>
#  <p>
#  Open Source Oltp Workstation Installation
#  </p>
  #endif
  sub new(){
    my $this =    
      {
      };
    bless $this;

    $this->setDefaults();
    $this->setTemplateCode("templ_desktop");


    $this->addAfterAdvice("SWDriveKey",sub{
                             my ($this) = @_;
                             my $newSWDrive = $this->getSWDrive();
                             if(defined $newSWDrive){
                               my $newSoftwarePath = $newSWDrive . ":\\Program Files\\sdb";
                               my $aPathStrategy = $this->getPathStrategy();
                               if(  "SAP" eq  $aPathStrategy) { 
                                 $newSoftwarePath = $newSWDrive . ":\\sdb";
                               }
                               $this->setAdvicedSoftwarePath($newSoftwarePath);
                             }
                           }
                          );
    $this->addAfterAdvice("ServerDriveKey",sub{
                             my ($this) = @_;
                             my $newServerDrive = $this->getServerDrive();
                             my $newServerPath = $newServerDrive . ":\\sdb";
                             $this->setAdvicedServerPath($newServerPath);
                           }
                          );

    $this->addAfterAdvice("SoftwarePathKey",sub{
                             my ($this) = @_;
                             my $newSoftwarePath = $this->getSoftwarePath();
                             my $newIndepProg = catfile($newSoftwarePath,"programs");
                             $this->setIndependendPrograms($newIndepProg);
                             $this->configureDBInstanceFromSoftwarePath();
                           }
                          );
    $this->addAfterAdvice("ServerPathKey",sub{
                             my ($this) = @_;
                             my $newServerPath = $this->getServerPath();
                             my $newIndepData = catfile($newServerPath,"data");
                             $this->setIndependendData($newIndepData);
                             $this->configureDBInstanceFromServerPath();
                           }
                          );

    $this->addAfterAdvice("PathStrategyKey",sub{
                             my ($this) = @_;
                             $this->preConfigurePaths();
                           }
                          );

    $this->setAdvicedPathStrategy("OS");



    return $this;
  }
#new



my $regexpIndepProg = "^(.*[\\\\\\/]s(ap)?db)[\\\\\\/]programs[\\\\\\/]?\$";
my $regexpIndepData = "^(.*[\\\\\\/]s(ap)?db)[\\\\\\/]data[\\\\\\/]?\$";


sub preConfigurePaths{
  my ($this) = @_;
  my $aPathStrategy = $this->getPathStrategy();
  if ( $^O =~ /mswin/i) {
    if ( "SAP" eq  $aPathStrategy) { 
      $this->preConfigureWinSAP();
    } elsif ("OS" eq $aPathStrategy) {
      $this->preConfigureWinOS();
    }
  } else {
    if ( "SAP" eq  $aPathStrategy) { 
      $this->preConfigureUxSAP();
    } elsif ("OS" eq $aPathStrategy) {
      $this->preConfigureUxOS();
    }
  }
}


sub preConfigureUxOS{
  my ($this) = @_;
  $this->setAdvicedSoftwarePath("/opt/sdb");
  $this->setAdvicedServerPath("/var/opt/sdb");
  my $aHome = $ENV{"HOME"};
  $this->setAdvicedAppDataRoot(catfile($aHome,"sdb"));
}
#preConfigureUxOS

sub preConfigureUxSAP{
  my ($this) = @_;
  my ($indep_data, $indep_prog ) = readIndepPath();
  if( defined $indep_prog and $indep_prog =~ /$regexpIndepProg/i){
    $this->setAdvicedSoftwarePath($1  );
  }
  else{
    $this->setAdvicedSoftwarePath("/sapdb");
  }
  if( defined $indep_data and $indep_data =~ /$regexpIndepData/i){
    $this->setAdvicedServerPath($1);
  }
  else{
    $this->setAdvicedServerPath("/sapdb");
  }
  $this->setAdvicedAppDataRoot("/sapdb");
}
#preConfigureUxSAP


 sub myReadReg{
#because SDBINST
   return readValues(@_);
#endbecause
   #ifdef SCRIPTING
#   return readRegistryValues(@_);
   #endif
}
sub preConfigureWinOS{
  my ($this) = @_;
  my $progKey = "SOFTWARE\\Microsoft\\Windows\\CurrentVersion"; 
  my $commonAppDataKey = "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders";
  my $addDataKey = "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders" ;
  my %hash;

  my ($indep_data, $indep_prog ) = readIndepPath();
  if( defined $indep_prog and $indep_prog =~ /$regexpIndepProg/i){
    $this->setAdvicedSoftwarePath($1  );
  }
  else{
    %hash=myReadReg(HKEY_LOCAL_MACHINE,$progKey);
    my $progs = $hash{'ProgramFilesDir'};
    if (defined $progs) {
      $this->setAdvicedSoftwarePath(catfile($progs,"sdb"));
    }
    else{
      $this->setAdvicedSoftwarePath("c:\\Program Files\\sdb");
    }
  }

  if( defined $indep_data and $indep_data =~ /$regexpIndepData/i  ){
    $this->setAdvicedServerPath($1);
  }
  else{
    %hash=myReadReg(HKEY_LOCAL_MACHINE,$commonAppDataKey);
    my $commonApps = $hash{'Common AppData'};
    if (defined $commonApps) {
      $this->setAdvicedServerPath(catfile($commonApps,"sdb"));
    }
    else{
      $this->setAdvicedServerPath("C:\\Documents and Settings\\All Users\\Application Data\\sdb");
    }
  }

  %hash=myReadReg(HKEY_CURRENT_USER,$addDataKey);
  my $apps = $hash{'AppData'};
  if (defined $apps) {
    $this->setAdvicedAppDataRoot($apps);
  }
}
#preConfigureWinOS




sub preConfigureWinSAP{
  my ($this) = @_;
  my ($indep_data, $indep_prog ) = readIndepPath();
  if( defined $indep_prog and $indep_prog =~ /$regexpIndepProg/i){
    $this->setSoftwarePath($1);
  }
  else{
    $this->setAdvicedSWDrive("c");
  }
  if( defined $indep_data and $indep_data =~ /$regexpIndepData/i ){
    $this->setAdvicedServerPath($1);
  }
  else{
    $this->setAdvicedServerDrive("c");
  }
  $this->setAdvicedAppDataRoot("c:\\sdb\\ApplicationData");
}


#+sub():void:Rekonfiguriert nach jedem Neusetzen von Parametern
sub configure{
  my ($this) = @_;
  my @configResult = (1);
  @configResult = $this->SUPER::configure();
  $this->configureDBInstanceFromSoftwarePath();
  $this->configureDBInstanceFromServerPath();
  return @configResult;
}


#+sub():void:Initialisiert nach dem Lesen der Konfigurationsdatei
sub initialize{
  my ($this) = @_;
  $this->SUPER::initialize();
  my $instance = $this->getDBInstance();
  if(defined $instance){
    $instance->addAfterAdvice
      ("sdb_name",sub{
         my ($aInstance) = @_;
         my $dbname = $aInstance->getDBName();
         my $aSoftwarePath = $this->getSoftwarePath();
         my $dependPath = catfile($aSoftwarePath,$dbname); 
         $aInstance->setDependendPath($dependPath);
         my $aServerPath = $this->getServerPath();
         if(defined $aServerPath ){
           $aInstance->setDataPathPrefix($aServerPath);
           $aInstance->setLogPathPrefix($aServerPath);
         }
       }
      );
    #Initialize depend path if not set
     my $dbname = $instance->getDBName();
     my $aSoftwarePath = $this->getSoftwarePath();
     my $dependPath = catfile($aSoftwarePath,$dbname); 
     my $aDepPath = $instance->getDependendPath();
     unless(defined $aDepPath){
       $instance->setDependendPath($dependPath);
     }
  }
  $this->initDBInstanceFromSoftwarePath();
  $this->initDBInstanceFromServerPath();
}




sub initDBInstanceFromSoftwarePath{
  my ($this) = @_;
   my $instance = $this->getDBInstance();
   if(defined $instance){
     my $aDepPath = $instance->getDependendPath();
     unless($aDepPath){
       my $dbname = $instance->getDBName();
       my $aSoftwarePath = $this->getSoftwarePath();
       my $dependPath = catfile($aSoftwarePath,$dbname); 
       $instance->setDependendPath($dependPath);
     }
   }
}



sub configureDBInstanceFromSoftwarePath{
  my ($this) = @_;
   my $instance = $this->getDBInstance();
   if(defined $instance){
     my $dbname = $instance->getDBName();
     my $aSoftwarePath = $this->getSoftwarePath();
     my $dependPath = catfile($aSoftwarePath,$dbname); 
     my $aDepPath = $instance->getDependendPath();
     $instance->setDependendPath($dependPath);
   }
}

sub configureDBInstanceFromServerPath{
  my ($this) = @_;
   my $instance = $this->getDBInstance();
   if(defined $instance){
     my $dbname = $instance->getDBName();
     my $aServerPath = $this->getServerPath();
     if(defined $aServerPath ){
#       $instance->setDataPathPrefix($aServerPath);
#       $instance->setLogPathPrefix($aServerPath);
     }
   }
}#configureDBInstanceFromServerPath

sub initDBInstanceFromServerPath{
  my ($this) = @_;
#  print "Initializing \n";
#  my ($package,$filename,$line) = caller(0);
#  print   "$filename, line $line\n";
#  ($package,$filename,$line) = caller(1);
#  print   "$filename, line $line\n";
   my $instance = $this->getDBInstance();
   if(defined $instance){
     my $dbname = $instance->getDBName();
     my $aServerPath = $this->getServerPath();
     if(defined $aServerPath ){
       my $dpp = $instance->getDataPathPrefix();
       unless($dpp){
         $instance->setDataPathPrefixInit($aServerPath);
       }
       my $lpp = $instance->getLogPathPrefix();
       unless($lpp){
         $instance->setLogPathPrefixInit($aServerPath);
       }
     }
   }
}#initDBInstanceFromServerPath


sub setDBInstance{
  my ($this, $instance) = @_;
  $this->SUPER::setDBInstance($instance);
  $this->initialize();
}




sub nextFrameDesktop{
  my ($this,$previousFrame) = @_;
  my $nextFrame=undef;
  if (defined $previousFrame) {
    my $frameId = $previousFrame->{"FrameId"};
    if($frameId eq "DBNamePasswdDesktop" ){
      if ( $^O =~ /mswin/i) {
        $nextFrame = $this->createFrameDescription("VolumesMinimal");
      }
      else{
        $nextFrame = $this->createFrameDescription("VolumesMinimalUx");
      }
    }
  } else {
    unless($this->getDBInstance()){
      my $instance = SAPDB::Install::Templates::Simple::new();
      $this->setDBInstance($instance);
      $this->setIsCreateInstance(1);
    }
    $nextFrame = $this->createFrameDescription("DBNamePasswdDesktop");
  }
  #else
  return $nextFrame;
}#nextFrameDesktop




sub nextFrameServerSmall{
  my ($this,$previousFrame) = @_;
  my $nextFrame=undef;
  if (defined $previousFrame) {
    my $frameId = $previousFrame->{"FrameId"};
    if($frameId eq "DBNamePasswdSimple" ){
      if ( $^O =~ /mswin/i) {
        $nextFrame = $this->createFrameDescription("VolumesMedium");
      }
      else{
        $nextFrame = $this->createFrameDescription("VolumesMediumUx");
      }
    }
  } else {
    unless($this->getDBInstance()){
      my $instance = SAPDB::Install::Templates::Simple::new();
      $this->setDBInstance($instance);
      $this->setIsCreateInstance(1);
    }
    $nextFrame = $this->createFrameDescription("DBNamePasswdSimple");
  }
  #else
  return $nextFrame;
}





sub nextFrameServerMedium{
  my ($this,$previousFrame) = @_;
  return $this->nextFrameServerSmall($previousFrame);
}
sub nextFrameServerAdvanced{
  my ($this,$previousFrame) = @_;
  my $nextFrame=undef;
  if (not defined $previousFrame) {
    my $instance = $this->getDBInstance();
    unless(defined $instance){
      $instance = SAPDB::Install::Templates::Simple::new();
      $this->setDBInstance($instance);
      $this->setIsCreateInstance(1);
    }
    $nextFrame = $this->createFrameDescription("DBNamePasswdSimple");
  } 
  else {
    my $frameId = $previousFrame->{"FrameId"};
    if($frameId eq "DBNamePasswdSimple" ){
      $nextFrame = $this->createFrameDescription("VolumesAdvanced");
    }
    elsif($frameId eq "VolumesAdvanced" ){
      $nextFrame = $this->createFrameDescription("SizeMemoryAdvanced");

    }
    elsif($frameId =~ "SizeMemoryAdvanced" ){
      if( $^O =~ /mswin/i){
        $nextFrame = $this->createFrameDescription("GlobalDBSettings");
      }
      else{
        $nextFrame = $this->createFrameDescription("GlobalDBSettingsUx");
      }
    }
  }
  #else
  return $nextFrame;
}#nextFrameDesktop




#+sub(Frame:previousFrame):Frame:Liefert die anzuzeigenden Frames in der richtigen Reihenfolge
sub nextFrameDefault{
  my ($this,$previousFrame) = @_;
  my $nextFrame=undef;
  if (defined $previousFrame) {
    my $frameId = $previousFrame->{"FrameId"};
  } else {
  }
  #else
  return $nextFrame;
}


sub nextFrame{
  my ($this) = @_;
  my $templateCode = $this->getTemplateCode();
  if( "templ_desktop" eq $templateCode){
    return nextFrameDesktop(@_);
  }
  elsif( "templ_server_small" eq $templateCode){
    return nextFrameServerSmall(@_);
  }
  elsif( "templ_server_medium" eq $templateCode){
    return nextFrameServerMedium(@_);
  }
  elsif( "templ_server_advanced" eq $templateCode){
    return nextFrameServerAdvanced(@_);
  }
  else{
    return nextFrameDefault(@_);
  }
}


sub createParameterTemplates{
  my ($this) = @_;
  my $ps =      {
                 "INSTANCE.DBNAME" =>
                 [
                  undef,
                  "MAXDB1",
                  undef,
                  undef,
                  undef,
                  undef
                 ]
                };
  return mergeParameterTemplates($this->SUPER::createParameterTemplates(),$ps);
}



sub createTemplateOverView{
  my ($this) = @_;
  my $descs = $this->SUPER::createTemplateOverView();

  my $own =  [
#              $this->createFrameDescription("InstanceOsOltpWs")
             ];

  push @$descs, @$own; 
  return $descs;
}






sub createFrameTemplates{
  my ($this) = @_;
  my $frs = 
    {
     "InstanceOsOltpWs" =>
     {
      "Title" => "Database Instance Parameters",
      "FrameId" => "InstanceOsOltpWs",
      "Data" => 
      {
      },
      "MetaData" => 
      {
       "INSTANCE.DBNAME" ,{"priority" => 0},
       "INSTANCE.DBSIZE" ,{"priority" => 1},
       "INSTANCE.DBMEMORY" ,{"priority" => 2},
       "INSTANCE.CPUCOUNT",{"priority" => 3} }
     }                          #frame  InstanceOsOltpWs

     ,

     "PathsOsOltpWsWin" => 
     {
      "Title" => "Installation Drives",
      "FrameId" => "PathsOsOltpWsWin",
      "Data" => {  },
      "MetaData" => 
      {
       "SWDRIVE" ,{"priority" => 0},
       "SERVERDRIVE" ,{"priority" => 1}
      }
     }                          #Frame PathsOsOltpWsWin
     ,

     "PathsOsOltpWsUx" =>  
     {
      "Title" => "Installation Paths",
      "FrameId" => "PathsOsOltpWsUx",
      "Data" => {  },
      "MetaData" => 
      {
       "SOFTWAREPATH" ,{"priority" => 0},
       "SERVERPATH",{"priority" => 1}
      }
     }                          #Frame PathsOsOltpWsUx
    };
  return mergeFrameTemplates($this->SUPER::createFrameTemplates(),$frs);
}





1;








# Local Variables: ***
# perl-dbg-flags:"-w      -I../../../../../../wrk/script -I../../../sdbrun  " ***
# sdbinst-defines: "-define SCRIPTING=YES -define PERL5.6.1=YES" ***
# sdbinst-testfile: "TemplateTest.pl" ***
# sdbinst-script-directory:  "sys/wrk/script/SAPDB/Install/Templates" ***
# End: ***

