#!/usr/bin/perl

package SAPDB::Install::Templates::Template;

#ifdef SCRIPTING
#use strict;
#use SAPDB::Install::Templates::Util;
#use SAPDB::Install::Registry;
#use SAPDB::Install::MachInfo;
#require Exporter;
#our @ISA = qw( Exporter );
#our @EXPORT = qw(
#                 voidValidate
#                 driveValidate
#                 createParameterDescription
#                 getParams
#                 createFrameDescription
#                );
#endif

#because SDBINST
our @ISA = qw (SDB::Common::Exporter);
our @EXPORT = qw (
                  voidValidate
                  driveValidate
                  createParameterDescription
                  getParams
                  createFrameDescription);

use SAPDB::Install::Templates::Util;
use SAPDB::Install::Registry;
use SAPDB::Install::MachInfo;

#endbecause

#ifdef doc
#<h1>SAPDB::Install::Templates::Template</h1>
#  <p> Basisklasse aller Install Templates</p>
#  <h2>Usage</h2>
#  <h3 label="liste">Liste mit Templatebeschreibungen beschaffen </h3>
#  <pre>
#  my $templateStruct =  SAPDB::Install::Templates::Setup::getTemplateDescriptions();
#
#foreach my $section (@$templateStruct) {
#  my $label = $section->{"label"};
#  my $templateDescriptions = $section->{"data"};
#  foreach my $templateDescription (@$templateDescriptions) {
#    my $templateCode = $templateDescription->{"TemplateCode"};
#    my $shortDesc = $templateDescription->{"ShortDescription"};
#    my $longDesc = $templateDescription->{"LongDescription"};
#  }
#}
#</pre>
#  <h3 label="template">Templateinstanz erzeugen</h3>
#  <pre>
#  my $template = 
#  SAPDB::Install::Templates::Setup::createTemplate($templateCode);
#</pre>
#  <p>Alternativ kann ein Template von Datei gelesen und geschrieben werden:</p>
#  <pre>
#  my $template = 
#  SAPDB::Install::Templates::Setup::createInstallTemplateFromFile
#  ("~/template.config");
#
#SAPDB::Install::Templates::Setup::writeInstallTemplateToFile
#  ($template,"~/template2.config");
#</pre>
#  <h3 label="frames">Frames fr die GUI-Anzeige beschaffen</h3>
#  <p>
#  Innerhalb einer template-basierten Installation 
#  klickt sich der Anwender durch mehrere Eingabemasken.
#  Jede dieser Eingabemasken bezeichnen wir als <code>Frame</code>
#  </p>
#  <pre>
#  my $aFrame=undef;
#while ( defined ($aFrame=$template->nextFrame($aFrame))) {
#  my $frameTitle = $aFrame->{"Title"};
#  my $frameDescription = $aFrame->{"Description"};
#  my $frameLongDescription = $aFrame->{"LongDescription"};
#  my $frameId = $aFrame->{"FrameId"};
#  my $parHash = $aFrame->{"Data"};
#}
#</pre>
#  <h3 label="params" >Info zu einem Parameter beschaffen</h3>
#  <pre>
#  foreach my $parKey (keys %$parHash) {
#    #Parameterinformationen abfragen:
#    my $param = $parHash->{$parKey};
#    my $default = $param->{'Default'};
#    my $minimum = $param->{'Minimum'};
#    my $maximum = $param->{'Maximum'};
#    my $label = $param->{'Label'};
#    my $description = $param->{'Description'};
#    my $longDescription = $param->{'LongDescription'};
#    my $type = $param->{'Type'};
#    my $getter = $param->{'Getter'};
#    my $validator = $param->{'Validator'};
#    my $setter = $param->{'Setter'};
#    my $control = $param->{'ControlType'};
#  }
#</pre>
#  <h3 label="controls">Beschreibung der Controls beschaffen. </h3>
#  <pre><![CDATA[
#                #############################################
#                # Welches Control soll verwendet werden???
#                #############################################
#                my $controlType =  $control->{'id'};
#                if ( $controlType eq 'stringType') {
#                } elsif ( $controlType eq 'booleanType') {
#                } elsif ( $controlType eq 'pagesType') {
#                } elsif ......
#                #############################################
#                # Anzeigereihenfolge
#                #############################################
#                my $priority =  $control->{'priority'};
#                #############################################
#                # Control ist ausgeschaltet
#                #############################################
#                my $controlTypeDisabled = $control->{'Disabled'}; 
#                #############################################
#                # Abhngigkeiten zwischen Controls auflsen:
#                #############################################
#                my $dependencies = $control->{'dependencies'};
#                foreach my $dependency (@dependencies) {
#                  # Aktuelle Version der abhngigen Parameter holen:
#                  my $param= $template->createParameterDescriptionById($dependency);
#                }
#               ]]>
#  </pre>
#
#  <h4 label="controltypes"> Schlssel/Wert-Paare im Hash  <code>$control</code> </h4>
#
#  <ul>
#  <li label="stringType" anchor="yes">
#  <ul>
#  <li label="id">stringType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  <li label="Disabled">Booleanwert, ob aktiv</li>
#  </ul>
#  </li>
#  <li label="booleanType" anchor="yes">
#  <ul>
#  <li label="id">booleanType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  <li label="Disabled">Booleanwert, ob aktiv</li>
#  </ul>
#  </li>
#  <li label="pathType" anchor="yes">
#  <ul>
#  <li label="id">pathType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  <li label="Disabled">Booleanwert, ob aktiv</li>
#  </ul>
#  </li>
#  <li label="pagesType" anchor="yes">
#  <ul>
#  <li label="id">pagesType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  <li label="Disabled">Booleanwert, ob aktiv</li>
#  </ul>
#  </li>
#  <li label="driveType" anchor="yes">
#  <ul>
#  <li label="id">driveType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  <li label="Disabled">Booleanwert, ob aktiv</li>
#  </ul>
#  </li>
#
#  <li label="passwordType" anchor="yes">
#  <ul>
#  <li label="id">passwordType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  <li label="Disabled">Booleanwert, ob aktiv</li>
#  </ul>
#  </li>
#  <li label="sliderType" anchor="yes">
#  (Schieberegler)
#  <ul>
#  <li label="id">sliderType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  <li label="Disabled">Booleanwert, ob aktiv</li>
#  <li label="unit">Einheit, eines von 
#  <ul> 
#  <li label="pages"> 8K-Pages</li>
#  <li label="number"> 1,2,3, ...</li>
#  </ul>
#  </li>
#  </ul>
# Wertebereich:
#<ul>
#  <li label="Minimalwert"><code>$param-&gt;{'Minimum'};</code> </li>
#  <li label="Initialwert"><code>$param-&gt;{'Default'};</code> </li>
#  <li label="Maximalwert"><code>$param-&gt;{'Maximum'};</code> </li>
#  </ul>
#  </li>
#
#  <li label="spinnerType" anchor="yes">
#  (Spin Control)
#  <ul>
#  <li label="id">spinnerType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  <li label="Disabled">Booleanwert, ob aktiv</li>
#  <li label="unit">Einheit, eines von 
#  <ul> 
#  <li label="pages"> 8K-Pages</li>
#  <li label="number"> 1,2,3, ...</li>
#  </ul>
#  </li>
#  </ul>
# Wertebereich:
#<ul>
#  <li label="Minimalwert"><code>$param-&gt;{'Minimum'};</code> </li>
#  <li label="Initialwert"><code>$param-&gt;{'Default'};</code> </li>
#  <li label="Maximalwert"><code>$param-&gt;{'Maximum'};</code> </li>
#  </ul>
#  </li>
#
#  <li label="labelType" anchor="yes">
#  Label mit statischem Text. Die Setter-Funktion hat hier keine Auswirkungen. Den
#  String fr das Label erhlt man ber die Getter-Funktion.
#  <ul>
#  <li label="id">labelType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  <li label="Disabled">Booleanwert, ob aktiv</li>
#  </ul>
#  </li>
#
#  <li label="datavolumesType" anchor="yes">
#  Fr die Datenvolumes im advanced template. 
#  <ul>
#  <li label="id">datavolumesType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  </ul>
#  </li>
#
#  <li label="logvolumesType" anchor="yes">
#  Fr die Logvolumes im advanced template.
#  <ul>
#  <li label="id">logvolumesType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  </ul>
#  </li>
#
#  <li label="simplevolumesType" anchor="yes">
#  Fr die Pfad bzw Laufwerksauswahl in allen Templates auer <code>"Advanced"</code>
#  <ul>
#  <li label="id">simplevolumesType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  </ul>
#  </li>
#  <li label="dbuserType" anchor="yes">
#  Fr den Datenbankuser im Desktop Template
#  <ul>
#  <li label="id">dbuserType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  </ul>
#  </li>
#  <li label="adminusersType" anchor="yes">
#  Fr System-User Auswahl in allen Templates auer Desktop-Template
#  <ul>
#  <li label="id">dbuserType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  </ul>
#  </li>
#
#  <li label="comboBoxType" anchor="yes">
#  Fr Auswahlliste in einer Combobox.
#  <ul>
#  <li label="id">dbuserType</li>
#  <li label="priority">Anzeigereihenfolge</li>
#  <li label="list">Referenz auf Liste mit den mglichen Werten</li>
#  </ul>
#  </li>
#
#
#  </ul>
#
#
#
#  <h2 label="compound_params">Zusammengesetzte Parameter</h2>
#  <p>
#  Fr manche Controls (z.b. Volumeauswahl und User) braucht man
#  Parameter, die aus anderen Parametern zusammengesetzt sind.
#  </p>
#  <p>
#  Die Werte zusammengesetzter Parameter erhlt man und setzt man
#  wie bei primitiven Parametern.
#  </p>
#  <p>
#  Momentan hngen an zusammengesetzten Parametern folgende Controls:
#</p>
#  <ul>
#  <li><a href="#datavolumesType">datavolumesType</a></li>
#  <li><a href="#logvolumesType">logvolumesType</a></li>
#  <li><a href="#simplevolumesType">simplevolumesType</a></li>
#  <li><a href="#dbuserType">dbuserType</a></li>
#  <li><a href="#adminusersType">adminusersType</a></li>
#  </ul>
#  <p> Man erkennt den Typ des zusammengesetzen
#  Parameters am Key <code>Type</code>. Dieser Key
#  ist nicht zu verwechseln mit <code>ControlType</code>, der das Control spezifiziert. Mehrere
#  unterschiedliche Controls knnen nmlich denselben zusammengesetzten
#  Parametertyp verwenden. Pseudocode: </p>
#  <pre><![CDATA[             # Ich habe die bliche Parameterstruktur:
#                my $param = ...;
#
#                #Und hole mir die Werte wie bei "primitiven" Parametern:
#                my $type = $param->{'Type'};
#                my $getter = $param->{'Getter'};
#                my $validator = $param->{'Validator'};
#                my $setter = $param->{'Setter'};
#                my $control = $param->{'ControlType'};
#                my $value = &$getter();
#
#                #Ich behandle die speziellen Controls:
#                if ( $control eq "datavolumesType") {
#                  #Hier wei ich, da ich eine Liste von Volumes habe ($type == "VOLUMELIST.COMPOUND")
#                  foreach my $volume (@$value) {
#                    my $location = $volume->{"Location"};
#                    ...
#                  }
#                } elsif ( $control eq "adminusersType") {
#                  #Hier wei ich, da ich eine Liste von Usern haben ($type == "USERLIST.COMPOUND").
#                  foreach my $user (@$value) {
#                    my $userName = $user->{"Name"};
#                    ...
#                  }
#                } elsif ...]]></pre>
#  <p>
#  Und hier die zusammengesetzten Parametertypen
#  </p>
#
#
#  <h3 label="volumeListCompound"><code>Type: VOLUMELIST.COMPOUND</code></h3>
#  <pre><![CDATA[my $param = ...
#                my $getter = $param->{'Getter'};
#                my $value = &$getter();
#                foreach my $volume (@$value) {
#                  my $drive = $volume->{"Drive"};
#                  my $dir = $volume->{"Dir"};
#                  my $number = $volume->{"Number"};
#                  my $art = $volume->{"Art"};
#                  my $location = $volume->{"Location"};
#                  my $prefix = $volume->{"Prefix"};
#                  my $postfix = $volume->{"Postfix"};
#                  my $type = $volume->{"Type"};
#                  my $size = $volume->{"Size"};
#                  my $name = $volume->{"Name"};
#                  my $label = $volume->{"Label"};
#                }
#               ]]></pre>
#  <p>Bedeutung der Keys fr die Volumepfade:</p>
#  <ul>
#  <li><code>"Location"</code> ist das Directory des Volumes. </li>
#  <li><code>"Name"</code> ist der Name des Volumes. </li>
#  <li><code>"Prefix"</code> ist der Anfang von <code>"Location"</code> </li>
#  <li><code>"Postfix"</code> ist das Ende von <code>"Location"</code> </li>
#  </ul>
#  <p>
#  Wenn <code>catfile</code> eine Funktion zum
#  Zusammensetzen von Pfadangaben ist, dann gilt also:
#</p>
#
#  <pre><![CDATA[$location = catfile($prefix,$postfix);
#                $completeVolumePath = catfile($location,$name);
#                #Ausserdem gilt aus "historischen" Grnden:
#                $dir = $location]]></pre>
#
#  <h3 label="userListCompound"><code>Type: USERLIST.COMPOUND</code></h3>
#  <pre><![CDATA[my $param = ...
#                my $getter = $param->{'Getter'};
#                my $value = &$getter();
#                foreach my $user (@$value) {
#                  my $desc = $user->{"Description"};
#                  my $name = $user->{"Name"};
#                  my $pwd = $user->{"Password"};
#                  my $cpwd = $user->{"ConfirmPassword"};
#                  my $role = $user->{"Role"};
#                }
#               ]]></pre>
#  <h3 label="userCompound"><code>Type: USER.COMPOUND</code></h3>
#  <pre><![CDATA[my $param = ...
#                my $getter = $param->{'Getter'};
#                my $value = &$getter();
#                my $desc = $value->{"Description"};
#                my $name = $value->{"Name"};
#                my $pwd = $value->{"Password"};
#                my $cpwd = $value->{"ConfirmPassword"};
#                my $role = $value->{"Role"};]]></pre>
#  <p>
#  (Ende der Beschreibung der zusammengesetzten Parameter.)
#  </p>
#
#  <h2 label="validierung">Validierung</h2>
#
#  <p>Vor dem Setzen eines Parameters, kann dieser folgendermaen validiert werden.
#  (Siehe <a href="#detailsvalidators">Details zu Validatoren</a>)</p>
#  <pre>
#  my @validationResult = &amp;$validator($currentValue);
#my $validationError = $validationResult[1];
#my $validationDescription = $validationResult[2];
#</pre>
#  <p>
#  Die Gesamtkonfiguration des Templates kann aktualisiert und auf Konsistenz getestet werden.
#  Der Rckgabewert von <code>configure</code> ist analog zum dem der Validatoren. 
#  (Siehe <a href="#detailsvalidators">Details zu Validatoren</a>)
#  </p>
#  <pre>
#  my @validationResult = $template->configure();
#my $validationError = $validationResult[1];
#my $validationDescription = $validationResult[2];
#</pre>
#
#  <h2>Parameterwerte bekommen und setzen</h2>
#  <pre>
#  #Wert eines Parameters abfragen:
#  my $currentValue = &amp;$getter();
##Wert eines Parameters setzen:
#&amp;$setter("A value");
#</pre>
#
#  <h2 label="paths"> Installationspfade beschaffen</h2>
#  <pre>
#  my $indepPath = $template->getIndependendPrograms();
#my $indepData = $template->getIndependendData();
#my $depend = $template->getDependendPath();
#my  $user = $template->getSDBUser();
#my  $group = $template->getSDBAGroup();
#</pre>
#
#  <h2 label="callback"> Callback-Funktion fr Statusmeldungen und Log setzen</h2>
#  <pre>
#  $template->setStatusCallback(sub{print ($_[0]);});
#$template->setLogCallback(sub{print ($_[0]);});
#</pre>
#  <h3 label="ueberblick"> Templatebeschreibung beschaffen</h3>
#  <p>
#  So beschafft man sich eine aus verschiedenen Sektionen zusammengesetzte
#  Beschreibung des Templates:
#</p>
#  <pre><![CDATA[   my $desc = $template->createTemplateOverView();
#                   foreach my $frame (@$desc) {
#                     my $frameTitle = $frame->{"Title"};
#                     my $frameExpandDescription = $frame->{"SummaryExpand"};
#                     print "\n" . $frameTitle . "\n";
#                     print  "==========================\n";
#                     my $parHash = $frame->{"Data"};
#                     my $sortedKeys =
#                       [sort   
#                        {
#                          $parHash->{$a}->{'ControlType'}->{'priority'}
#                            <=>
#                              $parHash->{$b}->{'ControlType'}->{'priority'} }
#                        keys   %$parHash];
#                     foreach my $name (@$sortedKeys) {
#                       my $param= $parHash->{$name};
#                       my $label = $param->{'Label'};
#                       my $description = $param->{'Description'};
#                       my $getter = $param->{'Getter'};
#                       my $value = &$getter();
#                       print $label . ": " . $value . "\n" ;
#                     }
#                   }
#               ]]></pre>
#  <h3>Installation vorbereiten und starten:</h3>
#  <pre>
#  $template->configure();
#$template->install();
#</pre>
#
#  <h2>Details zu Frames</h2>
#  <p>
#  Im Install Template wird die Reihenfolge und der Inhalt
#  der <code>Frames</code> festgelegt. Der jeweils als nchstes
#  anzuzeigende Frame wird ber die Funktion <a href="#nextFrame">nextFrame</a>
#  zurckgeliefert.
#  </p>
#  <h2 label="detailsvalidators">Details zu Validatoren</h2>
#  <p>Validatoren berprfen Benutzereingaben auf ihre Gltigkeit. Sie
#  erwarten einen Parameter, dessen Typ man dokumentieren mu. Folgenden
#  Rckgabewerte (Listen) sind zulssig:</p>
#  <ul>
#  <li label="(0,'Kurzbeschreibung Fehler','Langbeschreibung Fehler')">
#  Eingabewert war <strong>nicht gltig</strong> 
#  </li>
#  <li label="(1)"> Eingabewert war <strong>gltig</strong></li>
#  <li label="(2)"> Gltigkeitsprfung fehlgeschlagen (Interner Fehler)</li>
#  <li label="(3)"> Keine Gltigkeitsprfung erfolgt, da ungltiger Typ des bergabeparameters</li>
#  <li label="(4)"> Gltigkeitsprfung nicht mglich.</li>
#  </ul>
#  <p>
#  Alle Werte ungleich 0 sollten vom GUI nicht an den Benutzer rckgemeldet werden.
#  Parameter, fr die keine Gltigkeitsprfung implementiert ist, sind mit
#  einem <a href="#voidValidate">Dummy-Validator</a> versehen, der stets <code>(4)</code>
#  zurckliefert.
#  </p>
#
#
#
#
#
#  <p>Weiterverarbeitung der Parameter siehe <a href="#params">hier ...</a></p>
#
#
#  <h2 label="async-callbacks"> Callback fr asynchron abgearbeitet dbmcli-Befehle</h2>
#  <pre><![CDATA[ $template->setAsyncCallback( sub{
#                                                my $elapsed = $_[0];
#                                                my $txt = $_[1];
#                                                print "$txt: $elapsed seconds elapsed\n";
#                                              }
#                                            );
#               ]]>
#  </pre>
#
#  <h2 label="statuslist-callbacks"> Callback fr Statusanzeige beim Anlegen der Instanz.</h2>
#  <p>Die Callbackfunktion wird aufgerufen mit einem Parameter. Der Parameter ist 
#  eine Liste aus Statusmeldung. Jede Statusmeldung hat folgende Eigenschaften:</p>
#
#  <ul>
#  <li label="Label">Kurztext</li>
#  <li label="Id">Eindeutiger Kenner</li>
#  <li label="Status">Aktueller Status: 
#<ul>
#  <li label="PENDING"></li>
#  <li label="RUNNING"></li>
#  <li label="FINISHED_SUCCESS"></li>
#  <li label="FINISHED_ERROR"></li>
#  </ul>
#  </li>
#  <li label="StartTime">Startzeit (Perl-Funktion <code>time()</code>)</li>
#  <li label="EndTime">Endzeit (Perl-Funktion <code>time()</code>)</li>
#  </ul>
#  <p>Beispielcode:</p>
#  <pre><![CDATA[$template->setStatusListCallback
#                (sub{ my $statusList =  $_[0];
#                      foreach my $listItem (@$statusList) {
#                        my $label = $listItem->{"Label"};
#                        my $id = $listItem->{"Id"};
#                        my $status = $listItem->{"Status"};
#                        my $startTime = $listItem->{"StartTime"};
#                        $startTime = defined $startTime ? $startTime : "?";
#                        my $endTime = $listItem->{"EndTime"};
#                        $endTime = defined $endTime ? $endTime : "?";
#                      }
#                    });]]>
#  </pre>
  #endif


  #PARAMS
  sub getParams{
    my ($this) = @_;
    my $myparams  
      =  
        {
         "SOFTWAREPATH" => [
                            sub {$this->setAdvicedSoftwarePath($_[0]);},
                            sub {return $this->getSoftwarePath();},
                            \&defaultFormatter,
                            "PATH",
                            "Software",
                            \&voidValidate,
                            {"priority" => 99
                            }
                           ],
         "SWDRIVE" => [
                       sub {$this->setAdvicedSWDrive($_[0]);},
                       sub {return $this->getSWDrive();},
                       \&defaultFormatter,
                       "DRIVE",
                       "Software",
                       \&driveValidate,
                       {"priority" => 100
                       }
                      ],
         "SERVERPATH" => [
                          sub {$this->setAdvicedServerPath($_[0]);},
                          sub {return $this->getServerPath();},
                          \&defaultFormatter,
                          "PATH",
                          "Software",
                          \&voidValidate,
                          {"priority" => 99
                          }
                         ],
         "SERVERDRIVE" => [
                           sub {$this->setAdvicedServerDrive($_[0]);},
                           sub {return $this->getServerDrive();},
                           \&defaultFormatter,
                           "DRIVE",
                           "Software",
                           \&driveValidate,
                           {"priority" => 100
                           }
                          ],
         "ISSQLSTUDIO" => [
                           sub {$this->setAdvicedIsSQLStudio($_[0]);},
                           sub {return $this->getIsSQLStudio();},
                           \&booleanFormatter,
                           "BOOLEAN",
                           "Software", 
                           \&voidValidate,
                           {"priority" => 0
                           }
                          ],

         "ISUNINSTALLSQLSTUDIO" => [
                                    sub {$this->setIsUninstallSQLStudio($_[0]);},
                                    sub {return $this->getIsUninstallSQLStudio();},
                                    \&booleanFormatter,
                                    "BOOLEAN",
                                    "Software", 
                                    \&voidValidate,
                                    {"priority" => 0
                                    }
                                   ],

         "ISUNINSTALLDBMGUI" => [
                                 sub {$this->setIsUninstallDBMGui($_[0]);},
                                 sub {return $this->getIsUninstallDBMGui();},
                                 \&booleanFormatter,
                                 "BOOLEAN",
                                 "Software", 
                                 \&voidValidate,
                                 {"priority" => 0
                                 }
                                ],
         "ISDELETEINSTANCE" => [
                                sub {$this->setIsDeleteInstance($_[0]);},
                                sub {return $this->getIsDeleteInstance();},
                                \&booleanFormatter,
                                "BOOLEAN",
                                "Software", 
                                \&voidValidate,
                                {"priority" => 0
                                }
                               ],
         "INDEPDATA" => [
                         sub {$this->setIndependendData($_[0]);},
                         sub {return $this->getIndependendData();},
                         \&defaultFormatter,
                         "PATH",
                         "Software", 
                         \&voidValidate,
                         {"priority" => 0
                         }
                        ],


         "INDEPPROG" => [
                         sub {$this->setIndependendPrograms($_[0]);},
                         sub {return $this->getIndependendPrograms();},
                         \&defaultFormatter,
                         "PATH",
                         "Software", 
                         \&voidValidate,
                         {"priority" => 0
                         }
                        ],


         "ISCREATEINSTANCE" => [
                                sub {$this->setIsCreateInstance($_[0]);},
                                sub {return $this->getIsCreateInstance();},
                                \&booleanFormatter,
                                "BOOLEAN",
                                "Software", 
                                \&voidValidate,
                                {"priority" => 0
                                }
                               ],

         "ISINSTALLSOFTWARE" => [
                                 sub {$this->setIsInstallSoftware($_[0]);},
                                 sub {return $this->getIsInstallSoftware();},
                                 \&booleanFormatter,
                                 "BOOLEAN",
                                 "Software", 
                                 \&voidValidate,
                                 {"priority" => 0
                                 }
                                ],

         "SDBUSER" => [
                       sub {$this->setSDBUser($_[0]);},
                       sub {return $this->getSDBUser();},
                       \&defaultFormatter,
                       "STRING",
                       "Software", 
                       \&voidValidate,
                       {"priority" => 0
                       }
                      ],

         "SDBAGROUP" => [
                         sub {$this->setSDBAGroup($_[0]);},
                         sub {return $this->getSDBAGroup();},
                         \&defaultFormatter,
                         "STRING",
                         "Software", 
                         \&voidValidate,
                         {"priority" => 0
                         }
                        ],


         "ISDBMGUI" => [
                        sub {$this->setAdvicedIsDBMGui($_[0]);},
                        sub {return $this->getIsDBMGui();},
                        \&booleanFormatter,
                        "BOOLEAN",
                        "Software",
                        \&voidValidate,
                        {"priority" => 0
                        }
                       ]
         ,
         "PATHSTRATEGY" => [
                            sub {$this->setAdvicedPathStrategy($_[0]);},
                            sub {return $this->getPathStrategy();},
                            \&defaultFormatter,
                            "STRING",
                            "Software",
                            \&voidValidate,
                            {
                             "priority" => 150
                            }
                           ]
         ,
         "TUTORIAL.USER" => [
                            sub {},
                            sub {return "MONA, (Password: RED)";},
                            \&defaultFormatter,
                            "STRING",
                            "Software",
                            \&voidValidate,
                            {
                             "priority" => 0, "transient" => 1
                            }
                           ]

        };
    return $myparams;
  }                             #getParams





#+field(String:PathStrategy)
my $PathStrategy="PathStrategyKey";
#+sub(String):void:Set PathStrategy
sub setPathStrategy{
  my ($this, $aValue) = @_;
  $this->{$PathStrategy}=$aValue;
}
#+sub():String:Get PathStrategy
sub getPathStrategy{
  my ($this ) = @_;
  return $this->{$PathStrategy};
}
  
sub setAdvicedPathStrategy{
  my ($this, $aValue) = @_;
  willSet($this,$PathStrategy);
  $this->setPathStrategy($aValue);
  haveSet($this,$PathStrategy);
}




sub getAllParams{
  my ($this) = @_;
  my $myparams = getParams(@_); 
  if (defined $this) {
    my $instance = $this->getDBInstance();
    if (defined $instance) {
      my $instpars = $instance->getParams();
      foreach my $key (keys %$instpars) {
        my $val = $instpars->{$key};
        $myparams->{"INSTANCE." . $key}=$val;
      }
    }
  }
  return $myparams;
}


sub setDefaults{
  my ($this) = @_;
  unless ( $^O =~ /mswin/i) {
    $this->setSDBUser("sdb");
    $this->setSDBAGroup("sdba");
  }
  $this->setStatusCallback(sub{print $_[0];});
  $this->setLogCallback(sub{print $_[0];});
}



#ifdef sub:nextFrame
#<ul>
#  <li>Falls <code>undef</code> bergeben wird,
#  wird der erste Frame zurckgeliefert.</li>
#  <li>
#  Falls der gerade angezeigte Frame bergeben wird,
#  wird der nchste anzuzeigende Frame zurckgeliefert.
#  </li>
#  <li>Falls der letzte Frame bergeben wird, wird 
#  <code>undef</code> zurckgeliefert.</li>
#  </ul>
  #endif

  #+sub(Frame:previousFrame):Frame:Liefert die anzuzeigenden Frames in der richtigen Reihenfolge
  sub nextFrame{
  }

sub statusMessage{
  my ($this,$message ) = @_;
  my $statusCallback = $this->getStatusCallback();
  unless(defined $statusCallback){
    print "Template[]: " . $message;
    return;
  }
  my $atemplateCode = $this->getTemplateCode();
  &$statusCallback("$message");
}

#ifdef sub:install
#<p>
#  Der Rckgabewert von <code>install</code> ist analog zum dem der Validatoren. 
#  (Siehe <a href="#detailsvalidators">Details zu Validatoren</a>)
#  </p>
  #endif

  #+sub():void:Startet den Installationsvorgang
  sub install{
    my ($this ) = @_;
    my $success = 1;
    my @retVal = (1);
    if ($this->getIsCreateInstance()) {
      my $instance = $this->getDBInstance();
      unless(defined $instance->createInstance()){
        $success = 0;
        @retVal = ($success,$instance->getErrorMessage(),$instance->getErrorDescription());
      }
    } else {
      my $instance = $this->getDBInstance();
      if (defined $instance) {
        my $dbname = $instance->getDBName();
        $this->statusMessage("Skipped installation of database instance $dbname\n");
      } else {
        $this->statusMessage("Skipped installation of database instance\n");
      }
    }
    if ($this->getIsDBMGui() ) {
      my $dbmGui = $this->getDBMGui();
      if ( $dbmGui->install() < 0) {
        $success = 0;
        @retVal = ($success,$dbmGui->getErrorMessage(),$dbmGui->getErrorDescription());
      }
      ;
    }
    if ($this->getIsSQLStudio() ) {
      my $sqlStudio = $this->getSQLStudio();
      if ( $sqlStudio->install() < 0) {
        $success = 0;
        @retVal = ($success,$sqlStudio->getErrorMessage(),$sqlStudio->getErrorDescription());
      }
    }
    return @retVal;
  }

#+sub():void:Startet den Installationsvorgang
sub uninstall{
  my ($this ) = @_;
  my $success = 1;
  my @retVal = (1);
  if ($this->getIsDeleteInstance()) {
    my $instance = $this->getDBInstance();
    unless(defined $instance->deleteInstance()){
      $success = 0;
      @retVal = ($success,$instance->getErrorMessage(),$instance->getErrorDescription());
    }
  }
  if ($this->getIsUninstallDBMGui() and $success ) {
    my $dbmGui = $this->getDBMGui();
    if ( $dbmGui->uninstall() < 0) {
      $success = 0;
      @retVal = ($success,$dbmGui->getErrorMessage(),$dbmGui->getErrorDescription());
    }
    ;
  }
  if ($this->getIsUninstallSQLStudio() and $success ) {
    my $sqlStudio = $this->getSQLStudio();
    if ( $sqlStudio->uninstall() < 0) {
      $success = 0;
      @retVal = ($success,$sqlStudio->getErrorMessage(),$sqlStudio->getErrorDescription());
    }
  }
  return @retVal;
}

#+sub():void:Konfiguriert vor dem Installationsvorgang
sub configure{
  my ($this ) = @_;
  my @configResult = (1);
  if ( $this->getIsCreateInstance() or $this->getIsDeleteInstance()) {
    @configResult = $this->configureDBInstance();
  }
  if ($configResult[0]) {
    if ($this->getIsSQLStudio() or $this->getIsUninstallSQLStudio()) {
      @configResult = $this->configureSQLStudio();
    }
  }
  if ($configResult[0]) {
    if ($this->getIsDBMGui() or $this->getIsUninstallDBMGui()) {
      @configResult = $this->configureDBMGui();
    }
  }
  #  print "Result: $configResult[0]\n";
  return @configResult;
}


#+sub():void:Konfiguriert vor dem Installationsvorgang
sub initialize{
  my ($this ) = @_;
  if(defined $this->getDBInstance()){
    $this->configureDBInstance();
  }
  if(defined $this->getSQLStudio()){
    $this->configureSQLStudio();
  }
  if(defined $this->getDBMGui()){
    $this->configureDBMGui();
  }
}





 
sub configureDBInstance{
  my ($this ) = @_;
  my $instance = $this->getDBInstance();
  my @configResult = (1);
   
  if (defined $instance) {
    @configResult = $instance->configure();
    if ($configResult[0]) {
      my $statusCallback = $this->getStatusCallback();
      my $logCallback = $this->getLogCallback();
      my $asi = $this->getAsyncCallback();
      my $statusListCallback = $this->getStatusListCallback();
      $instance->setStatusCallback($statusCallback);
      $instance->setLogCallback($logCallback);
      if (defined $asi) {
        $instance->setAsyncCallback($asi);
      }
      if (defined $statusListCallback) {
        $instance->setStatusListCallback($statusListCallback);
      }
    }
  }
  return @configResult;
}

sub configureSQLStudio{
  my ($this ) = @_;
  my @configResult = (1);
  my $sqlStudio = $this->getSQLStudio();
  if (defined $sqlStudio) {
    @configResult = $sqlStudio->configure();
    my $indepProg = $this->getIndependendPrograms();
    my $statusCallback = $this->getStatusCallback();
    my $logCallback = $this->getLogCallback();
    if (defined $indepProg ) {
      $sqlStudio->configureFromProgDir($indepProg);
    }
    $sqlStudio->setStatusCallback($statusCallback);
    $sqlStudio->setLogCallback($logCallback);
  }
  return @configResult;
}

sub configureDBMGui{
  my ($this ) = @_;
  my $dbmGui = $this->getDBMGui();
  my @configResult = (1);
  if (defined $dbmGui) {
    @configResult = $dbmGui->configure();
    my $indepProg = $this->getIndependendPrograms();
    my $statusCallback = $this->getStatusCallback();
    my $logCallback = $this->getLogCallback();
    if (defined $indepProg ) {
      $dbmGui->configureFromProgDir($indepProg);
    }
    $dbmGui->setStatusCallback($statusCallback);
    $dbmGui->setLogCallback($logCallback);
  }
  return @configResult;
}


#+field(Base:DBInstance)
my $DBInstance="DBInstanceKey";
#+sub(Base):void:Set DBInstance
sub setDBInstance{
  my ($this, $aValue) = @_;
  $this->{$DBInstance}=$aValue;
  if ( defined $aValue) {
    $aValue->setAdvicedTemplateCode($this->getTemplateCode());
  }
  #  $this->configure();
}
#+sub():Base:Get DBInstance
sub getDBInstance{
  my ($this ) = @_;
  return $this->{$DBInstance};
}



#+field(Path:SoftwarePath)
my $SoftwarePath="SoftwarePathKey";
#+sub(Path):void:Set SoftwarePath
sub setSoftwarePath{
  my ($this, $aValue) = @_;
  $this->{$SoftwarePath}=$aValue;
}

sub setAdvicedSoftwarePath{
  my ($this, $aValue) = @_;
  willSet($this,$SoftwarePath);
  $this->setSoftwarePath($aValue);
  haveSet($this,$SoftwarePath);
}



#+sub():Path:Get SoftwarePath
sub getSoftwarePath{
  my ($this ) = @_;
  return $this->{$SoftwarePath};
}

#+field(String:InstallProfile)
my $InstallProfile="InstallProfileKey";
#+sub(String):void:Set InstallProfile
sub setInstallProfile{
  my ($this, $aValue) = @_;
  $this->{$InstallProfile}=$aValue;
}

sub setAdvicedInstallProfile{
  my ($this, $aValue) = @_;
  willSet($this,$InstallProfile);
  $this->setInstallProfile($aValue);
  haveSet($this,$InstallProfile);
}
#+sub():String:Get InstallProfile
sub getInstallProfile{
  my ($this ) = @_;
  return $this->{$InstallProfile};
}








#+field(Path:SWDrive)
our $SWDrive="SWDriveKey";

#+sub(Drive):void:Set SWDrive
sub setSWDrive{
  my ($this, $aValue) = @_;
  $this->{$SWDrive}=matchDrive($aValue);
}

sub setAdvicedSWDrive{
  my ($this, $aValue) = @_;
  willSet($this,$SWDrive);
  $this->setSWDrive($aValue);
  haveSet($this,$SWDrive);
}




#+sub():Drive:Get SWDrive
sub getSWDrive{
  my ($this ) = @_;
  return $this->{$SWDrive};
}


#+field(String:TemplateCode)
my $TemplateCode="TemplateCodeKey";
#+sub(String):void:Set TemplateCode
sub setTemplateCode{
  my ($this, $aValue) = @_;
  $this->{$TemplateCode}=$aValue;
}
#+sub():String:Get TemplateCode
sub getTemplateCode{
  my ($this ) = @_;
  return $this->{$TemplateCode};
}

sub setAdvicedTemplateCode{
  my ($this, $aValue) = @_;
  willSet($this,$TemplateCode);
  $this->setTemplateCode($aValue);
  haveSet($this,$TemplateCode);
}







#+field(Path:ServerPath)
my $ServerPath="ServerPathKey";
#+sub(Path):void:Set ServerPath
sub setServerPath{
  my ($this, $aValue) = @_;
  $this->{$ServerPath}=$aValue;
}
sub setAdvicedServerPath{
  my ($this, $aValue) = @_;
  willSet($this,$ServerPath);
  $this->setServerPath($aValue);
  haveSet($this,$ServerPath);
}
#+sub():Path:Get ServerPath
sub getServerPath{
  my ($this ) = @_;
  return $this->{$ServerPath};
}


#+field(Drive:ServerDrive)
my $ServerDrive="ServerDriveKey";
#+sub(Drive):void:Set ServerDrive
sub setServerDrive{
  my ($this, $aValue) = @_;
  $this->{$ServerDrive}=matchDrive($aValue);
}
sub setAdvicedServerDrive{
  my ($this, $aValue) = @_;
  willSet($this,$ServerDrive);
  $this->setServerDrive($aValue);
  haveSet($this,$ServerDrive);
}
#+sub():Drive:Get ServerDrive
sub getServerDrive{
  my ($this ) = @_;
  return $this->{$ServerDrive};
}







#+field(Boolean:IsSQLStudio)
my $IsSQLStudio="IsSQLStudioKey";
#+sub(Boolean):void:Set IsSQLStudio
sub setIsSQLStudio{
  my ($this, $aValue) = @_;
  $this->{$IsSQLStudio}=$aValue;
}
sub setAdvicedIsSQLStudio{
  my ($this, $aValue) = @_;
  willSet($this,$IsSQLStudio);
  $this->setIsSQLStudio($aValue);
  haveSet($this,$IsSQLStudio);
}




#+sub():Boolean:Get IsSQLStudio
sub getIsSQLStudio{
  my ($this ) = @_;
  return $this->{$IsSQLStudio};
}



#+field(Boolean:IsUninstallSQLStudio)
my $IsUninstallSQLStudio="IsUninstallSQLStudioKey";
#+sub(Boolean):void:Set IsUninstallSQLStudio
sub setIsUninstallSQLStudio{
  my ($this, $aValue) = @_;
  $this->{$IsUninstallSQLStudio}=$aValue;
}
#+sub():Boolean:Get IsUninstallSQLStudio
sub getIsUninstallSQLStudio{
  my ($this ) = @_;
  return $this->{$IsUninstallSQLStudio};
}









#+field(Boolean:IsDBMGui)
my $IsDBMGui="IsDBMGuiKey";
#+sub(Boolean):void:Set IsDBMGui
sub setIsDBMGui{
  my ($this, $aValue) = @_;
  $this->{$IsDBMGui}=$aValue;
}
sub setAdvicedIsDBMGui{
  my ($this, $aValue) = @_;
  willSet($this,$IsDBMGui);
  $this->setIsDBMGui($aValue);
  haveSet($this,$IsDBMGui);
}

#+sub():Boolean:Get IsDBMGui
sub getIsDBMGui{
  my ($this ) = @_;
  return $this->{$IsDBMGui};
}


#+field(Boolean:IsUninstallDBMGui)
my $IsUninstallDBMGui="IsUninstallDBMGuiKey";
#+sub(Boolean):void:Set IsUninstallDBMGui
sub setIsUninstallDBMGui{
  my ($this, $aValue) = @_;
  $this->{$IsUninstallDBMGui}=$aValue;
}
#+sub():Boolean:Get IsUninstallDBMGui
sub getIsUninstallDBMGui{
  my ($this ) = @_;
  return $this->{$IsUninstallDBMGui};
}






#+field(Path:IndependendPrograms)
my $IndependendPrograms="IndependendProgramsKey";
#+sub(Path):void:Set IndependendPrograms
sub setIndependendPrograms{
  my ($this, $aValue) = @_;
  $this->{$IndependendPrograms}=$aValue;
}
#+sub():Path:Get IndependendPrograms
sub getIndependendPrograms{
  my ($this ) = @_;
  my ($indep_data,$indep_prog)=readIndepPath(); 
  if (defined $indep_prog and not $indep_prog =~ /^\s*$/) {
    return $indep_prog;
  } else {
    return $this->{$IndependendPrograms};
  }
}



#+field(Path:IndependendData)
my $IndependendData="IndependendDataKey";
#+sub(Path):void:Set IndependendData
sub setIndependendData{
  my ($this, $aValue) = @_;
  $this->{$IndependendData}=$aValue;
}
#+sub():Path:Get IndependendData
sub getIndependendData{
  my ($this ) = @_;
  my ($indep_data,$indep_prog)=readIndepPath(); 
  if (defined $indep_data and not $indep_data =~ /^\s*$/) {
    return $indep_data;
  } else {
    return $this->{$IndependendData};
  }
}


#+field(WindowsInstaller:DBMGui)
my $DBMGui="DBMGuiKey";
#+sub(WindowsInstaller):void:Set DBMGui
sub setDBMGui{
  my ($this, $aValue) = @_;
  $this->{$DBMGui}=$aValue;
}
#+sub():WindowsInstaller:Get DBMGui
sub getDBMGui{
  my ($this ) = @_;
  return $this->{$DBMGui};
}

#+field(WindowsInstaller:SQLStudio)
my $SQLStudio="SQLStudioKey";
#+sub(WindowsInstaller):void:Set SQLStudio
sub setSQLStudio{
  my ($this, $aValue) = @_;
  $this->{$SQLStudio}=$aValue;
}
#+sub():WindowsInstaller:Get SQLStudio
sub getSQLStudio{
  my ($this ) = @_;
  return $this->{$SQLStudio};
}


#+field(Boolean:isCreateInstance)
my $isCreateInstance="isCreateInstanceKey";
#+sub(Boolean):void:Set isCreateInstance
sub setIsCreateInstance{
  my ($this, $aValue) = @_;
  $this->{$isCreateInstance}=$aValue;
}
#+sub():Boolean:Get isCreateInstance
sub getIsCreateInstance{
  my ($this ) = @_;
  return $this->{$isCreateInstance};
}

#+field(Boolean:IsDeleteInstance)
my $IsDeleteInstance="IsDeleteInstanceKey";
#+sub(Boolean):void:Set IsDeleteInstance
sub setIsDeleteInstance{
  my ($this, $aValue) = @_;
  $this->{$IsDeleteInstance}=$aValue;
}
#+sub():Boolean:Get IsDeleteInstance
sub getIsDeleteInstance{
  my ($this ) = @_;
  return $this->{$IsDeleteInstance};
}


#+field(Path:appDataRoot)
my $appDataRoot="appDataRootKey";
#+sub(Path):void:Set appDataRoot
sub setAppDataRoot{
  my ($this, $aValue) = @_;
  $this->{$appDataRoot}=$aValue;
}
#+sub():Path:Get appDataRoot
sub getAppDataRoot{
  my ($this ) = @_;
  return $this->{$appDataRoot};
}

sub setAdvicedAppDataRoot{
  my ($this, $aValue) = @_;
  willSet($this,$appDataRoot);
  $this->setAppDataRoot($aValue);
  haveSet($this,$appDataRoot);
}

#+field(Boolean:isInstallSoftware)
my $isInstallSoftware="isInstallSoftwareKey";
#+sub(Boolean):void:Set isInstallSoftware
sub setIsInstallSoftware{
  my ($this, $aValue) = @_;
  $this->{$isInstallSoftware}=$aValue;
}
#+sub():Boolean:Get isInstallSoftware
sub getIsInstallSoftware{
  my ($this ) = @_;
  return $this->{$isInstallSoftware};
}



#+field(Boolean:IsUninstallSoftware)
my $IsUninstallSoftware="IsUninstallSoftwareKey";
#+sub(Boolean):void:Set IsUninstallSoftware
sub setIsUninstallSoftware{
  my ($this, $aValue) = @_;
  $this->{$IsUninstallSoftware}=$aValue;
}
#+sub():Boolean:Get IsUninstallSoftware
sub getIsUninstallSoftware{
  my ($this ) = @_;
  return $this->{$IsUninstallSoftware};
}

















#+field(String:SDBUser)
my $SDBUser="SDBUserKey";
#+sub(String):void:Set SDBUser
sub setSDBUser{
  my ($this, $aValue) = @_;
  $this->{$SDBUser}=$aValue;
}
#+sub():String:Get SDBUser
sub getSDBUser{
  my ($this ) = @_;
  return $this->{$SDBUser};
}

#+field(String:SDBAGroup)
my $SDBAGroup="SDBAGroupKey";
#+sub(String):void:Set SDBAGroup
sub setSDBAGroup{
  my ($this, $aValue) = @_;
  $this->{$SDBAGroup}=$aValue;
}
#+sub():String:Get SDBAGroup
sub getSDBAGroup{
  my ($this ) = @_;
  return $this->{$SDBAGroup};
}



#+field(SubRef:statusCallback)
my $statusCallback="statusCallbackKey";
#+sub(SubRef):void:Set statusCallback
sub setStatusCallback{
  my ($this, $aValue) = @_;
  $this->{$statusCallback}=$aValue;
}
#+sub():SubRef:Get statusCallback
sub getStatusCallback{
  my ($this ) = @_;
  return $this->{$statusCallback};
}


#+field(SubRef:AsyncCallback)
my $AsyncCallback="AsyncCallbackKey";
#+sub(SubRef):void:Set AsyncCallback
sub setAsyncCallback{
  my ($this, $aValue) = @_;
  $this->{$AsyncCallback}=$aValue;
}
#+sub():SubRef:Get AsyncCallback
sub getAsyncCallback{
  my ($this ) = @_;
  return $this->{$AsyncCallback};
}




#+field(SubRef:LogCallback)
my $LogCallback="LogCallbackKey";
#+sub(SubRef):void:Set LogCallback
sub setLogCallback{
  my ($this, $aValue) = @_;
  $this->{$LogCallback}=$aValue;
}
#+sub():SubRef:Get LogCallback
sub getLogCallback{
  my ($this ) = @_;
  return $this->{$LogCallback};
}




#+field(Callback function:statusListCallback)
my $statusListCallback="statusListCallbackKey";
#+sub(Callback function):void:Set statusListCallback
sub setStatusListCallback{
  my ($this, $aValue) = @_;
  $this->{$statusListCallback}=$aValue;
}
#+sub():Callback function:Get statusListCallback
sub getStatusListCallback{
  my ($this ) = @_;
  return $this->{$statusListCallback};
}







#+field(Path:dependendPath)
my $dependendPath="dependendPathKey";
#+sub(Path):void:Set dependendPath
sub setDependendPath{
  my ($this, $aValue) = @_;
  $this->{$dependendPath}=$aValue;
  my $dbinstance = $this->getDBInstance();
  if ( defined $dbinstance) {
    $dbinstance->setDependendPath($aValue);
  }
}



#+sub():Path:Get dependendPath
sub getDependendPath{
  my ($this ) = @_;
  my $retVal = undef;
  my $dbinstance = $this->getDBInstance();
  if ( defined $dbinstance) {
    $retVal = $dbinstance->getDependendPath();
  } else {
    $retVal = $this->{$dependendPath};
  }
  return $retVal;
}




#+sub(ArrayRef:Lines):void:Configure from an Array of lines
sub configureFromLines($$){
  my ($this,$linesRef) = @_;
  my @lines = @$linesRef;
  my ($section,$key,$value)=(undef,undef,undef);
  foreach my $line (@lines) {
    parseConfigLine 
      ( $line,
        sub{
          $section =$_[0];
        },
        sub {
          $key = $_[0];
          $value = $_[1];
          $this->configureSectionKeyValue($section,$key,$value);
        }
        ,$this
      );
  }
}
#configureFromLines


#ifdef sub:configureSectionKeyValue
#<p>Hier kann das Template Werte aus einer
#  ini-Datei hereingereicht bekommen, ohne die Datei
#  selbst parsen zu mssen.</p>
  #endif
  #+sub(Section:aSection|Key:aKey|Value:aValue):void:Configure from a triple (section,key,value)
  sub configureSectionKeyValue {
    my ($this,$section,$key,$value) = @_;
    my $paramsRef = $this->getParams();
    my $pDesc = $paramsRef->{uc $key};
    if ($section eq "Software" and defined $pDesc) {
      my $setter = @$pDesc[0];
      &$setter($value);
    }
  }



sub writeConfig($$){
  my ($this,$write) = @_;
  my ($sec,$min,$our,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
  $year = 1900 + $year;
  &$write("#############################################################\n");
  &$write("#SDBINST INSTALL TEMPLATE\n");
  &$write("#GENERATED ON: $mday.$mon.$year $our:$min:$sec\n\n");
  &$write("[Template]\n");
  my $aCode = $this->getTemplateCode();
  &$write("TEMPLATECODE $aCode\n");
  &$write("\n[Software]\n");
  my $paramsRef = $this->getParams();
  defaultWriteParams($paramsRef,"Software",$write);
  my $instance = $this->getDBInstance();
  if ( defined $instance) {
    $instance->writeConfig($write);
  }
  &$write("\n");
  my $sqlStudio = $this->getSQLStudio();
  if ( defined $sqlStudio) {
    $sqlStudio->writeConfig($write);
  }
  &$write("\n");
  my $dbmGui = $this->getDBMGui();
  if ( defined $dbmGui) {
    $dbmGui->writeConfig($write);
  }
}
#end writeConfig();

sub createParameterDescription{
  my ($this,$id, $default,$label,$description,$controlType,$longDescription,$minimum,$maximum) =@_;
  my $params = $this->getAllParams();
  my $param = $params->{$id};
  my $setter = $param->[0];
  my $getter = $param->[1];
  my $type = $param->[3];
  my $validator = $param->[5];
  $longDescription = $longDescription ? $longDescription : "No description available";
  my      $paramDesc = {
                        "Default" => $default,
                        "Label" => $label,
                        "Description" => $description,
                        "LongDescription" => $longDescription,
                        "Type" => $type,
                        "Getter" => $getter,
                        "Setter" => $setter,
                        "Validator" => $validator,
                        "ControlType" => $controlType
                       };
  if (defined $minimum) {
    $paramDesc->{"Minimum"}=$minimum;
  }
  if (defined $maximum) {
    $paramDesc->{"Maximum"}=$maximum;
  }
  if ( "labelType" eq $controlType->{"id"}) {
    $paramDesc->{"Getter"} = sub{return $label;};
  }
  return $paramDesc;
}

#+sub(String:paramId):ParamaterDescription:Get a parameter description specified by  parameterId
sub createParameterDescriptionById{
  my ($this,$paramId) = @_;
  my $paramDescList = $this->getParameterTemplateById($paramId);
  my $errorMessage = "Internal error (fatal): Missing parameter template for parameter key \"$paramId\"\n";
  #ifdef SCRIPTING
#  unless(defined $paramDescList){
#    die $errorMessage;
#  }
  #endif
#because SDBINST
  unless(defined $paramDescList){
    $this->statusMessage($errorMessage);
    return undef;
  }
#endbecause
  my $aParamDesc = $this->createParameterDescription(@$paramDescList);
  return $aParamDesc;
}


#FRAMETEMPLATES
sub createFrameTemplates{
  my ($this) = @_;
  my $retVal =  {
                 "TemplateOverViewSection1" =>
                 {
                  "Title" => "General Properties",
                  "FrameId" => "TemplateOverViewSection1",
                  "SummaryExpand" => 1,
                  "Data" => 
                  {
                  },
                  "MetaData" => 
                  {
                   "INSTANCE.DBNAME" ,{"priority" => 0},
                   "INSTANCE.DBMUSER" ,{"priority" => 1},
                   "INSTANCE.DBAUSER" ,{"priority" => 2},
                   "INSTANCE.DBSIZE" ,{"priority" => 3},
                   "INSTANCE.DATAPATH" ,{"priority" => 4},
                   "INSTANCE.LOGPATH" ,{"priority" => 5}
                  }
                 } ,
                 "TemplateOverViewSection1Advanced" =>
                 {
                  "Title" => "General Properties",
                  "FrameId" => "TemplateOverViewSection1Advanced",
                  "SummaryExpand" => 1,
                  "Data" => 
                  {
                  },
                  "MetaData" => 
                  {
                   "INSTANCE.DBNAME" ,{"priority" => 0},
                   "INSTANCE.DBMUSER" ,{"priority" => 1},
                   "INSTANCE.DBAUSER" ,{"priority" => 2},
                   "INSTANCE.DBSIZE" ,{"priority" => 3}
                  }
                 }
                 ,
                 "TemplateOverViewSection2Advanced" =>
                 {
                  "Title" => "Global Settings",
                  "FrameId" => "TemplateOverViewSection2Advanced",
                  "SummaryExpand" => 0,
                  "Data" => 
                  {
                  },
                  "MetaData" => 
                  {
                   "INSTANCE.AUTO_UPDATE_STATISTICS" ,{"priority" => 0},
                   "INSTANCE.AUTORESTART" ,{"priority" => 1},
                   "INSTANCE.AUTO_EXTEND" ,{"priority" => 2},
                   "INSTANCE.LOGGING_MODE" ,{"priority" => 3}
                  }
                 }
                 ,
                 "TemplateOverViewSection2" =>
                 {
                  "Title" => "Assigned System Resources",
                  "FrameId" => "TemplateOverViewSection2",
                  "SummaryExpand" => 1,
                  "Data" => 

                  {
                  },
                  "MetaData" => 
                  {
                   "INSTANCE.DBMEMORY"  => {"priority" => 0},
                   "INSTANCE.CPUCOUNT"  => {"priority" => 1}
                  }
                 } ,
                 "VolumesMinimal" =>
                 {
                  "Title" => "Specify disk drives for database software and database areas",
                  "FrameId" => "VolumesMinimal",
                  "Description" => "Choose the drive of the file location" ,
                  "LongDescription" => "Template \"Minimal resource consumption\" defines a " .
                  "data area of 64 MB and a log area of 32 MB.",
                  "Data" => 
                  {
                  }
                  ,
                  "MetaData" => 
                  {
                   "SWDRIVE"  => {"priority" => 0},
                   "INSTANCE.VOLSTRUCT.SIMPLE"  => {"priority" => 1}
                  }
                 } ,
                 "VolumesMinimalUx" =>
                 {
                  "Title" => "Specify locations of database volumes",
                  "FrameId" => "VolumesMinimalUx",
                  "Description" => "Choose the directory of the file location" ,
                  "LongDescription" => "Template \"Minimal resource consumption\" defines a " .
                  "data area of 64 MB and a log area of 32 MB.",
                  "Data" => 
                  {
                  }
                  ,
                  "MetaData" => 
                  {
                   "SOFTWAREPATH"  => {"priority" => 0},
                   "INSTANCE.VOLSTRUCT.SIMPLE"  => {"priority" => 1}
                  }
                 } ,


                 "VolumesMedium" =>
                 {
                  "Title" => "Specify locations of database volumes",
                  "FrameId" => "VolumesMedium",
                  "Description" => "Choose the drive of the file location" ,
                  "LongDescription" => "",
                  "Data" => 
                  {
                  }
                  ,
                  "MetaData" => 
                  {
                   "INSTANCE.VOLSTRUCT.MEDIUM"  => {"priority" => 0}
                  }
                 } ,
                 "VolumesMediumUx" =>
                 {
                  "Title" => "Specify locations of database volumes",
                  "FrameId" => "VolumesMediumUx",
                  "Description" => "Choose the directory of the file location" ,
                  "LongDescription" => "",
                  "Data" => 
                  {
                  }
                  ,
                  "MetaData" => 
                  {
                   "INSTANCE.VOLSTRUCT.MEDIUM"  => {"priority" => 0}
                  }
                 } ,

                 "SizeMemoryAdvanced" =>
                 {
                  "Title" => "Specify database size and assign system resources",
                  "FrameId" => "SizeMemoryAdvanced",
                  "Description" => "Your system has the following hardware resources: \n\n"
                  .  "\$mem physical memory and \$cpucount.\n\n" . 
                  "Please determine the amount of resources which can be used by the database.",
                  "Data" => 
                  {
                  },
                  "MetaData" => 
                  {
                   "INSTANCE.DBSIZE"  => {"priority" => 0, "dependencies" =>["INSTANCE.DBMEMORY"],"Disabled" => 1},
                   "INSTANCE.DBMEMORY"  => {"priority" => 1},
                   "INSTANCE.CPUCOUNT"  => {"priority" => 2},
                   "INSTANCE.PARALLELCONNECTIONS"  => {"priority" => 3}
                  }
                 } ,

                 "GlobalDBSettings" =>
                 {
                  "Title" => "Specify global database settings",
                  "FrameId" => "GlobalDBSettings",
                  "Description" => "Specifiy global database settings",
                  "Data" => 
                  {
                  },
                  "MetaData" => 
                  {
                   "INSTANCE.AUTORESTART"  => {"priority" => 1 },
                   "INSTANCE.AUTO_EXTEND"  => {"priority" => 2},
                   "INSTANCE.AUTO_UPDATE_STATISTICS"  => {"priority" => 3},
                   "INSTANCE.LOGGINGSTRUCT" => {"priority" => 4}
                  }
                 } ,

                 "GlobalDBSettingsUx" =>
                 {
                  "Title" => "Specify global database settings",
                  "FrameId" => "GlobalDBSettingsUx",
                  "Description" => "Specifiy global database settings",
                  "Data" => 
                  {
                  },
                  "MetaData" => 
                  {
                   "INSTANCE.AUTO_EXTEND"  => {"priority" => 1},
                   "INSTANCE.AUTO_UPDATE_STATISTICS"  => {"priority" => 2},
                   "INSTANCE.LOGGINGSTRUCT" => {"priority" => 3}
                  }
                 } ,


                 "DBNamePasswdDesktop" =>
                 {
                  "Title" => "Specify database name and provide authorization data",
                  "FrameId" => "DBNamePasswdDesktop",
                  "Description" => "Please specify database name and provide authorization data",
                  "Data" => 
                  {
                  },
                  "MetaData" => 
                  {
                   "INSTANCE.DBNAME"  => {"priority" => 0 },
                   "INSTANCE.DBSIZE"  => {"priority" => 1, "Disabled" => 0},
                   "INSTANCE.LOAD_SAMPLE_SCHEMA"  => {"priority" => 2 },
                   "INSTANCE.GLOBAL_USER_STRUCT"  => {"priority" => 3}
                  }
                 }                            
                 ,
                 "DBNamePasswdSimple" =>
                 {
                  "Title" => "Specify installation prefix and  provide database authorization data",
                  "FrameId" => "DBNamePasswdSimple",
                  "Description" => "Please specify database name and a global password",
                  "Data" => 
                  {
                  },
                  "Tables" => 
                  [
                  ],
                  "MetaData" => 
                  {
                   "SOFTWAREPATH"  => {"priority" => 0},
                   "INSTANCE.DBNAME"  => {"priority" => 1 },
                   "INSTANCE.LOAD_SAMPLE_SCHEMA"  => {"priority" => 2 },
                   "INSTANCE.ADMINUSERSTRUCT"  => {"priority" => 3 }
                  }
                 }                            
                };
  $this->initGlobalDBSettings($retVal->{"GlobalDBSettings"});
  $this->initSizeMemoryAdvanced($retVal->{"SizeMemoryAdvanced"});
  $this->initTemplateOverViewSection1Advanced($retVal->{"TemplateOverViewSection1Advanced"});
  $this->initTemplateOverViewSection2Advanced($retVal->{"TemplateOverViewSection2Advanced"});
  $this->initTemplateOverViewSection1($retVal->{"TemplateOverViewSection1"});
  $this->initVolumeFrames($retVal);
  $this->initParamFrame($retVal);
  $this->initVolumesAdvanced($retVal);
  return $retVal;
}                               #createFrameTemplates





sub initGlobalDBSettings {
  my ($this,$frame) = @_;
  my $instance = $this->getDBInstance();
  unless(defined $instance){
    return;
  }
  my $md = $frame->{"MetaData"};
  my $autoSaveLog = $md->{"INSTANCE.AUTO_SAVE_LOG"};
  my $initialBackupDir = $md->{"INSTANCE.INITIAL_BACKUP_DIR"};
  my $logSaveDir = $md->{"INSTANCE.LOG_SAVE_DIR"};
  my $recMode = $instance->getRecoveryMode();
  if( $recMode =~ /^simple$/i){
    $autoSaveLog->{"Disabled"} = 1;
  }
  else{
    delete $autoSaveLog->{"Disabled"};
  }

}


sub initVolumesAdvanced (){
  my ($this,$retVal) = @_;
  my $volumesAdvancedId= "VolumesAdvanced";
  my  $volumesAdvanced = 
    {
     "Title" => "Specify locations of database volumes",
     "FrameId" => "VolumesAdvanced",
     "Description" => "You have configured a database size of \$dbsize Pages. Please specify"
     . " the locations of the data and log volumes."
     ,
     "Data" => 
     {
     },
     "Tables" => 
     [
     ],
     "MetaData" => 
     {
      "INSTANCE.VOLSTRUCT.DATA"  => {"priority" => 1},
      "INSTANCE.VOLSTRUCT.LOG"  => {"priority" => 2}
     }
    };                            
  my $instance = $this->getDBInstance();
  if (defined $instance) {
    my $volumesAdvancedDescription = $volumesAdvanced->{"Description"};
    my $dbsize = $instance->getDBSize();
    $volumesAdvancedDescription =~ s/\$dbsize/$dbsize/;
    $volumesAdvanced->{"Description"}=$volumesAdvancedDescription;
  }
  my $metaData = $volumesAdvanced->{"MetaData"};
#  if(  $^O =~ /mswin/i) {
#  $metaData->{"SWDRIVE"} = {"priority" => 0};
#   }
#   else{
#     $metaData->{"SOFTWAREPATH"} = {"priority" => 0};
#   }


  $retVal->{$volumesAdvancedId} = $volumesAdvanced;
}



sub initParamFrame (){
  my ($this,$retVal) = @_;
  my $paramTemplates = $this->getParameterTemplates();
  my $dbParamFrameId= "TemplateOverViewDbParams";
  my  $dbParamFrame = 
    {
     "Title" => "Internal Database Parameters",
     "FrameId" => "TemplateOverViewDbParams",
     "Data" => 
     {
     },
     "MetaData" => 
     {
     }
    };                            
  my $dbParamMetaData = $dbParamFrame->{"MetaData"};
  my $dbParamCounter=0;
  foreach my $aKey (keys %$paramTemplates) {
    if ( $aKey =~ /^INSTANCE.DBPARAM/) {
      $dbParamMetaData->{$aKey} = {"priority" => $dbParamCounter};
      $dbParamCounter++;
    }
  }
  $retVal->{$dbParamFrameId} = $dbParamFrame;
}

sub initVolumeFrames (){
  my ($this,$retVal) = @_;
  my $paramTemplates = $this->getParameterTemplates();
  my $dataVolumeFrameId= "TemplateOverViewDataVolumes";
  my  $dataVolumeFrame = 
    {
     "Title" => "Data Volumes",
     "FrameId" => "TemplateOverViewDataVolumes",
     "Data" => 
     {
     },
     "MetaData" => 
     {
     }
    };                            
  my $dataVolumeMetaData = $dataVolumeFrame->{"MetaData"};
  my $dataVolumeCounter=0;
  foreach my $aKey (keys %$paramTemplates) {
    if ( $aKey =~ /^INSTANCE.VOLUMEPATH.DATA/) {
      $dataVolumeMetaData->{$aKey} = {"priority" => $dataVolumeCounter};
      $dataVolumeCounter++;
    } elsif ( $aKey =~ /^INSTANCE.VOLUMESIZE.DATA/) {
      $dataVolumeMetaData->{$aKey} = {"priority" => $dataVolumeCounter+1};
    }
  }
  $retVal->{$dataVolumeFrameId} = $dataVolumeFrame;


  my $logVolumeFrameId= "TemplateOverViewLogVolumes";
  my  $logVolumeFrame = 
    {
     "Title" => "Log Volumes",
     "FrameId" => "TemplateOverViewLogVolumes",
     "Data" => 
     {
     },
     "MetaData" => 
     {
     }
    };                            
  my $logVolumeMetaData = $logVolumeFrame->{"MetaData"};
  my $logVolumeCounter=0;
  foreach my $aKey (keys %$paramTemplates) {
    if ( $aKey =~ /^INSTANCE.VOLUMEPATH.LOG/) {
      $logVolumeMetaData->{$aKey} = {"priority" => $logVolumeCounter};
      $logVolumeCounter++;
    } elsif ( $aKey =~ /^INSTANCE.VOLUMESIZE.LOG/) {
      $logVolumeMetaData->{$aKey} = {"priority" => $logVolumeCounter+1};
    }
  }
  $retVal->{$logVolumeFrameId} = $logVolumeFrame;
}



sub initSizeMemoryAdvanced(){
  my ($this,$sizeMemoryAdvanced) = @_;
  my $sizeMemoryAdvancedDescription = $sizeMemoryAdvanced->{"Description"};
  my ($memory,$unit)  = getPhysicalMemoryTotal();
  my $cpucount = getNumOfCPU();
  $cpucount = $cpucount == 1 ? "1 CPU" : "$cpucount CPUs";
  $sizeMemoryAdvancedDescription =~ s/\$mem/$memory  $unit/;
  $sizeMemoryAdvancedDescription =~ s/\$cpucount/$cpucount/;
  $sizeMemoryAdvanced->{"Description"}=$sizeMemoryAdvancedDescription ;
}

sub initTemplateOverViewSection2Advanced (){
  my ($this,$frame) = @_;
  my $instance = $this->getDBInstance();
  if (defined $instance) {
    if(isTrue($instance->getAutoSaveLog())){
      my $md = $frame->{"MetaData"};
      my $counter=5;
      $md->{"INSTANCE.PLANNED_INITIAL_BACKUP_PATH"} ={"priority" => $counter};
      ++$counter;
      $md->{"INSTANCE.PLANNED_LOG_SAVE_PATH"}={"priority" => $counter};
    }
  }
}
sub initTemplateOverViewSection1Advanced (){
  my ($this,$frame) = @_;
  my $instance = $this->getDBInstance();
  if (defined $instance) {
    my $md = $frame->{"MetaData"};
    my $paramTemplates = $this->getParameterTemplates();
    my $counter=4;
    if(isTrue($instance->getLoadSampleSchema())){
      $md->{"TUTORIAL.USER"} = {"priority" => $counter};
      $counter++;
    }
    foreach my $aKey (sort(keys %$paramTemplates)) {
      if ( $aKey =~ /^INSTANCE\.VOLUMEPATH\./) {
        $md->{$aKey} = {"priority" => $counter};
        $counter++;
      }
    }
  }
}

sub initTemplateOverViewSection1(){
  my ($this,$frame) = @_;
  my $instance = $this->getDBInstance();
  if (defined $instance) {
    my $md = $frame->{"MetaData"};
    my $paramTemplates = $this->getParameterTemplates();
    my $counter=3;
    if(isTrue($instance->getLoadSampleSchema())){
      $md->{"TUTORIAL.USER"} = {"priority" => $counter};
      $counter++;
    }
  }
}




#PARAMETERTEMPLATES
sub createParameterTemplates{
  my ($this) = @_;
  my $retVal =  {
                 "INDEPPROG" =>
                 ["INDEPPROG", 
                  "",           #Default
                  "Independend Programs", #Label
                  "",           #Description
                  {
                   "id" => "pathType"
                  }             #Control Type
                  ,
                  "Long description missing" # Long Description
                  #Minimum
                  #Maximum
                 ],

                 "INDEPDATA" =>
                 ["INDEPDATA", 
                  "",
                  "Independend Data",
                  "",
                  {
                   "id" => "pathType"
                  }
                  ,
                  "Long description missing"
                 ],

                 "INSTANCE.DEPENDPATH" =>
                 ["INSTANCE.DEPENDPATH", 
                  "",
                  "Dependend Path",
                  "",
                  {
                   "id" => "pathType"
                  }
                  ,
                  "Long description missing"
                 ],

                 "INSTANCE.LOG_AUTO_OVERWRITE" =>
                 ["INSTANCE.LOG_AUTO_OVERWRITE", 
                  0,
                  "Automatic Log Overwrite",
                  "Automatic Log Overwrite",
                  {
                   "id" => "booleanType"
                  }
                 ] ,

                 "INSTANCE.AUTORESTART" =>
                 ["INSTANCE.AUTORESTART", 
                  "YES",
                  "Automatic Start On System Reboot",
                  "",
                  {
                   "id" => "booleanType"
                  }
                  ,
                  "Long description missing"
                 ] ,

                 "INSTANCE.DBMUSER" =>
                 ["INSTANCE.DBMUSER", 
                  "DBM",
                  "Database Manager Operator",
                  "",
                  {
                   "id" => "stringType"
                  }
                  ,
                  "Long description missing"
                 ] ,

                 "INSTANCE.AUTO_EXTEND" =>
                 ["INSTANCE.AUTO_EXTEND", 
                  "YES",
                  "Automatic Extension of Data Area",
                  "",
                  {
                   "id" => "booleanType"
                  }
                  ,
                  "Long description missing"
                 ] ,

                 "INSTANCE.DBMPASS" =>
                 ["INSTANCE.DBMPASS", 
                  "",
                  "Initial Password",
                  "",
                  {
                   "id" => "passwordType"
                  }
                  ,
                  "Long description missing"
                 ]

                 ,

                 "INSTANCE.DBAUSER" =>
                 ["INSTANCE.DBAUSER", 
                  "DBA",
                  "Database System Administrator",
                  "",
                  {
                   "id" => "stringType"
                  }
                  ,
                  "Long description missing"
                 ]

                 ,

                 "INSTANCE.DBAPASS" =>
                 ["INSTANCE.DBAPASS", 
                  "",
                  "Initial Password",
                  "",
                  {
                   "id" => "passwordType"
                  }
                  ,
                  "Long description missing"
                 ]
                 ,
                 "INSTANCE.DBNAME" =>
                 [
                  "INSTANCE.DBNAME",
                  "MIDB1",
                  "Database Name",
                  "",
                  {
                   "id" => "stringType" 
                  }
                  ,
                  "Long description missing"
                 ]
                 ,
                 "INSTANCE.DBSIZE" =>
                 ["INSTANCE.DBSIZE",
                  400,
                  "Database Size",
                  "",
                  {
                   "id" => "pagesType"
                  }
                  ,
                  "Long description missing"
                 ]
                 ,
                 "INSTANCE.DRIVE" =>
                 ["INSTANCE.DRIVE",
                  "C:",
                  "Drive",
                  "Drive",
                  {
                   "id" => "driveType"
                  }
                  ,
                  "Long description missing"
                 ]

                 ,
                 "INSTANCE.DATADRIVE" =>
                 ["INSTANCE.DATADRIVE",
                  "C:",
                  "Data Area",
                  "Drive For Data Volumes",
                  {
                   "id" => "driveType"
                  }
                  ,
                  "Long description missing"
                 ]

                 ,
                 "INSTANCE.DATAPATH" =>
                 ["INSTANCE.DATAPATH",
                  "C:",
                  "Data Area",
                  "Path For Data Volumes",
                  {
                   "id" => "pathType"
                  }
                  ,
                  "Long description missing"
                 ]

                 ,
                 "INSTANCE.LOGDRIVE" =>
                 ["INSTANCE.LOGDRIVE",
                  "C:",
                  "Log Area",
                  "Drive For Log Volumes",
                  {
                   "id" => "driveType"
                  }
                  ,
                  "Long description missing"
                 ]


                 ,
                 "INSTANCE.LOGPATH" =>
                 ["INSTANCE.LOGPATH",
                  "C:",
                  "Log Area",
                  "Path For Log Volumes",
                  {
                   "id" => "pathType"
                  }
                  ,
                  "Long description missing"
                 ]

                 ,
                 "INSTANCE.DBMEMORY" =>
                 ["INSTANCE.DBMEMORY",
                  0,
                  "Memory",
                  "",
                  {
                   "id" => "sliderType",
                   "unit" => "pages",
                  }
                  ,
                  "Long description missing"
                 ] ,

                 "INSTANCE.CPUCOUNT" =>
                 ["INSTANCE.CPUCOUNT",
                  1,
                  "CPUs",
                  "",
                  {
                   "id" => "spinnerType"
                  }
                  ,
                  "Long description missing"
                 ] ,

                 "INSTANCE.PARALLELCONNECTIONS" =>
                 ["INSTANCE.PARALLELCONNECTIONS",
                  5,
                  "Parallel Database Sessions",
                  "",
                  {
                   "id" => "spinnerType"
                  }
                  ,
                  "Long description missing",
                  1,
                  50000
                 ] ,

                 "INSTANCE.DOMAINPASS" =>
                 ["INSTANCE.DOMAINPASS",
                  "DOMAIN",
                  "Domain user Password",
                  "",
                  {
                   "id" => "passwordType"
                  }
                  ,
                  "Long description missing"
                 ]
                 ,
                 "SWDRIVE" =>
                 ["SWDRIVE", 
                  "C:",
                  "Database Software",
                  "",
                  {
                   "id" => "driveType"
                  }
                  ,
                  "Long description missing"
                 ]
                 ,
                 "SERVERDRIVE" =>
                 ["SERVERDRIVE",
                  "C:",
                  "Server Area",
                  "",
                  {
                   "id" => "driveType"
                  }
                  ,
                  "Long description missing"
                 ]
                 ,
                 "ISSQLSTUDIO" =>
                 ["ISSQLSTUDIO",
                  0,
                  "Install SQLStudio",
                  "",
                  {
                   "id" => "booleanType"  
                  }
                  ,
                  "Long description missing"
                 ]
                 ,
                 "ISDBMGUI" =>
                 ["ISDBMGUI",
                  0,
                  "Install DBM Gui",
                  "",
                  {
                   "id" => "booleanType"  
                  }
                  ,
                  "Long description missing"
                 ],
                 "SOFTWAREPATH" =>
                 ["SOFTWAREPATH",
                  "/opt/sdb",
                  "Installation Prefix",
                  "",
                  {
                   "id" => "pathType" 
                  }
                  ,
                  "Long description missing"
                 ]
                 ,
                 "ISCREATEINSTANCE" =>
                 ["ISCREATEINSTANCE",
                  1,
                  "Create a database instance",
                  "If enabled, a database instance will be created",
                  {
                   "id" => "booleanType"  
                  }
                  ,
                  "Long description missing"
                 ]
                 , 
                 "SERVERPATH" =>
                 [
                  "SERVERPATH",
                  "/opt/sdb",
                  "Server Path",
                  "",
                  {
                   "id" => "pathType"  
                  }
                  ,
                  "Long description missing"
                 ]

                 , 
                 "INSTANCE.DATAVOLS" =>
                 [
                  "INSTANCE.DATAVOLS",
                  1,
                  "Number of Data Volumes",
                  "",
                  {
                   "id" => "stringType"  
                  }
                  ,
                  "Long description missing"
                 ]

                 , 
                 "INSTANCE.LOGVOLS" =>
                 [
                  "INSTANCE.LOGVOLS",
                  1,
                  "Number of Log Volumes",
                  "",
                  {
                   "id" => "stringType"  
                  }
                  ,
                  "Long description missing"
                 ] , 
                 "INSTANCE.GLOBALPASS" =>
                 [
                  "INSTANCE.GLOBALPASS",
                  "",
                  "User Password",
                  "User password",
                  {
                   "id" => "passwordType"  
                  }
                  ,
                  "Long description missing"
                 ] , 


                 "INSTANCE.CONFIRMGLOBALPASS" =>
                 [
                  "INSTANCE.CONFIRMGLOBALPASS",
                  "",
                  "Confirm password",
                  "Confirm password",
                  {
                   "id" => "passwordType"  
                  }
                  ,
                  "Long description missing"
                 ] , 

                 "INSTANCE.CONFIRMDBAPASS" =>
                 [
                  "INSTANCE.CONFIRMDBAPASS",
                  "",
                  "Confirm password (DBA)",
                  "Confirm password (DBA)",
                  {
                   "id" => "passwordType"  
                  }
                  ,
                  "Long description missing"
                 ] , 


                 "INSTANCE.CONFIRMDBMPASS" =>
                 [
                  "INSTANCE.CONFIRMDBMPASS",
                  "",
                  "Confirm password (DBM)",
                  "Confirm password (DBM)",
                  {
                   "id" => "passwordType"  
                  }
                  ,
                  "Long description missing"
                 ] , 


                 "INSTANCE.GLOBALUSER" =>
                 [
                  "INSTANCE.GLOBALUSER",
                  "DBADMIN",
                  "Database System Administrator",
                  "Database System Administrator",
                  {
                   "id" => "stringType"  
                  }
                  ,
                  "Long description missing"
                 ] , 
                 "INSTANCE.SQLUSERNAME" =>
                 [
                  "INSTANCE.SQLUSERNAME",
                  "SSS",
                  "Application user",
                  "Use this User for running your own applicatin or working with SQL Studio",
                  {
                   "id" => "stringType"  
                  }
                  ,
                  "Long description missing"
                 ]

                 , 
                 "INSTANCE.VOLSTRUCT" =>
                 [
                  "INSTANCE.VOLSTRUCT",
                  0,
                  "Database Areas",
                  "Database Areas",
                  {
                   "id" => "volumesType"  
                  }
                  ,
                  "Long description missing"
                 ] , 
                 "INSTANCE.VOLSTRUCT.SIMPLE" =>
                 [
                  "INSTANCE.VOLSTRUCT.SIMPLE",
                  0,
                  "Database Areas",
                  "Database Areas",
                  {
                   "id" => "simplevolumesType"  
                  }
                  ,
                  "Long description missing"
                 ] , 

                 "INSTANCE.VOLSTRUCT.MEDIUM" =>
                 [
                  "INSTANCE.VOLSTRUCT.MEDIUM",
                  0,
                  "Database Areas",
                  "Database Areas",
                  {
                   "id" => "mediumvolumesType"  
                  }
                  ,
                  "Long description missing"
                 ] , 

                 "INSTANCE.VOLSTRUCT.LOG" =>
                 [
                  "INSTANCE.VOLSTRUCT.LOG",
                  0,
                  "Log Volumes",
                  "Log Volumes",
                  {
                   "id" => "logvolumesType"  
                  }
                  ,
                  "Long description missing"
                 ]
                 , 
                 "INSTANCE.VOLSTRUCT.DATA" =>
                 [
                  "INSTANCE.VOLSTRUCT.DATA",
                  0,
                  "Data Volumes",
                  "Data Volumes",
                  {
                   "id" => "datavolumesType"  
                  }
                  ,
                  "Long description missing"
                 ]

                 , 
                 "INSTANCE.ADMINUSERSTRUCT" =>
                 [
                  "INSTANCE.ADMINUSERSTRUCT",
                  0,
                  "Database Users",
                  "Database Users",
                  {
                   "id" => "adminusersType"  
                  }
                  ,
                  "Database System Administrator"
                 ]


                 , 
                 "INSTANCE.LOGGINGSTRUCT" =>
                 [
                  "INSTANCE.LOGGINGSTRUCT",
                  0,
                  "Log Settings",
                  "Log Settings",
                  {
                   "id" => "loggingType"  
                  }
                  ,
                  "Log Settings"
                 ]

                 , 
                 "INSTANCE.GLOBAL_USER_STRUCT" =>
                 [
                  "INSTANCE.GLOBAL_USER_STRUCT",
                  "Database System Administrator",
                  "Database System Administrator",
                  "Database System Administrator",
                  {
                   "id" => "dbuserType"  
                  }
                  ,
                  "User Name"
                 ] , 


                 "TUTORIAL.USER" =>
                 [
                  "TUTORIAL.USER",
                  "MONA,RED",
                  "Tutorial User",
                  "Tutorial User",
                  {
                   "id" => "stringType"  
                  }
                  ,
                  "Tutorial User"
                 ] , 



                 "DBM.DESC" =>
                 [
                  "LABEL",
                  0,
                  "Database Manager Operator",
                  "Database Manager Operator",
                  {
                   "id" => "labelType"  
                  }
                  ,
                  "Database Manager Operator"
                 ] , 

                 "DBA.DESC" =>
                 [
                  "LABEL",
                  0,
                  "Database System Operator",
                  "Database System Operator",
                  {
                   "id" => "labelType"  
                  }
                  ,
                  "Database System Operator"
                 ],

                 "INSTANCE.RECOVERYMODE" =>
                 [
                  "INSTANCE.RECOVERYMODE",
                  "Normal",
                  "Recovery Mode",
                  "Recovery Mode",
                  {
                   "id" => "comboBoxType"  ,
                   "list" => ["Simple", "Normal"]
                  }
                  ,
                  "Recovery Mode"
                 ]
                 ,
                 "INSTANCE.AUTO_SAVE_LOG" =>
                 [
                  "INSTANCE.AUTO_SAVE_LOG",
                  0,
                  "Automatic Log Backup",
                  "Automatic Log Backup",
                  {
                   "id" => "booleanType"  
                  }
                  ,
                  "Automatic Log Backup"
                 ]

                 ,
                 "INSTANCE.INITIAL_BACKUP_DIR" =>
                 [
                  "INSTANCE.INITIAL_BACKUP_DIR",
                  ".",
                  "Directory of Initial Backup",
                  "Directory of Initial Backup",
                  {
                   "id" => "pathType"  
                  }
                  ,
                  "Directory of Initial Backup"
                 ]

                 ,
                 "INSTANCE.LOAD_SAMPLE_SCHEMA" =>
                 [
                  "INSTANCE.LOAD_SAMPLE_SCHEMA",
                  0,
                  "Load Tutorial Data",
                  "Load Tutorial Data",
                  {
                   "id" => "booleanType"  
                  }
                  ,
                  "Load Tutorial Data"
                 ]
                 ,
                 "INSTANCE.LOG_SAVE_DIR" =>
                 [
                  "INSTANCE.LOG_SAVE_DIR",
                  ".",
                  "Directory For Log Backups",
                  "Directory For Log backups",
                  {
                   "id" => "pathType"  
                  }
                  ,
                  "Directory For Log Backups"
                 ]

                 ,
                 "INSTANCE.AUTO_UPDATE_STATISTICS" =>
                 [
                  "INSTANCE.AUTO_UPDATE_STATISTICS",
                  0,
                  "Automatic Statistics Update",
                  "Automatic Statistics Update",
                  {
                   "id" => "booleanType"  
                  }
                  ,
                  "Automatic Statistics Update"
                 ]


                 ,
                 "INSTANCE.LOGGING_MODE" =>
                 [
                  "INSTANCE.LOGGING_MODE",
                  0,
                  "Log Settings",
                  "Log Settings",
                  {
                   "id" => "stringType"  
                  }
                  ,
                  "Log Settings"
                 ]

                 ,
                 "INSTANCE.PLANNED_INITIAL_BACKUP_PATH" =>
                 [
                  "INSTANCE.PLANNED_INITIAL_BACKUP_PATH",
                  0,
                  "Initial Data Backup File",
                  "Initial Data Backup File",
                  {
                   "id" => "pathType"  
                  }
                  ,
                  "Initial Data Backup File"
                 ]

                 ,
                 "INSTANCE.PLANNED_LOG_SAVE_PATH" =>
                 [
                  "INSTANCE.PLANNED_LOG_SAVE_PATH",
                  0,
                  "Log Backups",
                  "Log Backups",
                  {
                   "id" => "pathType"  
                  }
                  ,
                  "Log Backups"
                 ]



                } ;
  my $instance = $this->getDBInstance();
  if ($instance) {


    my $loadSampl = $retVal->{"INSTANCE.LOAD_SAMPLE_SCHEMA"};
    if (defined $loadSampl) {
      $loadSampl->[1]= isTrue($instance->getLoadSampleSchema());
    }



    my $parallelC = $retVal->{"INSTANCE.PARALLELCONNECTIONS"};
    if (defined $parallelC) {
      $parallelC->[1]= $instance->getParallelConnections();
    }

    my $recMode = $retVal->{"INSTANCE.RECOVERYMODE"};
    if (defined $recMode) {
      $recMode->[1]= $instance->getRecoveryMode();
    }
    my $swPath = $retVal->{"SOFTWAREPATH"};
    if (defined $swPath) {
      $swPath->[1]= $this->getSoftwarePath();
    }
    my $dbsize = $retVal->{"INSTANCE.DBSIZE"};
    if (defined $dbsize) {
      $dbsize->[1]= $instance->getDBSize();
    }
    my $sqluser = $retVal->{"INSTANCE.SQLUSERNAME"};
    if (defined $sqluser) {
      $sqluser->[1]= $instance->getSQLU();
    }
    my $dbmemory = $retVal->{"INSTANCE.DBMEMORY"};
    if (defined $dbmemory) {
      $dbmemory->[1]= $instance->proposeMemory();
      $dbmemory->[6]= $instance->getMinimalMemory();
      $dbmemory->[7]= $instance->getMaximalMemory();
    }

    my $cpus = $retVal->{"INSTANCE.CPUCOUNT"};
    if (defined $cpus) {
      $cpus->[1]= $instance->proposeCPU();
      my $realCPU = getNumOfCPU();
      $cpus->[6]= 1;
      $cpus->[7]= $realCPU;
      if ( $realCPU == 1) {
        $cpus->[4]->{"Disabled"}= 1;
      }
    }

    my $datavols = $retVal->{"INSTANCE.DATAVOLS"};
    if (defined $datavols) {
      $datavols->[1]= $instance->getDataVolCount();
    }

    my $logvols = $retVal->{"INSTANCE.LOGVOLS"};
    if (defined $logvols) {
      $logvols->[1]= $instance->getLogVolCount();
    }

    my $logpath = $retVal->{"INSTANCE.LOGPATH"};
    if (defined $logpath) {
      $logpath->[1]= $instance->getLogPath();
    }

    my $datapath = $retVal->{"INSTANCE.DATAPATH"};
    if (defined $datapath) {
      $datapath->[1]= $instance->getDataPath();
    }

    my $params = $instance->getParams();
    while ((my $key,my $param) = each %$params) {
      if ($key =~ /^VOLUMEPATH\.DATA\.(\d+)/) {
        my $paramDescKey = "INSTANCE." .  $key;
        my $number = $1;
        $retVal->{$paramDescKey}=
          $this->createParameterTemplateDynamic
            ($key,$param,"Data volume #$number","Data Volume Path", "No long Description","pathType");
      } elsif ($key =~ /^VOLUMESIZE\.DATA\.(\d+)/) {
        my $paramDescKey = "INSTANCE." .  $key;
        my $number = $1;
        $retVal->{$paramDescKey}=
          $this->createParameterTemplateDynamic
            ($key,$param,"Size #$number","Data Volume Size", "No long Description","pagesType");
      } elsif ($key =~ /^VOLUMEPATH\.LOG\.(\d+)/) {
        my $paramDescKey = "INSTANCE." .  $key;
        my $number = $1;
        $retVal->{$paramDescKey}=
          $this->createParameterTemplateDynamic
            ($key,$param,"Log volume #$number","Log Volume Path", "No long Description","pathType");
      } elsif ($key =~ /^VOLUMESIZE\.LOG\.(\d+)/) {
        my $paramDescKey = "INSTANCE." .  $key;
        my $number = $1;
        $retVal->{$paramDescKey}=
          $this->createParameterTemplateDynamic
            ($key,$param,"Size #$number","Log Volume Size", "No long Description","pagesType");
      } elsif ($key =~ /^VOLUMENAME\.(LOG|DATA)\.(\d+)/) {
        my $paramDescKey = "INSTANCE." .  $key;
        my $number = $1;
        $retVal->{$paramDescKey}=
          $this->createParameterTemplateDynamic
            ($key,$param,"Volume name $number","Volume Name", "No long Description","stringType");
      } elsif ($key =~ /^DBPARAM\.(\w+)/) {
        my $paramDescKey = "INSTANCE." .  $key;
        my $paramName = $1;
        $retVal->{$paramDescKey}=
          $this->createParameterTemplateDynamic
            ($key,$param,$paramName,$paramName, "No long Description","stringType");
      }
    }
  }
  return $retVal;
}                               #createParameterTemplates

sub createParameterTemplateDynamic{
  my ($this,$key,$param,$label,$shortDesc,$longDesc,$type,$minimum,$maximum) = @_;
  my $paramDescKey = "INSTANCE." .  $key;
  my $getter = $param->[1];
  return [
          $paramDescKey,        #Id
          &$getter(),           #Default
          $label,               #Label
          $shortDesc,           #Short Description
          {
           "id" => $type
          }                     # Control Type
          ,
          $longDesc             # Long Description
          ,
          $minimum,
          ,
          $maximum
         ]
}







sub getParameterTemplates{
  my ($this) = @_;
  my  $params = $this->createParameterTemplates();
  return $params;
}

#+sub(String:id):Frame:Get a parameter template identified by its id
sub getParameterTemplateById{
  my ($this,$id) = @_;
  my $templates = $this->getParameterTemplates();
  return $templates->{$id};
}



sub createTemplateOverView{
  my ($this) = @_;
  my $retVal = undef;
  if ("templ_server_advanced" eq $this->getTemplateCode()) {
    $retVal=
      [
       $this->createFrameDescription("TemplateOverViewSection1Advanced"),
       $this->createFrameDescription("TemplateOverViewSection2Advanced"),
       $this->createFrameDescription("TemplateOverViewSection2") #,
       #       $this->createFrameDescription("TemplateOverViewDataVolumes"),
       #          $this->createFrameDescription("TemplateOverViewLogVolumes")
      ];
  } else {
    $retVal=
      [
       $this->createFrameDescription("TemplateOverViewSection1"),
       $this->createFrameDescription("TemplateOverViewSection2") #,
       #          $this->createFrameDescription("TemplateOverViewDataVolumes"),
       #          $this->createFrameDescription("TemplateOverViewLogVolumes")
      ];
  }
  return $retVal;
}



sub getFrameTemplates{
  my ($this) = @_;
  my     $frames = $this->createFrameTemplates();
  return $frames;
}

#+sub(String:id):Frame:Get a frame template identified by its id
sub getFrameTemplateById{
  my ($this,$id) = @_;
  my $templates = $this->getFrameTemplates();
  return $templates->{$id};
}



sub createParameterData{
  my ($this,$id,$frameDesc) = @_;
  my $data = $frameDesc->{"Data"};
  my $metaData = $frameDesc->{"MetaData"};
  foreach my $key (keys %$metaData) {
    my $aParamDesc = $this->createParameterDescriptionById($key);
    my $aControlMeta = $metaData->{$key};
    foreach my $aKey (keys  %$aControlMeta) {
      $aParamDesc->{"ControlType"}->{$aKey}=$aControlMeta->{$aKey};
    }
    $data->{$key}=$aParamDesc;
  }
}

sub createTableData{
  my ($this,$id,$frameDesc) = @_;
  my $metaTables = $frameDesc->{"MetaTables"};
  my $tables = $frameDesc->{"Tables"};
  my $tableCounter=0;
  foreach my $tableTemplate (@$metaTables) {
    my $rows = $tableTemplate->{"rows"};
    my $rowCount=0;
    my $aTable = {};
    foreach my $row (@$rows) {
      my $cellCount=0;
      foreach my $aCol (@$row) {
        my $aParamKey = $aCol->[0];
        my $aControlMeta = $aCol->[1];
        my $aParamDesc = $this->createParameterDescriptionById($aParamKey);
        foreach my $aKey (keys  %$aControlMeta) {
          $aParamDesc->{"ControlType"}->{$aKey}=$aControlMeta->{$aKey};
        }
        $row->[$cellCount]=$aParamDesc;
        $cellCount++;
      }
      $rowCount++;
    }
    $tables->[$tableCounter]=$tableTemplate;
    $tableCounter++;
  }                             #foreach table
}                               #sub createTableData


sub createFrameDescription{
  my ($this,$id) = @_;
  my $frameDesc = $this->getFrameTemplateById($id);
  $this->createParameterData($id,$frameDesc);
  $this->createTableData($id,$frameDesc);
  return $frameDesc;
}                               #createFrameDescription






1;



# Local Variables: ***
# perl-dbg-flags:"-w      -I../../../../../../wrk/script -I../../../sdbrun  " ***
# sdbinst-defines: "-define SCRIPTING=YES -define PERL5.6.1=YES" ***
# sdbinst-testfile: "TemplateTest.pl" ***
# sdbinst-script-directory:  "sys/wrk/script/SAPDB/Install/Templates" ***
# End: ***

