package SAPDB::Install::Templates2::CmdLine;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Templates2::Properties;
use SAPDB::Install::Templates2::InstTemplate;
use strict;

our @ISA;
if($SCRIPTING){
  require Exporter;
  push @ISA,qw(Exporter);
  require Getopt::Long;
  import Getopt::Long;
#  Getopt::Long::Configure  qw(bundling bundling_override);
}
else{
  require SDB::Common::Exporter;
  push @ISA,qw(SDB::Common::Exporter);
  require SDB::Common::Getopt::Long;
  import SDB::Common::Getopt::Long;
}
push  @ISA, qw( SAPDB::Install::Templates2::Properties);
our @EXPORT = qw (newCmdLine show);



#ATTRIBUTES:

my ($file,$template,@params,);

my $PARAMREGEXP = '\s*(\w+)\s*=\s*(.+)\s*';


our %OPTIONS = ('f=s'=>\$file,  'p=s' =>\@params,'t=s' =>\$template,
                'template=s'=>\$template  );


my $data = << "DATA";
<ParameterDefinitions>
  <Parameter id="usage" />
  <Parameter id="iParams" />
  <Parameter id="dBParams" />
  <Parameter id="templateDescriptions" />
  <Parameter id="template" />
  <Parameter id="templateObject" />
</ParameterDefinitions>
<ParameterData>
  <ParameterValue id="usage">
===============================================
COMMANDLINE OPTIONS FOR INSTALLATION TEMPLATES:
===============================================

-t TEMPLATEID  -h
        Show template specific help

-t TEMPLATEID  -dump_configfile
        Write a sample configuration file to standard output

-t TEMPLATEID  [-f FILE] [-p KEY=VALUE]...
        Start the template installation.

        FILE:      Read template configuration  file from File FILE

        KEY=VALUE: Specify parameter name and  value separated by \"=\".
                   Values containing spaces must be quoted. 
                   This option can be used multiple times. For every parameter
                   you have to use one -p option.

                   Example:
                   -p keyA=value -p keyB=\"second value\"

                   Possible  keys depend on the selected template. See
                   template specific help.


==================================
Supported templates and there IDs:
==================================

\${TEMPLATEDESCRIPTIONS}
  </ParameterValue>
</ParameterData>
DATA
my @data = split(/\n/,$data);
#OVERWRITTEN METHODS:
sub GetXmlData{
  return \@data;
}

#overwrite standard line parsing 
sub parseLine{
  my ($this,$aLine) = @_;
  if ( $aLine =~ /^\s*TEMPLATE\s+(\w+)\s*$/i) {
    #set the template id
    $this->setTemplate($1);
  } else {
    my $section = $this->{'section'};
    if($section and $section =~ /Database Software/i){
      my $iparams = $this->getIParams();
      push @$iparams,($aLine);
    }
    elsif($section and $section =~ /Database Instance/i){
      my $dbparams = $this->getDBParams();
      push @$dbparams,($aLine);
    }
  }
}                               #parseLine




#PARAMETERS:
#TEMPLATEDESCRIPTIONS:
sub defaultTemplateDescriptions{
  my $retVal ='';
  my $list = SAPDB::Install::Templates2::TemplateList::createTemplateList('batch');
  foreach my $section (@$list) {
    my $label = $section->{'label'};
#    $retVal .=$label . "\n";
    my $data = $section->{'data'};
    foreach my $tmpl (@$data) {
      my $code = $tmpl->{'TemplateCode'};
      my $space = 20 - length($code);
      my $indent = " " x $space;
      $retVal .= "  ".$code.$indent. $tmpl->{'ShortDescription'}."\n";
    }
  }
  return $retVal;
}

sub initializeIParams{
  return ["[Database Software]"];
}

sub initializeDBParams{
  return ["[Database Instance]"];
}


sub validateTemplate{
  my ($val) = @_;
  $val or return "Missing parameter 'TEMPLATE'";
  my $list = SAPDB::Install::Templates2::TemplateList::createTemplateList('batch');
  my $retVal ='';
  foreach my $section (@$list) {
    my $data = $section->{'data'};
    foreach my $tmpl (@$data) {
      my $aCode = $tmpl->{'TemplateCode'};
      $aCode eq $val and return 0;
    }
  }
  return "Invalid value '$val'";
}










sub initOptions3{
  my ($this,$atemplate,$afile,$aparams,$novalidate)=@_;
  my $retVal;

  #if config file specified, read it
  $afile and $retVal = $this->parseFile($afile)
    and return "Error parsing '$file':\n".$retVal;

  #if template specified on the command line, set it
  $atemplate and $this->setTemplate($atemplate);
  my $ti = $this->getTemplate();

  #check the template id 
  $ti or return "You have not specified the template";
  $retVal = validateTemplate($ti) and return "Unknown template '$ti'\nUse options -t -h for help!";
  

  #template id is correct, initialize the template object:
  my $templ = SAPDB::Install::Templates2::InstTemplate::new(undef,$ti);

  my $db = $templ->getDBInstance();
  #separate software parameters from database instance parameters:
  my ($adbparams,$aiparams) = ([],[]);
  foreach my $line (@$aparams) {
#    print "Line: $line\n";
    my ($key,$value) ; 
    ($key,$value) = $line =~ /^\s*"?\s*(\w+)\s*=?\s*(.*)\s*"?\s*$/
      or return "Invalid parameter format: $line";
    if ($templ->ExistsKey($key)) {
      push @$aiparams, ($line);
    } elsif ($db->ExistsKey($key)) {
      push @$adbparams, ($line);
    } else {
      return "Unknown parameter '$key'";
    }
  }

  #collect and check the software specific parameters: 
  foreach my $line (@$aiparams) {
    $retVal = $templ->ValidateLine($line) and return $retVal;
    push @{$this->getIParams()},$line;
  }
  
  
  #Set all software parameters from command line and config file.
  $templ->parseLineArray($this->getIParams());

  #collect and check the database specific parameters specified on the command line:
  foreach my $line (@$adbparams) {
    $retVal = $templ->getDBInstance()->ValidateLine($line) and return $retVal;
    push @{$this->getDBParams()},($line);
  }
  #Set all database parameters from command line and config file.
  $templ->getDBInstance()->parseLineArray($this->getDBParams());

  unless(defined $novalidate and 'novalidate' eq $novalidate){
    #Validate the installation parameters and the database parameters:
    $retVal  = $templ->Validate() and return $retVal;
  }

  #Finally set the template object.
  $this->setTemplateObject($templ);
  return 0;
}#initOptions3





sub main {
  GetOptions(%OPTIONS);
  my $action = $ARGV[0];
  my $retVal =0;
  unless($action){
   print "Usage: -t <TEMPLATEID> show|dump|validate|debug\n";
   return;
  }
  my $cmd = newCmdLine();
  $template or die "Missing parameter: -t\n";
  $retVal = $cmd->initOptions3($template,$file,\@params,'novalidate') and die $retVal;
  my $templ = $cmd->getTemplateObject();

  if( "show" eq $action){
    my $txt ='';
    showInfo($templ,\$txt);
    print $txt;
  }
  elsif( "dump" eq $action){
    my $txt ='';
    $retVal = $templ->GetConfigParameters(\$txt) and die $retVal;
    print $txt;
  }
  elsif( "validate" eq $action){
    my $txt ='';
    $txt = $templ->Validate() ; 
    $txt or $txt='';
    $txt or $txt ="Validation successfull\n";
    print $txt;
  }
  elsif( "debug" eq $action){
    my $indepProg = $templ->getIndepProg();
    print "IndepProg: ".$indepProg . "\n";
    my $indepData = $templ->getIndepData();
    print "IndepData: ".$indepData . "\n";
    my $gr = $templ->getOwnerGroup();
    print "Group: ".$gr . "\n";
    my $u = $templ->getOwnerName();
    print "User: ".$u . "\n";
    my $archiveDir = $templ->getArchiveDir();
    print "ArchiveDir: ".$archiveDir . "\n";
    my $dbVolumes = $templ->getDBInstance()->getDBVolume();
    print "Volumes:\n";
    foreach(@$dbVolumes){
      print $_ . "\n";
    }
    my $dbAdmin = $templ->getDBInstance()->getDBAdmin();
    print "dbadmin: $dbAdmin\n";
    my $dbm = $templ->getDBInstance()->getDBManager();
    print "dbmanager: $dbm\n";
    my $autoRestart = $templ->getDBInstance()->getAutoRestart();
    print "autorestart: $autoRestart\n";
    my $logStrategy = $templ->getDBInstance()->getLogStrategy();
    print "logStrategy: $logStrategy\n";
  }
  elsif( "create" eq $action){
    print "Creating instance ...\n";
    push @{$templ->GetListeners('log')},(sub {print($_[0]);});
    push @{$templ->GetListeners('warning')},(sub {print( "WARNING: " . $_[0]);});
    push @{$templ->GetListeners('debug')},(sub{shift;print(@_);});
    push @{$templ->GetListeners('async_status')},(sub{shift;print(@_);});
    $retVal =    $templ->getDBInstance()->createInstance();
    $retVal and print $retVal . "\n";
  }
}#main










#FUNCTIONS:

#constructor:
sub newCmdLine(){
  my $this = {};
  bless $this ;
  $this->SUPER::initData();
  return  $this;
}

#usage output to stdout:
sub usage{
  my $this = shift;
  print ($this->getUsage());
  return 0;
}


sub templateUsage{
  my ($this,$id)  = @_;
  my $info ;
  my $retVal;
  $retVal = show($id,\$info) and return $retVal;
  print "\n";
  print ($info);
  return 0;
}

sub show{
  my ($tn,$info) = @_;
  my $retVal = validateTemplate($tn) and return "Unknown template '$tn'\nUse option -h to list all templates!";
  my $templ = SAPDB::Install::Templates2::InstTemplate::new(undef,$tn);
  return showInfo($templ,$info);
}


sub showInfo{
  my ($templ,$info) = @_;
  my $instance = $templ->getDBInstance();
  my $tn = $templ->getTemplateID();
  my $descr = SAPDB::Install::Templates2::TemplateList::GetTemplateDescription($tn);
  chomp $descr;
  
  $$info = '';
  $$info .= "==========================================================================\n";
  $$info .= "TEMPLATE ".uc $tn . " \n";
  $$info .= "($descr)\n";
  $$info .= "==========================================================================\n";
  $$info .= "\n";
  $$info .= "GLOBAL DATABASE SETTINGS:\n";
  $$info .= "=========================\n";
  my $isAutoStatistics = $instance->getAutoUpdateStatistics() ? 'YES' : 'NO';
  my $isAutoRestart = $instance->getAutoRestart() ? 'YES' : 'NO';
  my $isAutoVolumes = $instance->getAutoExtend() ? 'YES' : 'NO';
  my $logStrategy =  $instance->getLogStrategy();
  my $defaultDBA = $instance->GetDefault('dBAdmin');
  my $defaultDBM = $instance->GetDefault('dBManager');
  my $logStrategyString ;
  if('AUTO' eq $logStrategy){
    $logStrategyString = "auto save log"
  }
  elsif('NONE' eq $logStrategy){
    $logStrategyString = "auto overwrite log"
  }
  elsif('MANUAL' eq $logStrategy){
    $logStrategyString = "log must be cleared manually"
  }
  my $indepProg = $templ->GetDefaultResolved('indepProg');
  my $indepData = $templ->GetDefaultResolved('indepData');
  my $depPath = $instance->GetDefaultResolved('dependendPath');
  my $dbname = $instance->getDBName();

  $$info .= "Instance Name:                    ".  $dbname."\n";
  $$info .= "Instance        (local default):  ".  $depPath."\n";
  $$info .= "Indep. Programs (local default):  ".  $indepProg."\n";
  $$info .= "Indep. Data     (local default):  ".  $indepData."\n";
  $$info .= "Automatic statistics update:      ".$isAutoStatistics."\n";
  $$info .= "Automatic start on system reboot: ".$isAutoRestart."\n";
  $$info .= "Automatic volume extension:       ".$isAutoVolumes."\n";
  $$info .= "Log strategy:                     ".$logStrategyString."\n";
  $$info .= "Database Administrator (default): ".$defaultDBA."\n";
  $$info .= "Database Manager (default):       ".$defaultDBM."\n";
  $$info .= "\n";

  #show Volumes;
  my $ps = $instance->getDBParams();
  if (keys %$ps) {
    $$info .= "\n";
    $$info .= "INSTANCE PARAMETERS:\n";
    $$info .= "====================\n";
    foreach my $key (keys %$ps) {
      my $val = $ps->{$key};
      $$info .= $key."\t\t".$val."\n";
    }
  }

  my $vols = $instance->getDBVolume(undef,'no subst');
  if (@$vols) {
    $$info .= "\n";
    $$info .= "VOLUMES:\n";
    $$info .= "========\n";
    foreach my $aVol (@$vols) {
      my ($num,$art,$path,$type,$size) = split(/;/,$aVol);
      my $ss;
      if( $size =~ /^\s*\d+\s*$/){
        my ($s,$u) = readableUnit($size,'page');
        $ss = "$s $u"; 
      }
      else{
        $ss = "$size";
      }
      #print "Pages: $size\n";
      $$info .= "$art $num:\n\tSize: $ss\n\tPath: $path\n";
    }
  }
  $$info .= "\n";
  $$info .= "CONFIGURABLE SOFTWARE PARAMETERS:\n";
  $$info .= "=================================\n";
  $$info .= $templ->Describe();
  $$info .= "\n";
  $$info .= "CONFIGURABLE DATABASE PARAMETERS:\n";
  $$info .= "==================================\n";
  $$info .= $instance->Describe();
  return 0;
}











sub ConfigurationLines{
  my ($this,$txt) = @_;
  $$txt = '';
  my $retVal;
  my $tpl = $this->getTemplateObject();
  $tpl or return "Cannot generated configuration: Template not initialized";
  my $atxt ;
  $retVal = $tpl->GetConfigParameters(\$atxt) and return $retVal;
  $$txt = $atxt;
  return 0;
}

1;

