package SAPDB::Install::Templates2::Properties;
use strict;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Templates2::Util;
our @ISA;
if($SCRIPTING){
  require Exporter;
  push @ISA,qw(Exporter);
}
else{
  require SDB::Common::Exporter;
  push @ISA,qw(SDB::Common::Exporter);
}
our @EXPORT = qw( MYAUTOLOAD);

my  $INVALID = "SAPDB::Install::Properties::Invalid";


my   $LINEREGEXP='^\s*(\w+)\s*[\s=]\s*(.*\S|.*)\s*$';



sub AUTOLOAD {
  our $AUTOLOAD;
  return MYAUTOLOAD($AUTOLOAD,@_);
}

sub MYAUTOLOAD {
  my $MYAUTOLOAD = shift;
  my $this = shift;
  return if $MYAUTOLOAD =~ /::DESTROY$/;
  my $retVal =  Setter($this,$MYAUTOLOAD,@_);
  if (not defined $retVal or $INVALID ne $retVal) {
    return $retVal;
  }
  $retVal =  Getter($this,$MYAUTOLOAD,@_);
  if (not defined $retVal or $INVALID ne $retVal) {
    return $retVal;
  }
  $retVal =  Validator($this,$MYAUTOLOAD,@_);
  if (not defined $retVal or $INVALID ne $retVal) {
    return $retVal;
  }
  if($retVal and $INVALID eq $retVal){
    dieWithStack("Error in AUTOLOAD of  $MYAUTOLOAD");
  }
  else{
    dieWithStack("No such method: $MYAUTOLOAD");
  }
}


sub parseDataSize{
  my ($this,$key,$value) = @_;
  my $r = $value;
  if($value =~ /^\s*\d+\s*$/){
    $r = $value;
  }
  elsif($value =~ /^\s*(\d+)\s*kb\s*$/i){
    $r = sprintf "%d",($1  / 8) ;
  }
  elsif($value =~ /^\s*(\d+)\s*mb\s*$/i){
    $r = sprintf "%d",($1  * 128) ;
  }
  elsif($value =~ /^\s*(\d+)\s*gb\s*$/i){
    $r = sprintf "%d",($1  * 1024 * 128) ;
  }
  return $r;
}

my  %PARSERS = (
                'BOOLEAN' =>sub {
                  my ($this,$key,$value) = @_;
                  return isTrue($value); },
                'DATASIZE' =>\&parseDataSize
                );

my  %FORMATTERS = (
                'BOOLEAN' =>sub {my ($this,$key,$value) = @_;
                                 return  (uc $key) . "=" .  (isTrue($value) ? 'TRUE' : 'FALSE');}
                );


#set initial values, if there are any
sub initData{
  my ($this,$file) = @_;
  exists $this->{'KEYS'} or $this->{'KEYS'}={};
  #Get keys from overwritten method 'mykeys'
  my $myKeys = $this->myKeys();
  #Get keys that are specified in XML-Format
  $this->handleKeyDefinitions($myKeys);
  #Put the keys into the object
  $this->MergeKeys($myKeys);

  ################################################################################
  #Now handle initialization. Priority ist crucial here ...
  #Configuration file overwrites perl initialization overwrites XML-Initialization
  ################################################################################

  #Set the initial values specified in XML-Format
  $this->handleInitialValues();

  #Set initial values specified as perl code:
  my $keys = $this->{'KEYS'};
  foreach my $aKey (keys %$keys){
    my $initValue = $this->GetInitial($aKey);
    unless(defined $initValue and $INVALID eq $initValue){
      $this->Set($aKey,$initValue);
    }
  }

  #Set initial values specified in config file:
  my $retVal=0;
  $file and
    $retVal = $this->parseFile($file)
      and return Logit("Configuration file '$file' could not be read:\n".$retVal,undef);
}#initData




my $listeners = 'SAPDB::Install::Templates2::Properties::Listeners';
sub GetListeners{
  my ($this,$key)  = @_;
  $this->{$listeners} or $this->{$listeners}={};
  my $list = $this->{$listeners};
  $list->{$key} or $list->{$key}=[];
  return $list->{$key};
}


sub Setter{
  my ($this,$setter,$newVal) = @_;
  unless($setter =~/^.*::set(\w+)$/){
    return $INVALID;
  }
  my $aName = $1;
  my $key = $this->GetKey($aName);
  unless($key){
    return Logit("ERROR: Cannot set value for  key '$aName':",$INVALID);
  }
  return $this->Set($key,$newVal);
}
sub Set{
  my ($this,$key,$newVal,$noCan) = @_;
  my $setFunc = $this->can('set'.(ucfirst $key));
  if(defined $setFunc and not $noCan){
    &$setFunc($this,$newVal);
  }
  else{
    $this->{$key}=$newVal;
  }
  return $newVal;
}



sub Getter{
  my ($this,$getter) = @_;
  my $retVal=undef;
  unless($getter =~/^.*::get(\w+)$/){
    return $INVALID;
  }
  my $aName = $1;
  my $key = $this->GetKey($aName);
  $key or return Logit("ERROR: Cannot get value for unknown key '$aName':",$INVALID);
  return $this->Get($key);
}

sub Get{
  my ($this,$key,$noCan,$noSubst) = @_;
  my $retVal ;
  my $getFunc = $this->can('get'.(ucfirst $key));
  #Get value from getterFunction
  if(defined $getFunc and not $noCan){
    $retVal = &$getFunc($this);
   }
   elsif(exists  $this->{$key}){
    $retVal = $this->resolveValue($key,$this->{$key},$noSubst);
  }
  else{
    $retVal = undef;
     my $aDefault = $this->GetDefaultResolved($key,$noSubst);
     unless($INVALID eq $aDefault){
       $retVal = $aDefault;
     }
  }
  return $retVal;
}




sub hasBeenSet{
  my ($this,$key) = @_;
  exists($this->{$key});
}




sub resolveValue{
  my ($this,$key,$retVal,$noSubst) = @_;
  my $isRef = ref $retVal;
  if ($isRef) {
    if ( 'CODE' eq $isRef) {
      $retVal = &$retVal($this,$key);
    }
  } elsif (defined $retVal and not $noSubst) {
    $retVal = $this->recursiveSubstitute($retVal);
    my $type = $this->GetType($key);
    $type and $type eq 'PATH' and $retVal = slashFix($retVal);
  }
  return $retVal;
}



sub recursiveSubstitute{
  my ($this,$val) = @_;
  my $retVal = $val;
  ref $retVal and return $retVal;
#  Debugit(4,"recursiveSubstitute: Performing replacements in  '$val'");
  while ( $retVal =~ /\${(\w+)}/) {
    my $aVar = $1;
    my $aKey = $this->GetKeyIgnoreCase($aVar);
    unless($aKey){
      dieWithStack("ERROR: Cannot substitute value for  key '$aVar':");
    }
    my $aVal = $this->Get($aKey);
    $aVal = defined $aVal ? $aVal : "";
#    Debugit(4,"recursiveSubstitute: Replace '$aVar' by '$aVal'");
    $retVal =~ s/\${$aVar}/$aVal/g;
  }
  return $retVal;
}




sub parseLine{
  my ($this,$aLine) = @_;
  if( $aLine =~ /$LINEREGEXP/){
    $this->InternalSet($1,$2);
  }
  else{
  }
}

sub ValidateLine{
  my ($this,$aLine) = @_;
  if( $aLine =~ /$LINEREGEXP/){
    my $aaKey = $1;
    my $aVal =$2;
    my $aKey = $this->GetKeyIgnoreCase($aaKey);
    $aKey or return "Parameter '$aaKey' is unknown";
  }
  else{
    return 0;
  }
  return 0;
}

sub ValidateLineArray{
  my ($this,$aArray) = @_;
  my $retVal=0;
  foreach my $line (@$aArray){
    $retVal = $this->ValidateLine($line) and return $retVal;
  }
  return 0;
}



sub InternalSet{
  my ($this,$aaKey,$aValue) = @_;
  my $aKey = $this->GetKeyIgnoreCase($aaKey);
  $aKey or return Logit("Cannot set unknown parameter '$aaKey'",undef);
  my $parser = $this->GetParser($aKey);
  $parser and $aValue = &$parser($this,$aKey,$aValue);
  $this->Set($aKey,$aValue);
}

sub parseFile{
  my ($this,$aFile) = @_;
  my $retVal = 0;
  $aFile or return "No input file specified!\n";
  open INFILE , "<$aFile" or return $!;
  $this->parseStream(*INFILE);
  close INFILE;
  return $retVal
}

sub parseStream{
  my ($this,$stream) = @_;
  my @lines = <$stream>;
  $this->parseLineArray(\@lines);
}

sub parseString{
  my ($this,$string) = @_;
  my @lines = split /\n/,$string;
  $this->parseLineArray(\@lines);
}




sub parseLineArray{
  my ($this,$lineArray) = @_;
  my $aLine = '';
  foreach (@$lineArray) {
    /^\s*#/ and next;
    /^\s*\[([^\]]+)\]\s*$/ and $this->{'section'}=$1;
    chomp;
    s/\r$// unless $^O =~ /mswin/i;
    $aLine .= $_;
    if (/\\$/) {
      $aLine =~ s/\\$//;
    } else { 
      $this->parseLine($aLine);
      $aLine='';
    }
  }
  $aLine and $this->parseLine($aLine);
  delete $this->{'section'};
}
sub GetLineForKey{
  my ($this,$theKey) = @_;
  my $value = $this->resolveValue($theKey,$this->{$theKey},"noSubstitute");
  return $this->GetLineForKeyValue($theKey,$value);
}
sub GetLineForKeyValue{
  my ($this,$theKey,$value) = @_;
  my $definition = $this->{'KEYS'}->{$theKey};
  my $retVal = undef;
  if( $definition ){
    my $formatter = $this->GetFormatter($theKey);
    if( $formatter){
      $retVal = &$formatter($this,$theKey,$value);
    }
    else{
      $value = defined $value?$value : "";
      $retVal = (uc $theKey) . "=" . $value . "\n";
    }
  }
  return $retVal;
}

#Get the lines for a configuration file
sub GetConfigurationLines{
  my $this =  shift;
  my $txt='';
  my $keys = $this->{'KEYS'};
#  print "keys is $this\n";
  foreach my $key (keys %$keys) {
    $this->IsPersistent($key) or next;
    $txt .= "\n";
    $txt .=  $this->GetDescription($key,"# ");
    my $lines = $this->GetConfigurationLinesForKey($key);
    if($lines){
      $txt .= $lines ;
    }
    else{
      $txt .= "# Value has no default. Edit here to set value:\n# ".uc $key."\n";
    }
  }
  return $txt;
}





#Get the lines that define a key in a configuration file
sub GetConfigurationLinesForKey{
  my ($this,$key) = @_;
    my $lines = '';
    if($this->hasBeenSet($key)){
      my $val =  $this->Get($key);
       $lines = $this->GetLineForKey($key);
    }
    elsif($this->hasDefault($key)){
      my $default = $this->GetDefault($key);
      unless(defined $default and $default eq $INVALID){
        $lines = $this->GetLineForKeyValue($key,$default);
      }
    }
  return $lines;
}





sub GetLines{
  my ($this) = @_;
  my  @retVal;
  my $keys = $this->{'KEYS'};
  foreach my $aKey (keys %$keys){
    push @retVal, ($this->GetLineForKey($aKey));
  }
  return @retVal;
}
sub saveToFile{
  my ($this,$file) = @_;
  my $retVal=0;
  my @lines = $this->GetLines();
  open OUTFILE , ">$file" or return $!;
  print OUTFILE @lines;
  close OUTFILE;
  return $retVal;
}


#Validator liefert 0 bei Erfolg, ungleich null bei Fehler
sub Validator{
  my ($this,$validator,$newVal) = @_;
  unless($validator =~/^.*::validate(\w+)$/){
    return $INVALID;
  }
  my $key = $this->GetKey($1);
  return $this->ValidateKey($newVal,$key);
}


#Validator liefert 0 bei Erfolg, ungleich null bei Fehler
sub ValidateKey{
  my ($this,$newVal,$key) = @_;
  my $retVal=0;
  my $aVal = $this->GetValidator($key);
  if ($aVal) {
    my $type =  ref $aVal;
    if ('CODE' eq $type) {
      $retVal = &$aVal($this,$newVal,$key);
    } else {
      if (not defined $newVal) {
        $retVal =  0;
      } elsif ($newVal =~ /$aVal/) {
        $retVal=0;
      } else {
        $retVal =  "Invalid value '$newVal' for property '$key'";
      }
    }
  }
#  print "Validating $newVal against $aVal\n";
#  print "Returning $retVal\n";
  return $retVal;
}                               #validateKey

sub GetKey{
  my ($this,$val) = @_;
  my $key = lcfirst $val;
  my $retVal = undef;
  if (exists $this->{'KEYS'}->{$key}) {
    $retVal = $key;
  }
  return $retVal;
}

sub GetKeyIgnoreCase{
  my ($this,$val) = @_;
  my $key = lc $val;
  my $retVal = undef;
  foreach(keys %{$this->{'KEYS'}}){
    if(lc $_ eq $key){
      $retVal = $_;
      last;
    }
  }
  return $retVal;
}


sub IsPersistent{
  my ($this,$key) = @_;
  my $aVal =  $this->GetMetaProperty($key,'PERSISTENT');
  return isTrue($aVal);
}

sub GetDescription{
  my ($this,$key,$indent) = @_;
  my $descr =  $this->GetMetaProperty($key,'DESCRIPTION');
  $descr or return "$indent(Description missing)\n";
  $indent or $indent='';
  my $txt = '';
  if ($descr =~ /\n\s*\S/) {
    $txt .= "\n";
    my @sl = split(/\n/,$descr);
    foreach my $al (@sl) {
      $txt .= "$indent$al\n";
    }
  } else {
    chomp $descr;
    $txt .= "$indent$descr\n";
  }
  return $txt;
}



sub GetUnit{
  my ($this,$key) = @_;
  my $aVal =  $this->GetMetaProperty($key,'UNIT');
  return $aVal;
}


sub GetValidator{
  my ($this,$key) = @_;
  my $aVal =  $this->GetMetaProperty($key,'VALIDATOR');
  $aVal and return $aVal;
  $aVal = $this->can('validate'.(ucfirst $key));
  return $aVal;
}


sub hasDefault{
  my ($this,$key) = @_;
  my $default = $this->GetDefault($key);
  if (defined $default and $default eq $INVALID) {
    return 0;
  } else {
    return 1;
  }
}

sub GetDefault{
  my ($this,$key) = @_;
  my $metadata = $this->{'KEYS'}->{$key};
  my $retVal ; 
  my $defaultFunc;
  if($defaultFunc = $this->can('default'.(ucfirst $key))){
    $retVal = &$defaultFunc($this,$key);
  }
  elsif(exists $metadata->{'DEFAULT'}){
    $retVal = $metadata->{'DEFAULT'};
  }
  else{
    $retVal =  $INVALID;
  }
  return $retVal;
}

sub GetDefaultResolved{
  my ($this,$key,$noSubst) = @_;
  my $aDefault = $this->GetDefault($key);
  unless($INVALID eq $aDefault){
    $aDefault = $this->resolveValue($key,$aDefault,$noSubst);
  }
  return $aDefault;
}

sub GetFormatter{
  my ($this,$key) = @_;
  my $aVal = $this->GetMetaProperty($key,'FORMATTER');
  $aVal and return $aVal;
  my $type = $this->GetType($key);
  $type and $aVal = $FORMATTERS{$type};
  $aVal and return $aVal;
  $aVal = $this->can('format'.(ucfirst $key));
  return $aVal;
}

sub GetInitial{
  my ($this,$key) = @_;
  my $initFunction;
  if ($this->ExistsInitial($key)) {
    my $retVal = $this->GetMetaProperty($key,'INITIAL');
    return $this->resolveValue($key,$retVal);
  }
  elsif ($initFunction =  $this->can('initialize'.(ucfirst $key))){
    return &$initFunction($this);
  }else {
    return $INVALID;
  }
}
sub GetDependencies{
  my ($this,$key) = @_;
  return $this->GetMetaProperty($key,'DEPENDENCIES');
}
sub GetType{
  my ($this,$key) = @_;
  return $this->GetMetaProperty($key,'TYPE');
}
sub GetParser{
  my ($this,$key) = @_;
  my $aVal =  $this->GetMetaProperty($key,'PARSER');
  $aVal and return $aVal;
  my $type = $this->GetType($key);
  $type and $aVal = $PARSERS{$type};
  $aVal and return $aVal;
  $aVal = $this->can('parse'.(ucfirst $key));
  return $aVal;
}

sub GetMetaProperty{
  my ($this,$key,$name) = @_;
  unless(exists $this->{'KEYS'}->{$key}){
    return undef;
  }
  unless(exists $this->{'KEYS'}->{$key}->{$name}){
    return undef;
  }
  return $this->{'KEYS'}->{$key}->{$name};
}

sub ExistsKey{
  my ($this,$key) = @_;
  my $keys = $this->{'KEYS'};
  foreach my $aKey( keys %$keys){
    if(lc $aKey eq lc $key){
      return 1;
    }
  }
  return 0;
}




sub ExistsInitial{
  my ($this,$key) = @_;
  return $this->ExistsMetaProperty($key,'INITIAL');
}
sub ExistsMetaProperty{
  my ($this,$key,$name) = @_;
  unless (exists $this->{'KEYS'}->{$key}) {
    return 0;
  }
  unless (exists $this->{'KEYS'}->{$key}->{$name}) {
    return 0;
  }
  return 1;
}

sub MergeKeys{
  my ($this,$newKeys) = @_;
  my $thisKeys = $this->{'KEYS'};
  $this->MergeKeysLR($thisKeys,$newKeys);
}


sub MergeKeysLR{
  my ($this,$thisKeys,$newKeys) = @_;
  foreach my $aKey (keys %$newKeys) {
    if (exists $thisKeys->{$aKey}) {
      my $thisKeyVals = $thisKeys->{$aKey};
      my $newKeyVals = $newKeys->{$aKey};
      foreach (keys %$newKeyVals) {
        $thisKeyVals->{$_} = $newKeyVals->{$_};
      }
    } else {
      my $aValue = $newKeys->{$aKey};
      $thisKeys->{$aKey}=$aValue;
    }
  }
}


sub MergeData{
  my($this,$newData,$oldData) = @_;
  my @newData = split(/\n/,$newData);
  my @mergedData = @$oldData;
  push @mergedData,(@newData);
  return \@mergedData;
}





sub myKeys{
  return {};
}



sub Validate{
  my $this =  shift;
  my %valRes;
  my $keys = $this->{'KEYS'};
  foreach my $key (keys %$keys) {
      my $val = $this->Get($key);
      my $res = $this->ValidateKey($val,$key);
      if ($res) {
        $valRes{$key}=$res;
      }
  }
  if(keys %valRes){
    my $retVal = "\nOne or more  validation errors occured:\n";
    $retVal .= " PARAMETER\tERRORMESSAGE\n";
    $retVal .= " =========\t============\n";
    foreach my $key(keys %valRes){
      $retVal .= " ". uc ($key) . "\t". $valRes{$key}."\n";
    }
    return $retVal;
  }
  return 0;
}

sub Describe{
  my $this =  shift;
  my $txt='';
  my $keys = $this->{'KEYS'};
  foreach my $key (keys %$keys) {
    $this->IsPersistent($key) or next;
    my $nm = uc($key);
    my $aType = $this->GetType($key);
    $txt .= "$nm:" . $this->GetDescription($key,"\t");
    $txt .= "\tINITIAL:";
    my $lines = $this->GetConfigurationLinesForKey($key);
    $lines or $lines = '(NO INITIAL VALUE)';
    if($lines =~ /\n\s*\S/){
      $txt .= "\n";
      my @sl = split(/\n/,$lines);
      foreach my $al(@sl){
        $al =~ s/^\w+\s+//;
        if($al =~ /^\s*#/){
          $txt .= "\t$al";
        }
        else{
          $txt .= "\t". $nm .  " $al";
        }
        $txt .= "\n";
      }
    }
    else{
      $lines =~ s/^\s*\w+\s+//;
      chomp $lines;
      $txt .= " '$lines'";
      if($aType and uc $aType eq 'DATASIZE' and $lines and $lines =~ /$LINEREGEXP/ ){
        my ($val,$unit) = readableUnit($2,'PAGE');
        my $convertedSize = "$val $unit";
        $txt .= " ($convertedSize)\n";
      }
      $txt .= "\n";
    }
  }
  return $txt;
}






sub slashFix{
  my $path = shift;
  if( $^O =~ /mswin/i){
    $path =~ s/\//\\/g;
  }
  else{
    $path =~ s/\\/\//g;
  }
  return $path;
}

#XML-Handling
sub GetXmlData(){
  return [];
}
my ($currentKey,$currenttxt,$currentid,$currentKeys,$superKeys);
sub handleKeyDefinitions{
  my ($this,$aSuperKeys) = @_;
  $superKeys = $aSuperKeys;
  my $retVal ; 
  my @lines= filterXMLTag($this->GetXmlData(),'ParameterDefinitions');
  $retVal = parseXML( \@lines,$this ) and return Logit("Error parsing base parameter description: ".$retVal,undef);
  return $superKeys;
}

sub handleInitialValues{
  my ($this)=@_;
  my $retVal;
  my @lines= filterXMLTag($this->GetXmlData(),'ParameterData');
  my @configLines;
  my $multiLineParameter=undef;
  my $multiLineId=undef;
  #parse the parameter data with xml parser to remove xml tags
  $retVal = parseXML(\@lines,$this,
           {
            'cdata'=>
             sub {
               my  ($this,$line,$stack) = @_;
               if(defined $multiLineParameter){
                 $multiLineParameter .= $line . "\n";
               }
               else{
                 push @configLines,($line);
               }
             },

            'beginParameterValue'=>
             sub { my  ($this,$atts,$stack) = @_;
                   $multiLineId = $atts->{'id'};
                   $multiLineParameter='';
                 },
            'endParameterValue'=>
             sub { my  ($this,$stack) = @_;
                   $this->InternalSet($multiLineId,$multiLineParameter);
#                   print "Have set $multiLineId to '$multiLineParameter'\n";
                   undef $multiLineParameter;
                   undef $multiLineId;
                 }

           });
  $retVal and return Logit("Error reading default values for parameter: ".$retVal,undef);
  $this->parseLineArray(\@configLines);
}

sub GetMaximum{
  my ($this,$key) = @_;
  return $this->GetLimit($key,'max');
}

sub GetMinimum{
  my ($this,$key) = @_;
  return $this->GetLimit($key,'min');
}
sub GetLimit{
  my ($this,$key,$limit) = @_;
  my $aVal =  $this->can($limit.(ucfirst $key));
  $aVal and return &$aVal($this);
  return undef;
}




sub getParameterDescriptions{
  my $this = shift;
  my @red = filterXMLTag($this->GetXmlData(),'ParameterDefinitions');
  return \@red;
}

sub beginParameterDefinitions{
  my  ($this,$atts,$stack) = @_;
  $currentKeys = {};
}
sub endParameterDefinitions{
  my  ($this,$atts,$stack) = @_;
  $this->MergeKeysLR($superKeys,$currentKeys);
  undef $currentKeys;
}
sub beginParameter{ my  ($this,$atts,$stack) = @_;
                    $currentKey ={};
                    $currentid = $atts->{'id'};
                    $currentKey->{'UNIT'} = $atts->{'unit'};
                    $currentKey->{'DEPENDENCIES'} = $atts->{'dependencies'};
                    my $pa = $atts->{'persistent'};
                    if (defined $pa) {
                      if ($pa =~ /:(\w+)$/) {
                        my $osk = $1;
                        if ($^O =~ /mswin/i) {
                          $osk eq 'win' and
                            $currentKey->{'PERSISTENT'} = $atts->{'persistent'};
                        } else {
                          $osk eq 'ux' and
                            $currentKey->{'PERSISTENT'} = $atts->{'persistent'};
                        }
                      }
                      else{
                        $currentKey->{'PERSISTENT'} = $atts->{'persistent'};
                      }
                    }
                  }
sub endParameter {$currentKeys->{$currentid}=$currentKey;undef $currentKey;undef $currentid;}
sub beginDefault{ }
sub endDefault { $currentKey->{'DEFAULT'}=$currenttxt;$currenttxt='';}
sub beginValidator{ }
sub endValidator { $currentKey->{'VALIDATOR'}=$currenttxt;$currenttxt='';}
sub beginInitial { }
sub endInitial { $currentKey->{'INITIAL'}=$currenttxt;$currenttxt='';}
sub beginDescription { }
sub endDescription { $currentKey->{'DESCRIPTION'}=$currenttxt;$currenttxt='';}
sub beginType { };
sub endType { $currentKey->{'TYPE'}=$currenttxt;$currenttxt='';}
sub cdata{
  my ($this,$txt,$stack) = @_;  
  my $current=  @{$stack}[$#{@{$stack}}]->[0];
  $currenttxt .= $current eq 'Description' ? $txt . "\n" : $txt;
}



#Get the validator function for the gui functionality
sub GetGuiValidator{
  my ($this,$key) = @_;
  my $aVal =  $this->can('guiValidate'.(ucfirst $key));
  $aVal and return $aVal;
  $aVal =  $this->can('validate'.(ucfirst $key));
  $aVal and return sub{
    my ($this,$val) = @_;
    my @retVal = (1);
    my $res = &$aVal($this,$val);
    $res and @retVal = (0,$res,$res);
    return @retVal;
  };
  $aVal = sub {return (4);};
  return $aVal;
}

sub createParameterDescriptionById{
  my ($this,$id) = @_;
  $id or return Logit("Missing key when trying to create parameter description",undef);
  $id =~ /^string:(.+)$/ and return $this->createParameterDescriptionByLabel($id,$1);
  $id =~ /^array:(\w+)(:\d+)?$/ and return $this->createParameterDescriptionArray($1);
  my $aKey;
  $aKey = $this->GetKey($id) or 
    return Logit("Cannot create parameter description for unknown key '$id'",undef);
  my $param = {'id' => $id};
  my $deps = $this->GetDependencies($aKey);
  if($deps){
    my @da = split(/,/,$deps);
    $param->{'dependencies'}=\@da;
  }
  my $getterName = 'get'.(ucfirst $id);
  $param->{'Getter'} =  sub {return $this->Getter(ref($this).'::'.$getterName);};
  my $setterName = 'set'.(ucfirst $id);
  $param->{'Setter'} = sub {return $this->Setter(ref($this).'::'.$setterName,$_[0]);};
  $param->{'Maximum'} = $this->GetMaximum($id);
  $param->{'Minimum'} = $this->GetMinimum($id);
  $param->{'Type'} = $this->GetType($id);
  $param->{'unit'} = $this->GetUnit($id);

  my $aDefault =  $this->GetDefaultResolved($id);
  $param->{'Default'} = $aDefault;
  $param->{'Value'}= $this->Get($id);
  my $guiValidator =  $this->GetGuiValidator($id);
  $param->{'Validator'} = sub {return &$guiValidator($this,$_[0]);};
  return $param;
}


sub createParameterDescriptionByLabel{
  my ($this,$id,$label) = @_;
  my $param = {'id' => $id};
  $param->{'Getter'} =  sub {return $label;};
  $param->{'Setter'} = sub {};
  $param->{'Type'} = 'STRING';
  return $param;
}



sub createParameterDescriptionArray{
  my ($this,$array) = @_;
  $array or return Logit("Array not specified in createParameterDescriptionArray!",undef);
  return Logit("Array $array not known!",undef);
}




sub passwordValidator{
  my ($this,$aPassword) = @_;
  my @errorRet = (0,"Password missing", 
                  "Enter a password");
  my @successRet = (1);
  my @retVal = @successRet;
  if ((not (defined $aPassword)) or  ("" eq $aPassword)) {
    @retVal = @errorRet;
  }
  return @retVal;
} 

sub confirmPasswordValidator{
  my ($this,$aPassword,$aConfirmPassword) = @_;
  return $this->confirmPasswordValidatorLabeled($aPassword,$aConfirmPassword);
}

sub confirmPasswordValidatorLabeled{
  my ($this,$aPassword,$aConfirmPassword,$label) = @_;
  my $shit = $label ? " for " . $label  : "";
  my @errorRet = (0,"Confirm password different", 
                  "Enter the same password twice$shit");

  my @errorRet2 = (0,"Missing password", 
                   "Enter a password$shit");
  my @errorRet3 = (0,"Missing password confirmation", 
                   "Confirm password$shit");

  my @successRet = (1);
  my @retVal = @successRet;
  if (not defined $aPassword or "" eq $aPassword ) {
    @retVal = @errorRet2;
  } elsif (not defined $aConfirmPassword or "" eq $aConfirmPassword) {
    @retVal = @errorRet3;
  } elsif ($aPassword ne $aConfirmPassword) {
    @retVal = @errorRet;
  }
  return @retVal;
}



1;


