#!/usr/bin/perl
package SAPDB::Install::Templates2::Util;
use strict;
use SAPDB::Install::Templates2::Debug;
our @ISA;
if($SCRIPTING){
  require Exporter;
  push @ISA,qw(Exporter);
}
else{
  require SDB::Common::Exporter;
  push @ISA,qw(SDB::Common::Exporter);
}
use SAPDB::Install::PipeExec;
use SAPDB::Install::Registry;

our @EXPORT = qw ( 
                   fileparse
                  basename
                  file_name_is_absolute
                  catdir
                  catfile
                  path
                  canonpath 
                  dirname
                  mkpath
                  mymkpath
                  rmtree
                  isPathAbsolute
                  willSet
                  haveSet
                  addBeforeAdvice
                  addAfterAdvice 
                  matchPath
                  matchNumber
                  matchUserPassword
                  matchDrive
                  matchStorage
                  matchDBParam
                  matchBoolean
                  matchString
                  isTrue
                  defaultFormatter
                  booleanFormatter
                  stringFormatter
                  noFormatter
                  defaultWriteParams
                  getMatchers
                  parseConfigLine
                  getTypeMapping
                  voidValidate
                  driveValidate
                  mergeFrameTemplates
                  mergeParameterTemplates
                  pagesToMB
                  yamakedir
                  readableUnit
                  normalizeAPath
                  copyFile
                  callSystem
                  parseXML
                  filterXMLTag
                  unitTest
                  dieWithStack
                  makePathOSDependend
                  pathsEqualIgnoreCase
                  utilFindInstances
                  utilInstanceExists
                 );



sub makePathOSDependend{
  my $path = shift;
  if ( $^O =~ /mswin/i) {
    $path =~ s/\//\\/g;
  } else {
    $path =~ s/\\/\//g;
  }
  return $path;
}

#downcase path and replace \\ by /
sub makePathComparable{
  my $path = shift;
  $path = lc $path;
  $path =~ s/\\/\//g;
  $path =~ s/(\s+)$//;
  $path =~ s/^(\s+)//;
  $path =~ s/\/$//;
  return $path;
}
sub pathsEqualIgnoreCase{
  my ($path1,$path2) = @_;
  $path1 = makePathComparable($path1);
  $path2 = makePathComparable($path2);
  return $path1 eq $path2;
}


###################################################
#Copy & Pase aus verschiedenen Perl-Standardmodulen:
###################################################

my $Fileparse_fstype;
my $Fileparse_igncase;


#   fileparse_set_fstype() - specify OS-based rules used in future
#                            calls to routines in this package
#
#   Currently recognized values: VMS, MSDOS, MacOS, AmigaOS, os2, RISCOS
#       Any other name uses Unix-style rules and is case-sensitive

sub fileparse_set_fstype {
  my @old = ($Fileparse_fstype, $Fileparse_igncase);
  if (@_) {
    $Fileparse_fstype = $_[0];
    $Fileparse_igncase = ($_[0] =~ /^(?:MacOS|VMS|AmigaOS|os2|RISCOS|MSWin32|MSDOS)/i);
  }
  wantarray ? @old : $old[0];
}




#Dirty test if a path is absolute
sub isPathAbsolute{
my $aPath = $_[0];
return file_name_is_absolute($aPath);
#$aPath =~ "^[a-zA-Z]:[\\\\/]" or  $aPath =~ "^/";
}



#fileparse_set_fstype

#   fileparse() - parse file specification
#


sub fileparse {
  my($fullname,@suffices) = @_;
  my($fstype,$igncase) = ($Fileparse_fstype, $Fileparse_igncase);
  my($dirpath,$tail,$suffix,$basename);
  my($taint) = substr($fullname,0,0);  # Is $fullname tainted?

  if ($fstype =~ /^VMS/i) {
    if ($fullname =~ m#/#) { $fstype = '' }  # We're doing Unix emulation
    else {
      ($dirpath,$basename) = ($fullname =~ /^(.*[:>\]])?(.*)/);
      $dirpath ||= '';  # should always be defined
    }
  }
  if ($fstype =~ /^MS(DOS|Win32)/i) {
    ($dirpath,$basename) = ($fullname =~ /^((?:.*[:\\\/])?)(.*)/);
    $dirpath .= '.\\' unless $dirpath =~ /[\\\/]$/;
  }
  elsif ($fstype =~ /^MacOS/i) {
    ($dirpath,$basename) = ($fullname =~ /^(.*:)?(.*)/);
  }
  elsif ($fstype =~ /^AmigaOS/i) {
    ($dirpath,$basename) = ($fullname =~ /(.*[:\/])?(.*)/);
    $dirpath = './' unless $dirpath;
  }
  elsif ($fstype !~ /^VMS/i) {  # default to Unix
    ($dirpath,$basename) = ($fullname =~ m#^(.*/)?(.*)#);
    if ($^O eq 'VMS' and $fullname =~ m:/[^/]+/000000/?:) {
      # dev:[000000] is top of VMS tree, similar to Unix '/'
      ($basename,$dirpath) = ('',$fullname);
    }
    $dirpath = './' unless $dirpath;
  }

  if (@suffices) {
    $tail = '';
    foreach $suffix (@suffices) {
      my $pat = ($igncase ? '(?i)' : '') . "($suffix)\$";
      if ($basename =~ s/$pat//) {
        $taint .= substr($suffix,0,0);
        $tail = $1 . $tail;
      }
    }
  }

  $tail .= $taint if defined $tail; # avoid warning if $tail == undef
  wantarray ? ($basename . $taint, $dirpath . $taint, $tail)
            : $basename . $taint;
}
#fileparse


#   basename() - returns first element of list returned by fileparse()

sub basename {
  my($name) = shift;
  (fileparse($name, map("\Q$_\E",@_)))[0];
}
#basename


#    dirname() - returns device and directory portion of file specification
#        Behavior matches that of Unix dirname(1) exactly for Unix and MSDOS
#        filespecs except for names ending with a separator, e.g., "/xx/yy/".
#        This differs from the second element of the list returned
#        by fileparse() in that the trailing '/' (Unix) or '\' (MSDOS) (and
#        the last directory name if the filespec ends in a '/' or '\'), is lost.

sub dirname {
    my($basename,$dirname) = fileparse($_[0]);
    my($fstype) = $Fileparse_fstype;

    if ($fstype =~ /VMS/i) { 
        if ($_[0] =~ m#/#) { $fstype = '' }
        else { return $dirname || $ENV{DEFAULT} }
    }
    if ($fstype =~ /MacOS/i) { return $dirname }
    elsif ($fstype =~ /MSDOS/i) { 
        $dirname =~ s/([^:])[\\\/]*$/$1/;
        unless( length($basename) ) {
	    ($basename,$dirname) = fileparse $dirname;
	    $dirname =~ s/([^:])[\\\/]*$/$1/;
	}
    }
    elsif ($fstype =~ /MSWin32/i) { 
        $dirname =~ s/([^:])[\\\/]*$/$1/;
        unless( length($basename) ) {
	    ($basename,$dirname) = fileparse $dirname;
	    $dirname =~ s/([^:])[\\\/]*$/$1/;
	}
    }
    elsif ($fstype =~ /AmigaOS/i) {
        if ( $dirname =~ /:$/) { return $dirname }
        chop $dirname;
        $dirname =~ s#[^:/]+$## unless length($basename);
    }
    else { 
        $dirname =~ s:(.)/*$:$1:;
        unless( length($basename) ) {
	    $Fileparse_fstype = $fstype;
	    ($basename,$dirname) = fileparse $dirname;
	    $dirname =~ s:(.)/*$:$1:;
	}
    }

    $dirname;
}
#dirname

fileparse_set_fstype $^O;

#######################################################################
# File::Spec geklaut : (Pseudo-Objektorientiert! Instance methods!!!
########################################################################

sub file_name_is_absolute {
  $^O =~ /mswin/i ? file_name_is_absolute_win32(@_) : file_name_is_absolute_unix(@_);
}
#filename_is_absolute
sub file_name_is_absolute_win32 {
    my($file) = @_;
    $file =~ m{^([a-z]:)?[\\/]}i ;
}
#file_name_is_absolute_win32

sub catdir {
  $^O =~ /mswin/i ? catdir_win32(@_) : catdir_unix(@_);
}
#catdir
sub catdir_win32 {
    my @args = @_;
    for (@args) {
	# append a slash to each argument unless it has one there
	$_ .= "\\" if $_ eq '' or substr($_,-1) ne "\\";
    }
    my $result = canonpath_win32(join('', @args));
    $result;
}
#catdir_win32

sub catfile {
  $^O =~ /mswin/i ? catfile_win32(@_) : catfile_unix(@_);
}
#catfile
sub catfile_win32 {
    my $file = pop @_;
    return $file unless @_;
    my $dir = catdir_win32(@_);
    $dir =~ s/(\\\.)$//;
    $dir .= "\\" unless substr($dir,length($dir)-1,1) eq "\\";
    return $dir.$file;
}
#catfile_win32

sub path {
  $^O =~ /mswin/i ? path_win32(@_) : path_unix(@_);
}
#path
sub path_win32 {
    local $^W = 1;
    my $path = $ENV{'PATH'} || $ENV{'Path'} || $ENV{'path'};
    my @path = split(';',$path);
    foreach(@path) { $_ = '.' if $_ eq '' }
    @path;
}
#path_win32

sub canonpath {
  $^O =~ /mswin/i ? canonpath_win32(@_) : canonpath_unix(@_);
}
#canonpath

sub canonpath_win32 {
    my($path) = @_;
    $path =~ s/^([a-z]:)/\u$1/;
    $path =~ s|/|\\|g;
    $path =~ s|\\+|\\|g ;                          # xx////xx  -> xx/xx
    $path =~ s|(\\\.)+\\|\\|g ;                    # xx/././xx -> xx/xx
    $path =~ s|^(\.\\)+|| unless $path eq ".\\";   # ./xx      -> xx
    $path =~ s|\\$|| 
             unless $path =~ m#^([a-z]:)?\\#;      # xx/       -> xx
    $path .= '.' if $path =~ m#\\$#;
    $path;
}
#canonpath_win32


sub canonpath_unix {
    my($path) = @_;
    $path =~ s|/+|/|g ;                            # xx////xx  -> xx/xx
    $path =~ s|(/\.)+/|/|g ;                       # xx/././xx -> xx/xx
    $path =~ s|^(\./)+|| unless $path eq "./";     # ./xx      -> xx
    $path =~ s|/$|| unless $path eq "/";           # xx/       -> xx
    $path;
}
#canonpath_unix
sub catdir_unix {
    my @args = @_;
    for (@args) {
	# append a slash to each argument unless it has one there
	$_ .= "/" if $_ eq '' or substr($_,-1) ne "/";
    }
    my $result = join('', @args);
    # remove a trailing slash unless we are root
    substr($result,-1) = ""
	if length($result) > 1 && substr($result,-1) eq "/";
    $result;
}
#catdir_unix

sub catfile_unix {
    my $file = pop @_;
    return $file unless @_;
    my $dir = catdir_unix(@_);
    for ($dir) {
	$_ .= "/" unless substr($_,length($_)-1,1) eq "/";
    }
    return $dir.$file;
}
#catfile_unix



sub file_name_is_absolute_unix {
    my($file) = @_;
    $file =~ m:^/: ;
}
#file_name_is_absolute_unix

sub path_unix {
    my $path_sep = ":";
    my $path = $ENV{PATH};
    my @path = split $path_sep, $path;
    foreach(@path) { $_ = '.' if $_ eq '' }
    @path;
}
#path_unix




#############################
# File::Path geklaut
##############################



my $Is_VMS = 0;
my $Is_MacOS = $^O eq 'MacOS';

# These OSes complain if you want to remove a file that you have no
# write permission to:
my $force_writeable = ($^O eq 'os2' || $^O eq 'dos' || $^O eq 'MSWin32' ||
		       $^O eq 'amigaos' || $^O eq 'MacOS' || $^O eq 'epoc');

sub carp_replacement{
    print $_[0];
}


sub mkpath {
    my($paths, $verbose, $mode,$uid,$gid) = @_;
    # $paths   -- either a path string or ref to list of paths
    # $verbose -- optional print "mkdir $path" for each directory created
    # $mode    -- optional permissions, defaults to 0777
    # $uid,$gid    -- optional uid  gid of os owner
    local($")=$Is_MacOS ? ":" : "/";
    $mode = 0777 unless defined($mode);
    $paths = [$paths] unless ref $paths;
    my(@created,$path);
    foreach $path (@$paths) {
	$path .= '/' if $^O eq 'os2' and $path =~ /^\w:\z/s; # feature of CRT 
	next if -d $path;
	my $parent = dirname($path);
	unless (-d $parent or $path eq $parent) {
	    push(@created,mkpath($parent, $verbose, $mode,$uid,$gid));
 	}
        $verbose and Logit("[mkdir($path,$mode)]");
	unless (mkdir($path,$mode)) {
            unless (-d $path){
            $verbose and Logit("Failed making directory  $path]");
              return undef;
            }
          }
        if(not $^O =~ /mswin/i){
            $verbose and Logit("[chmod $mode $path]");
	     chmod ($mode, $path);
             my @stb = stat($path);
             my $am = $stb[2] & 0777;
             if($am != $mode){
              Logit("Cannot change directory mode to $mode for $path");
              return undef;
             }
	   if($uid and $gid){
            $verbose and Logit("[chown $uid:$gid $path]");
            unless(chown( $uid,$gid,$path)){
              Logit("Cannot chown $uid:$gid $path");
              return undef;
            }
           }
        } 
	push(@created, $path);
    }
    @created;
}
#mkpath

sub mymkpath {
    my($paths,  $mode) = @_;
    local($")=$Is_MacOS ? ":" : "/";
    $mode = 0777 unless defined($mode);
    $paths = [$paths] unless ref $paths;
    my($path);
    my $e;
    foreach $path (@$paths) {
	$path .= '/' if $^O eq 'os2' and $path =~ /^\w:\z/s; # feature of CRT 
	next if -d $path;
	my $parent = dirname($path);
	unless (-d $parent or $path eq $parent) {
	    $e = mymkpath($parent,$mode) and return $e;
 	}
	unless (mkdir($path,$mode)) {
	    $e = $!;
            unless (-d $path){
              return $e;
            }
	}
    }
    return 0;
}
#mkpath


#+sub():int:Create a directory recursevily
sub yamakedir{
  my ($dir,$perms) = @_;
  my $retVal=0;
  my $uid = "";
  my $gid = "";
  unless(defined $perms){
    $perms = 0777;
  }
  unless ($^O =~ /mswin/i){
    $uid = getpwnam($SAPDB::Install::Values::owner);
    $gid = getgrnam($SAPDB::Install::Values::group);
  }
  if(defined $dir){
    if(-d $dir){
      $retVal=1;
    }
  }
  else{
    $retVal=2;
  }
  if( $retVal == 0){
    SAPDB::Install::System::makedir($dir,$perms,$uid,$gid);
    unless(-d $dir){
      $retVal=3;
    }
  }
  return $retVal;
}








sub rmtree {
    my($roots, $verbose, $safe) = @_;
    my(@files);
    my($count) = 0;
    $verbose ||= 0;
    $safe ||= 0;

    if ( defined($roots) && length($roots) ) {
      $roots = [$roots] unless ref $roots;
    }
    else {
      carp_replacement "No root path(s) specified\n";
      return 0;
    }

    my($root);
    foreach $root (@{$roots}) {
    	if ($Is_MacOS) {
	    $root = ":$root" if $root !~ /:/;
	    $root =~ s#([^:])\z#$1:#;
	} else {
	    $root =~ s#/\z##;
	}
	(undef, undef, my $rp) = lstat $root or next;
	$rp &= 07777;	# don't forget setuid, setgid, sticky bits
	if ( -d _ ) {
	    # notabene: 0777 is for making readable in the first place,
	    # it's also intended to change it to writable in case we have
	    # to recurse in which case we are better than rm -rf for 
	    # subtrees with strange permissions
	    chmod(0777, ($Is_VMS ? $root : $root))
	      or carp_replacement "Can't make directory $root read+writeable: $!"
		unless $safe;

	    if (opendir my $d, $root) {
		if (!defined ${"\cTAINT"} or ${"\cTAINT"}) {
		    # Blindly untaint dir names
		    @files = map { /^(.*)$/s ; $1 } readdir $d;
		} else {
		    @files = readdir $d;
		}
		closedir $d;
	    }
	    else {
	        carp_replacement "Can't read $root: $!";
		@files = ();
	    }

	    # Deleting large numbers of files from VMS Files-11 filesystems
	    # is faster if done in reverse ASCIIbetical order 
	    @files = reverse @files if $Is_VMS;
	    if ($Is_MacOS) {
		@files = map("$root$_", @files);
	    } else {
		@files = map("$root/$_", grep $_!~/^\.{1,2}\z/s,@files);
	    }
	    $count += rmtree(\@files,$verbose,$safe);
	    if ($safe &&
		($Is_VMS ? !-w $root : !-w $root)) {
		next;
	    }
	    chmod 0777, $root
	      or carp_replacement "Can't make directory $root writeable: $!"
		if $force_writeable;
	    if (rmdir $root) {
		++$count;
	    }
	    else {
		carp_replacement "Can't remove directory $root: $!";
		chmod($rp, ($Is_VMS ? $root : $root))
		    or carp_replacement("and can't restore permissions to "
		            . sprintf("0%o",$rp) . "\n");
	    }
	}
	else { 
	    if ($safe && !(-l $root || -w $root))
	    {
		next;
	    }
	    chmod 0666, $root
	      or carp_replacement "Can't make file $root writeable: $!"
		if $force_writeable;
	    # delete all versions under VMS
	    for (;;) {
		unless (unlink $root) {
		    carp_replacement "Can't unlink file $root: $!";
		    if ($force_writeable) {
			chmod $rp, $root
			    or carp_replacement("and can't restore permissions to "
			            . sprintf("0%o",$rp) . "\n");
		    }
		    last;
		}
		++$count;
		last unless $Is_VMS && lstat $root;
	    }
	}
    }

    $count;
}
#rmtree




#############################
# Definitions vor advices,
#############################




sub getBeforeAdvices{
  my ($this) = @_;
  return  getAdvices($this,"beforeAdvicesKey");
}

sub getAfterAdvices{
  my ($this) = @_;
  return  getAdvices($this, "afterAdvicesKey");
}

sub getAdvices{
  my ($this,$aKey) = @_;
  my $advices = $this->{$aKey};
  unless( defined $advices){
    $advices = {};
    $this->{$aKey} = $advices;
  }
  return $advices;
}

sub  getAdviceList{
  my ( $key,$hashRef) = @_;
  my $aListRef = $hashRef->{$key};
  unless (defined $aListRef){
    $aListRef = [];
  }
  return $aListRef;
}

sub  getBeforeAdviceList{
  my ($this,$key) = @_;
  my $beforeAdvices = getBeforeAdvices($this);
  return getAdviceList($key,$beforeAdvices);
}

sub  getAfterAdviceList{
  my ($this, $key) = @_;
  my $afterAdvices = getAfterAdvices($this);
  return getAdviceList($key,$afterAdvices);
}
sub  addAdvice{
  my ($key,$hashRef,$subRef) = @_;
  my $aListRef = $hashRef->{$key};
  unless (defined $aListRef){
    $aListRef = [];
    $hashRef->{$key} = $aListRef;
  }
  push  @$aListRef, ($subRef);
#  print "Added advice  $subRef for $key to $aListRef\n";
  return $subRef;
}
sub  addBeforeAdvice{
  my ($this, $key,$subRef) = @_;
  my $beforeAdvices = getBeforeAdvices($this);
  return addAdvice($key,$beforeAdvices,$subRef);
}
sub  addAfterAdvice{
  my ($this, $key,$subRef) = @_;
  my $afterAdvices = getAfterAdvices($this);
  return addAdvice($key,$afterAdvices,$subRef);
}
sub callAdvices{
  my ($this,$key,$list) = @_;
  foreach my $subRef (@$list){
    &$subRef($this);
  }
}
sub willSet{
  my ($this,$key) = @_;
  my $befores = getBeforeAdviceList($this,$key);
  callAdvices($this, $key,$befores);
}
sub haveSet{
  my ($this,$key) = @_;
  my $afters = getAfterAdviceList($this,$key);
  callAdvices($this,$key,$afters);
}




sub matchDrive{
  my $retVal=undef;
  my $aVal = $_[0];
  if(defined $aVal and  $aVal =~ m/^\s*([a-zA-Z])[:]?\\?\s*$/){
    $retVal=$1;
  }
  return $retVal;
}
#matchPath


#match for "<USER>,<PASSWORD>"
sub matchUserPassword{
  my @retVal=();
  if( $_[0] =~ m/^\s*([\w\d]+)(\s*,\s*([\w\d]+))?\s*$/){
   @retVal = ($1,$3); 
  }
  return @retVal;
}
#matchUserPassword

#match for "<USER>,<PASSWORD>"
sub matchDBParam{
  my @retVal=();
  if( $_[0] =~ m/^\s*([\w\d]+)\s+(.+)\s*$/){
    my $val = $2;
    my $key = $1;
    $val =~ s/\s*$//;
   @retVal = ($key,$val); 
  }
  return \@retVal;
}
#matchUserPassword



#match for "(number,kind,path,type,size)"
sub matchDBVolume{
  my @retVal= split /;/,$_[0];
  return \@retVal;
}
#matchUserPassword


sub matchPath{
  my $retVal=undef;
  if( $_[0] =~ m/^\s*(.+)\s*$/){
    $retVal=$1;
    $retVal =~ s/\s*$//;
  }
#  print "Path: \"$retVal\"\n";
  return $retVal;
}
#matchPath

sub matchNumber{
  my $retVal=0;
  if( $_[0] =~ m/^\s*([\d]+)\s*$/){
    $retVal=$1;
  }
  return $retVal;
}
#matchNumber

sub matchStorage{
  my $retVal=0;
  if( $_[0] =~ m/^\s*([\d.,]+)\s*$/){
    $retVal=$1;
  }
  return $retVal;
}
#matchStorage

sub matchString{
  my $retVal="";
  if( $_[0] =~ m/^\s*([^\s].*[^\s])\s*$/){
    $retVal=$1;
  }
  return $retVal;
}


sub matchBoolean{
  my $retVal = undef;
  my $val = $_[0];
  if(defined $val){
    unless ( $val =~ /^\s*$/ or  $val =~ /^\s*false\s*$/i or $val =~ /^\s*no\s*$/i
             or $val =~ /^\s*0\s*$/ or $val =~ /^\s*off\s*$/i ){
      $retVal = $val;
    }
  }
  return $retVal;
}
sub isTrue{
return defined matchBoolean(@_) ? 1 : 0;
}

sub volumeFormatter{
  return join (";",@$_[0]);
}

sub defaultFormatter{return $_[0];}
sub booleanFormatter{
  my $retVal=undef;
  if( isTrue($_[0])){
    $retVal="yes";
  }
  else {
    $retVal="no";
  }
  return $retVal;
}
sub stringFormatter{return defined $_[0] ? $_[0] : "";}
sub noFormatter{return undef;}

 
sub defaultWriteParams{
  my ($paramsRef,$section,$write) = @_;
  my @paramDescs;
  while( (my $key, my $value) = each %$paramsRef){
    push @paramDescs, ( [$key,$value]);
  }
  
  my $counter = @paramDescs;
 




   my @sortedParamDescs = 
     sort { 
       my $retVal=0;
       my $pDesc1 = $a->[1];
       my $Props1 = $pDesc1->[6];
       my $priority1 = defined $Props1 ? $Props1->{"priority"} : 0;
       my $key1 = $b->[0];
       my $pDesc2 = $b->[1];
       my $Props2 = $pDesc2->[6];
       my $priority2 = defined $Props2? $Props2->{"priority"}: 0;
       my $key2 = $a->[0];
#      print "$priority1 - $priority2  \n";
       return $priority2 - $priority1;
#       print "$key1 - $key2 : ". ($key1 gt $key2) . "\n";
#       return ($key1 cmp $key2);
   }
       @paramDescs;

  foreach my $pDescKeys (@sortedParamDescs){
    my $pDesc = $pDescKeys->[1];
    my $key = $pDescKeys->[0];
    my $getter = @$pDesc[1];
    my $formatter = @$pDesc[2];
    my $aType = @$pDesc[3];
    my $aSection =@$pDesc[4]; 
    my $aValue = &$getter();

    my $Props = @$pDesc[6];
    my $transient = defined $Props ? $Props->{"transient"} : 0;
    $transient = defined $transient ? $transient : 0;


    if(  $aSection eq $section 
         and (defined $aValue or $aType eq "BOOLEAN" )
         and not $transient
      ){
      my $aFormattedValue = &$formatter($aValue);
    &$write($key . " " . $aFormattedValue . "\n");
    }
  }
}

#+sub(Parameter):Liste:Default-Validator, der immer (4) zurckliefert.
sub voidValidate{
return (4);
}


sub driveValidate{
  my $path = $_[0];
  my $userInput = $_[0];
  unless(defined $path){
    return (3);
  }
  $path = matchDrive($path);
  unless ($path ){
    return (0,"Invalid Drive: \"$userInput\"", "Invalid Drive \"$userInput\": Use a-z or A-Z");
  }
  $path .= ":\\";
  unless (-d  $path){
    return (0,"Drive error in \"$path\": $!", "Drive error in \"$path\": $!");
  }
  return (1);
}


my %MATCHERS = (
             "PATH" => \&matchPath,
             "DRIVE" =>  \&matchDrive,
             "STRING" =>  \&matchString,
             "DATASIZE" => \&matchStorage  ,
             "INTEGER" =>  \&matchNumber,
             "BOOLEAN" =>  \&matchBoolean,
             "USERPASSWORD" => \&matchUserPassword,
             "DBPARAMETER" => \&matchDBParam,
             "DBVOLUME" => \&matchDBVolume
);

sub getMatchers{
  return \%MATCHERS;
}

sub getTypeMapping{
  my ($aparamContainer) = @_;
  my %allTypes;
  my $params = undef;
  if(defined $aparamContainer){
    $params = $aparamContainer->getParams();
  }
  else{
    $params = {};
  }
 foreach my $key (keys %$params){
   my $value = $params->{$key};
   $allTypes{$key} = $value->[3];
 }
  return \%allTypes;
}


sub parseConfigLine{
 my ($line,$section,$keyVal,$paramContainer) =@_;
 my ($key,$value,$type,$matcher);
 my $typeRef = {};
 if(defined $paramContainer){
   $typeRef = getTypeMapping($paramContainer);
 }
 my %allTypes = %$typeRef;
    #Comment
    if ($line =~ /^\s*#/) {
    } 
    #Section title
    elsif ($line =~ /^\s*\[(.+)\]\s*$/) {
      &$section($1);
    } 
    #Key value pair seperated by =
    elsif ($line =~ /^\s*(\w+)\s*=\s*(.+)$/) {
      $key=$1; $value=$2;
      $type= $allTypes{$key};
      $matcher = defined $type? $MATCHERS{$type} : undef;
      if(defined $matcher){
        $value = &$matcher($value);
      }
       &$keyVal($key,$value);
    }
    #Key value pair seperated by spaces
    elsif ($line =~ /^\s*(\w+)\s+(\S.*)$/) {
      $key=$1; $value=$2;
      $type= $allTypes{$key};
      $matcher = defined $type? $MATCHERS{$type} : undef;
      if(defined $matcher){
        $value = &$matcher($value);
      }
       &$keyVal($key,$value);
    }
    #Key without value
    elsif ($line =~ /^\s*(\w+)\s*$/) {
       &$keyVal($1,"");
    }
    #Generic type
    elsif ($line =~ /^\s*!(\w+):\s+(\w+)\s+(\S.*)$/) {
      $type = $1;
      $key = $2;
      $value = $3;
      $matcher = defined $type? $MATCHERS{$type} : undef;
      if(defined $matcher){
        $value = &$matcher($value);
      }
       &$keyVal($key,$value,$type);
    }
}


sub mergeFrameTemplates{
  my ($h1,$h2) = @_;
  while( (my $key,my $value) = each %$h2){
    $h1->{$key}=$value;
  } 
  return $h1;
}

sub mergeParameterTemplates{
  my ($h1,$h2) = @_;
  while( (my $key,my $source) = each %$h2){
    if(defined $h1->{$key}){
      foreach ( my $i =0; $i <= @$source - 1; $i++){
        my $target = $h1->{$key};
        my $listElement = ${$source}[$i]; 
        if(defined $listElement){
           ${$target}[$i] = $listElement;
        }
      }
    }
    else{
      $h1->{$key}=$source;
    }
  } 
  return $h1;
}


#+sub(pages):int:Convert pages to Megabytes
sub pagesToMB{
  my ($pages) = @_;
  my $retVal = ($pages /  1024) * 8;
  return  sprintf "%d" ,$retVal,
}

#+sub(Integer:storage|String:unit):int:Convert a storage unit to another that is better for reading
sub readableUnit{
  my ($size,$unit) = @_;
  my @units = ("byte","KB","MB","GB","TB");
  if($unit =~ /^Page/i){
    $unit = "KB";
    $size *= 8;
  }
  my $currentUnit=undef;
  my $retVal =$size;
  while(my $aunit = shift @units) {
    if( $unit eq $aunit){
      $currentUnit = $aunit;
      last ;
    }
  }
  while(my $aunit = shift @units) {
    if( $retVal < 1024){
      last;
    }
    $currentUnit = $aunit;
    $retVal = sprintf "%d",($retVal/1024);
  }
  return ($retVal,$currentUnit);
}




sub normalizeAPath{
  my ($srPath)=@_;
  unless($^O=~/mswin/i){
    return $srPath;
  }
  my $returnvalue = $srPath;
  $returnvalue =~ s/\\/\//g;
  $returnvalue =~ tr/[A-Z]/[a-z]/;
  return $returnvalue;
}





sub copyFile{
  my ($src,$dst)=@_;
  $src=~s/\\/\//g;
  $dst=~s/\\/\//g;
  my ($buf,$written);
  -f $src or return "File \"$src\" not found\n";
  my @statbuf = stat($src);
  open(SRC,$src) or  return "Cannot open file \"$src\": $!\n";
  binmode(SRC);
  open(DST,">$dst") or  return "Cannot create $dst: $!\n" ;
  my $bufSize=(stat SRC)[11] || 32768;
  while (my $len = sysread(SRC,$buf,$bufSize)) {
    unless(defined $len){
      next if $! =~ /^Interrupted/;
      return "Read failure: $!\n";
    } 
    my $offset=0;
    while ($len) {
      $written=syswrite(DST,$buf,$len,$offset);
      defined $written or return "Write failure: $!\n";
      $len-=$written;
      $offset+=$written;
    }
  }
  close(SRC);
  close(DST);
  utime ($statbuf[8],$statbuf[9],$dst) or return "Cannot set time of copied file \"$dst\": $!\n";
  return 0;
}



sub callSystem{
  my ($cmd, $chk) = @_;
  my $pipehandle=SAPDB::Install::PipeExec::new();
  $pipehandle->Open($cmd);
  my $res = $pipehandle->GetError;
  if($^O =~ /mswin/i and defined $SAPDB::Install::Values::RefreshGui){
		SAPDB::Install::PipeExec::SetCallback($pipehandle->{sess},
			$SAPDB::Install::Values::RefreshGui);
  }
  if ($res) { return $res; }
  my $text;
  do{
    $text = $pipehandle->Readline();
    if ($text and $chk) {
      &$chk($text);
    }
  }until (not defined $text);
  $res = $pipehandle->GetError;
  if ($res) { return $res; }
  $res = $pipehandle->Close();
  return $res;
}



my $attrRegexp =  '((\w+)="([^"]*)")';
my $beginElementRegexp = '^\s*<\s*(\w+)(\s*('. $attrRegexp. '\s*)*)>\s*$';
my $emptyElementRegexp=  '^\s*<\s*(\w+)(\s*('. $attrRegexp .'\s*)*)\/>\s*$';
my $endElementRegexp = '^\s*<\/\s*(\w+)\s*>\s*$';
my $simpleElementRegexp = '^\s*<\s*(\w+)\s*>([^<]*)</\s*\1\s*>\s*$';


# filters to one tag only
sub filterXMLTag{
  my ($lines,$tag,$attsRegexp) = @_;
  my @retVal;
  my @tokens;
  my $skip=1;
  foreach(@$lines){
    if ( @tokens =(/$beginElementRegexp/)) {
      my $name = $1;
      my $atts = $2;
      if($name eq $tag){
        if($attsRegexp and $atts and $atts =~ /$attsRegexp/){
            $skip=0;
            push @retVal, ($_);
        }
        elsif(not $attsRegexp){
          $skip=0;
          push @retVal, ($_);
        }
      }
      else{
        $skip or push @retVal, ($_);
      }
    }
    elsif ( @tokens =(/$emptyElementRegexp/)) {
      my $name = $1;
      my $atts = $2;
      if($name eq $tag){
        if($attsRegexp and $atts and $atts =~ /$attsRegexp/){
          push @retVal, ($_);
        }
        elsif(not $attsRegexp){
          push @retVal, ($_);
        }
      }
      else{
        $skip or push @retVal, ($_);
      }
    }
    elsif ( /$endElementRegexp/) {
      my $name = $1;
      if($name eq $tag){
        not $skip and push @retVal , ($_);
        $skip=1;
      }
      else{
        $skip or push @retVal, ($_);
      }
    }
    else{
      $skip or push @retVal, ($_);
    }
  }
  return @retVal;
}

sub parseAttributes{
  my ($as)= @_;
  my %atts;
  while($as and $as =~ /$attrRegexp/){
    $atts{$2}=$3;
#    print "$2='$3'\n";
    $as =~ s/$attrRegexp//;
  }
  return \%atts;
} 


#parses pseudo xml. only one tag per line.
# first argument is ref to line array
# second argument may be an object ref
# callback functions must be defined according to naming 
#convention
sub parseXML{
  my ($lines,$obj,$handlers) = @_;
  my @stack;
  my $retVal; 
  my $pkg = caller();
#  print "Caller $pkg\n";
  #Parse the lines
  foreach(@$lines){
    my @tokens;
    if ( @tokens =(/$beginElementRegexp/)) {
    #Element begin:
      push @stack, (\@tokens);
      $retVal = beginElement(\@tokens,\@stack,$obj,$pkg,$handlers) and return $retVal;
    }
    elsif ( @tokens =(/$emptyElementRegexp/)) {
    #Empty element
      push @stack, (\@tokens);
      $retVal = beginElement(\@tokens,\@stack,$obj,$pkg,$handlers) and return $retVal;
      $retVal = endElement($tokens[0],\@stack,$obj,$pkg,$handlers) and return $retVal;
      delete $stack[$#stack];
      #    print "<$1>\n";
    }
    elsif ( @tokens =(/$simpleElementRegexp/)) {
    #one line element
      my ($name,$txt) = @tokens;
      my @st = ($name);
      push @stack, (\@st);
      $retVal = beginElement(\@st,\@stack,$obj,$pkg,$handlers) and return $retVal;
      $retVal = rawData($txt,\@stack,$obj,$pkg,$handlers) and return $retVal;
      $retVal = endElement($name,\@stack,$obj,$pkg,$handlers) and return $retVal;
      delete $stack[$#stack];
    }
    elsif ( /$endElementRegexp/) {
    #Element end 
      my $tokens = $stack[$#stack];
      my ($currentName,$newName) = ($tokens->[0],$1);
      if ( $currentName eq $newName) {
        $retVal = endElement($currentName,\@stack,$obj,$pkg,$handlers) and return $retVal;
        delete $stack[$#stack];
      } else {
        return "$newName does not close  '$currentName'";
      }
    } elsif ( /^\s*<[^<]+>\s*$/) {
      return "Cannot parse: '$_'";
    }
    elsif ( /^\s*<[^<]+>.*<\/[^<]+>\s*$/) {
      return "Cannot parse: '$_'";
    }
    else{
      $retVal = rawData($_,\@stack,$obj,$pkg,$handlers) and return $retVal;
    }
  }
  if (@stack) {
    my $tokens = $stack[$#stack];
    return "Unclosed element: " . $tokens->[0];
  }
  return $retVal;
}
sub rawData{
  my ($line,$stack,$obj,$pkg,$handlers) = @_;
  my $retVal =0;
  my $funcName = $pkg.'::cdata';
  my @params = $obj ? ($obj,$line,$stack) : ($line,$stack);
  if ($handlers) {
    $retVal = callHandler('cdata',$handlers,\@params) and return $retVal;
  } else {
    $retVal = callSoftRefs($funcName,\@params) and return $retVal;
  }
  return $retVal;
}                               #rawData

sub callHandler{
  my ($funcName,$handlers,$params) = @_;
  my $retVal =0;
  my $refSort = ref $handlers;
  if ( 'CODE' eq $refSort) {
       &$handlers(@$params);
  }
  elsif('HASH' eq $refSort){
    my $smth =  $handlers->{$funcName};
    $smth or $smth = $handlers->{'DEFAULT'};
    $smth and callHandler(undef,$smth,$params);
  }
  else{
    return "Cannot handle reference $refSort as handler";
  }
  return $retVal;
}
sub callSoftRefs{
  my ($funcName,$params) = @_;
  if ( defined &{$funcName}) {
    no strict 'refs';
    &{$funcName}(@$params);
  } else {
    return "'$funcName' not found";
  }
  return 0;
}

sub beginElement{
  my ($tokensRef,$stack,$obj,$pkg,$handlers) = @_;
  my ($name,$retVal,$atts) = ($tokensRef->[0],0,parseAttributes($tokensRef->[1]));
  my $funcName = $pkg.'::begin'.$name;
  my @params = $obj ? ($obj,$atts,$stack) : ($atts,$stack);
  if ($handlers) {
    $retVal = callHandler('begin'.$name,$handlers,\@params) and return $retVal;
  } else {
    $retVal = callSoftRefs($funcName,\@params) and return $retVal;
  }
  return $retVal;
}#beginElement

sub endElement{
  my ($name,$stack,$obj,$pkg,$handlers) = @_;
  my ($retVal) = (0);
  my $funcName = $pkg.'::end'.$name;
  my @params = $obj ? ($obj,$name,$stack) : ($name,$stack);
  if ($handlers) {
    $retVal = callHandler('end'.$name,$handlers,\@params) and return $retVal;
  } else {
    $retVal = callSoftRefs($funcName,\@params) and return $retVal;
  }
  return $retVal;
} #endElement



sub unitTest{
  my $curPack = shift;
  my @retVal;
  my $res;
  no strict 'refs';
  foreach my $symName (keys (%{*{$curPack . '::'}})) {
    if ( $symName =~ /^test/) {
      my $testFunc = $curPack.'::'.$symName;
      my $setUp = $curPack.'::setUp';
      my $tearDown = $curPack.'::tearDown';
      if (defined &$setUp) {
        &$setUp() ;
        $res = &$testFunc();
        $res and push @retVal, ('['.$symName.'] '.$res . "\n");
        &$tearDown() ;
      }
    }
  }
  return @retVal;
}

sub formatStack{
  my ($package,$file,$line,$subroutine) = @_;
  if($package){
    $package =~ /^.*::([^:]+)$/;
    if($1){
      $package = $1;
    }
  }
  else{
    $package= "<UNKNOWN>";
  }
  if($file){
  }
  else{
    $file= "<UNKNOWN>";
  }
  if($line){
  }
  else{
    $line= "<UNKNOWN>";
  }
  if($subroutine){
    $subroutine =~ /^.*::([^:]+)$/;
    if($1){
      $subroutine = $1;
    }
  }
  else{
    $subroutine= "<UNKNOWN>";
  }
  return $subroutine . " in " . $file . " line " . $line;
}
sub dieWithStack{
  my ($msg) = @_;
  $msg = "\n". $msg . "\n";
  my @c;
  my $i=0;
  while(@c = caller($i++)){
    $msg .= formatStack(@c) . "\n";
  }
  die $msg;
}



sub utilInstanceExists{
  my ($dbname,$dependendPath) = @_;
  my @instances =  utilFindInstances($dependendPath);
  my $f=0;
  foreach(@instances){
    if((uc $_) eq (uc $dbname)){
      $f=1;
      last;
    }
  }
  return $f;
}

sub utilFindInstances{
  my ($dependendPath)  = @_;
  my $this = shift;
  my $mainPath;
  $dependendPath and $mainPath = normalizeAPath($dependendPath);
  my %instances = SAPDB::Install::Registry::getInstances();  	
  my @retVal;
  foreach my $instance_name (keys(%instances)){
    my $dbroot = normalizeAPath($instances{$instance_name});
    if($mainPath){
      if($dbroot eq $mainPath ){
        push(@retVal,($instance_name));
      }
    }
    else{
      push(@retVal,($instance_name));
    }
  }
  return @retVal;
}










1;

