#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/Trace.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $ 
# $Change: 124736 $
#
# Desc: writes a trace if activated

package SAPDB::Install::Trace;

require SDB::Common::Exporter;

our @ISA = qw (SDB::Common::Exporter);
our @EXPORT = qw (TraceLevel TraceMsg WriteTrace);

use SAPDB::Install::Values;
use SAPDB::Install::StdIO;
use SAPDB::Install::Registry;

my @msgbuff;
my $level = 0; # 0 - 5, 0 is default


# 0 -> no trace
# 1 -> STDOUT, STDERR, STDIN
# 2 -> LOG
# 3 -> medium trace output (main flow decisions)
# 4 -> more trace (creation / destruction of objects, minor flow decisions)
# 5 -> lot of details (values)

sub TraceLevel{
	my ($lvl) = @_;
	$level = $lvl;
};

sub TraceMsg{
	my ($msg,$lvl,$debug,$caller_level) = @_;
	$level == 0 && !$$debug && return 1;
	$caller_level =~ /^\d+$/ or $caller_level = 1; 
	my ($package, $filename, $line, $subroutine,$hasargs, $wantarray, $evaltext, $is_require) = caller ($caller_level);
	my $fmt_msg = $subroutine."(): $msg";
	$$debug && print "DEBUG: $fmt_msg";
	$fmt_msg = "$lvl: $fmt_msg";
	$fmt_msg =~ s/\n/\n$lvl: => /gs;
	$fmt_msg =~ s/$lvl:\s=>\s$//s;
	$level < $lvl || push @msgbuff , $fmt_msg;
	return 1; 
}

sub WriteTrace{
	my ($force) = @_;
	$force || $level > 0 || return;
	$#msgbuff == -1 and print2stdout ("no trace in buffer\n") and return 1;  
	my $filename = $SAPDB::Install::Values::TraceFileName;
	my ($data,$prog) = readIndepPath();
	my $path = "$data/wrk";
	-d $path or $path = $SAPDB::Install::Values::curDir;
	open (TRACE, ">$path/$filename") || print2stderr ("cannot create trace file \"$path/$filename\"\n");
	foreach my $line (@msgbuff){
		print TRACE $line;
	}
	close (TRACE);
	return 1;
}

1;