#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/Uninstall/Start.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $
# $Change: 124736 $
#
# Desc: start uninstallation SAP DB packages


package SAPDB::Install::Uninstall::Start;

require SDB::Common::Exporter;

use SDB::Common::Getopt::Long;

use SAPDB::Install::Values;
use SAPDB::Install::System;
use SAPDB::Install::Registry;
use SAPDB::Install::StdIO;
use SAPDB::Install::InstallRegistry;
use SAPDB::Install::Version;
use SAPDB::Install::MD5Sum;
use SAPDB::Install::SetDebug;
use SAPDB::Install::Log;
use SAPDB::Install::Tools;
use SAPDB::Install::Cwd;
use SAPDB::Install::Misc;
use SAPDB::Install::Cleaner;
use SAPDB::Install::Uninstall::Collector;
use SAPDB::Install::Diagnose::InstallRegistry;
use SAPDB::Install::Migrate::Registry;
use SAPDB::Install::SigHandler;
use SAPDB::Install::Cleaner;
use SAPDB::Install::Trace;
use SAPDB::Install::Uninstall::RemoveFilesInUse;
use SAPDB::Install::Term::ReadKey;

our @ISA = qw (SDB::Common::Exporter);

our @EXPORT = qw (main);
our $DEBUG = 0;

sub uninstall{
	my ($regpackobj,$ext_args)=@_;
	my @undeleted;
	defined $regpackobj || print2stderr("package not defined\n") && diesoft($diemsg);
	my %regmd5sums = %{$regpackobj->FileList};
	my @delete_later;
	foreach my $file (keys(%regmd5sums)){
		my $realpath_name;
		my $fullpath = $regpackobj->Path."/$file";
		-f $fullpath or print2stderr("cannot delete file \"$fullpath\": no such file\n") and next; 
		if($^O =~ /mswin/i and exists $ext_args->{'callback_progress'}){
				$realpath_name = getRealPathName($fullpath);
		}
		if($fullpath =~ /\/rmfile$/){
			push  @delete_later,$fullpath;
			$SAPDB::Install::Values::log->SetMsg("MSG: skipping \"$fullpath\", delete it later\n")
				if defined $SAPDB::Install::Values::log;
			next;	
		}					
		#if($regmd5sums{$file} eq MD5Sum($fullpath)){
			if(exists $ext_args->{'callback_progress'}){
				my $path = $fullpath;
				$path = $realpath_name if $^O =~ /mswin/i; 
				$ext_args->{'callback_progress'}($dummy,-1,'Deleting '.$path);
			}
			unless(unlink($fullpath)){
				push @undeleted, $fullpath;
				if($^O =~ /mswin/i){
					$regpackobj->Registry->Log->SetMsg("cannot delete file \"$fullpath\": $!\n");
				}
				else{	
					print2stderr("cannot delete file \"$fullpath\": $!\n");
				}
			}
			$regpackobj->Registry->Log->SetMsg("file \"$fullpath\" deleted\n") unless (-f $fullpath);
		#}
		#else{
		#	print2stderr("dont delete file \"$fullpath\": file modified\n");
		#}
	}
	return \@undeleted,\@delete_later;	
}


sub main {
 	local @ARGV=@_;
	my $argvstring = join(' ',@ARGV);
	SetDebug(\@ARGV);
	if (printVersion(\@ARGV)){
		return 0;
	}
 	%opt_ctrl=(
		'h' => \$opt_help,
		'help' => \$opt_help,
		'package=s' => \$opt_displayname,
		'packagetype=s' => \$opt_packname,
		'package_dir=s' => \$opt_instpath,
		'autoresolve' => \$opt_delete_dependent,
		'all' => \$opt_all,
		'l' => \$opt_list,
		'list' => \$opt_list
	);	
	
	my $usage= " [-h|--help] | [-v|--version] | [-l|--list] |\n".
				"\t-all | -package <package name> [-package_dir <package directory>] [-autoresolve] \n"; 
	my $usage_desc =	"\t\t-h --help\t\t\tshow this\n".
				"\t\t-v --version\t\t\tshow version\n".
				"\t\t-l --list\t\t\tlist installed packages\n".
				"\t\t-package <package name>\t\tuninstall a package with such name\n".
				"\t\t-package_dir <package directory>\t\tuninstall package <package name> installed in <package directory>\n".
				"\t\t-all\t\t\tuninstall all registered MaxDB packages\n".
				"\t\t-autoresolve\t\tuninstall package <package name> and all dependent packages\n";	
						
	SDB::Common::Getopt::Long::Configure ('no_pass_through');					
	if(!GetOptions(%opt_ctrl) || $opt_help){
		print ("\n\nusage: ".$SAPDB::Install::Config{'ProgramName'}."$usage \n\n $usage_desc\n\n\n");
		if($SAPDB::Install::Config{'HasOwnConsole'}){
			print2stdout("\npress any key...");
			SAPDB::Install::Term::ReadKey::ReadMode('raw');
			SAPDB::Install::Term::ReadKey::ReadKey();
		}
		die("\n\n\n");
	}
 	
	$SAPDB::Install::Values::diemsg = 'MaxDB uninstall tool exited abnormally';


	setSigHandler();
	
	unless($opt_list){
		 unless (IsAdmin ()){
			print STDERR ($^O =~ /mswin/i ? "please start uninstallation as local administrator!\n" : "please restart as user root!\n");
			if($SAPDB::Install::Config{'HasOwnConsole'}){
				print2stdout("\npress any key...");
				SAPDB::Install::Term::ReadKey::ReadMode('raw');
				SAPDB::Install::Term::ReadKey::ReadKey();
			}	
			die("\n\n\n");
		 }
		 		
	}
	local $log = SAPDB::Install::Log->new();
	$log->SetMsg('CALL: '.$SAPDB::Install::Config{'ProgramName'}." $argvstring\n");
	$SAPDB::Install::Values::log=$log;
	$log->LogName("MaxDB Uninstall");
	local $cleaner = SAPDB::Install::Cleaner->new();
	$cleaner->SetRef(\$SAPDB::Install::Values::log);
	my ($data_path,$prog_path)=readIndepPath();
 	-d $data_path || print2stderr("no data path found\n") && diesoft($diemsg);
 	-d "$data_path/config/install" || print2stderr("path \"$data_path/config/install\" not found\n") && diesoft($diemsg);
 	setPathInEnv("$prog_path/bin");
	setPathInEnv("$prog_path/pgm") if $^O =~ /mswin/i;
	local $instRegistry=SAPDB::Install::InstallRegistry->new("$data_path/config/install",! $opt_list,$log);
 	defined $instRegistry || print2stderr("cannot find install registry\n") && diesoft($diemsg); 	
	migrateReg($instRegistry);
	if($opt_list){
		$log->LogOff(1); # write no logfile
		$instRegistry->printPackageList($opt_packname);
		if($SAPDB::Install::Config{'HasOwnConsole'}){
			print2stdout("\npress any key...");
			SAPDB::Install::Term::ReadKey::ReadMode('raw');
			SAPDB::Install::Term::ReadKey::ReadKey();
		}
		return 0;
	}
	$cleaner->SetRef(\$instRegistry);
	my @packages;
	if($opt_all){
		@packages=collect('instreg' => \$instRegistry, 'collect_all' => 1);
	}	
	else{
		($opt_packname =~ /\S/ or $opt_displayname) or print2stdout("no packagename\n") and diesoft($diemsg);
		$DEBUG && print "DEBUG: start collection\n";
		my $instpath;
		if($opt_instpath){
			$instpath = $opt_instpath;	
			normalizePath(\$instpath);
		}
		@packages=collect('instreg' => \$instRegistry,'package_name' => $opt_packname,'display_name' => $opt_displayname,'package_path' => $instpath, 'auto_resolve' => $opt_delete_dependent); 	
		$DEBUG && print "DEBUG: collection finsished: ".($#packages+1)." packages\n";
	}
	$#packages == -1 && print2stderr("no package selected\n") && diesoft($diemsg);	
	
    	
	
	foreach my $regpackobj (@packages){	
		$regpackobj->evalScript;
		if(defined $regpackobj->preuninstall){
			eval{
				&{$regpackobj->preuninstall};
			}; $@ and print2stderr("error in preuninstall(): $@\n");
		}
	}
	
	$instRegistry->DenyWrite(0); # scharf schalten
	
	my @unremoved_dirs;
	my @removed_dirs;
	my ($data,$prog) = readIndepPath();
	
	my ($sdb_uid, $sdba_gid, $sdb_user, $sdba_group);
 	
 	unless ($^O =~ /mswin/i){
 		my $globals = readGlobals();
 		if (exists $globals->{'SdbOwner'}){
 			$sdb_user = $globals->{'SdbOwner'};
 			$sdb_uid = getpwnam ($sdb_user);
 		}
 		if (exists $globals->{'SdbGroup'}){		
 			$sdba_group = $globals->{'SdbGroup'};
 			$sdba_gid = getgrnam ($sdba_group);
 		}
 	}

	
	foreach my $regpackobj (@packages){	
		$DEBUG && print "DEBUG: uninstallation of package ".$regpackobj->DispName.' ('.$regpackobj->Path.")\n";
		$regpackobj->setValid(0);
		if(defined $regpackobj->unregister){
			eval{
				&{$regpackobj->unregister};
			}; $@ and print2stderr("error in unregister(): $@\n");
		}
		
		my ($busy,$delete_later) = uninstall($regpackobj);
		my  @files_left = @$busy;
		
		my @undeleted_files;		
		if(defined $regpackobj->postuninstall){
				eval{
				@undeleted_files = &{$regpackobj->postuninstall};
			}; $@ and print2stderr("error in postuninstall(): $@\n");
		}
		
		if($#undeleted_files != -1){
			foreach my $file (@undeleted_files){
				push @files_left,$file if -f $file;
			}
		}
		
		if($^O =~ /mswin|hpux/i and $#files_left > -1){
			killUsedFiles($prog,@files_left);
		}
		
		foreach my $file (@$delete_later){
			if(unlink($file)){
				$SAPDB::Install::Values::log->SetMsg("MSG: $file deleted\n")
					if defined $SAPDB::Install::Values::log;
			}
			else{
				$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete $file: $!\n")
					if defined $SAPDB::Install::Values::log;
			}
		}
		
		if(defined $regpackobj->ParentPackage){
			my $parent_id = $regpackobj->ParentPackage;
			if(defined $parent_id){
				my $parent = $instRegistry->getPackage(@$parent_id);
				if(defined $parent){
					my @subpackages = @{$parent->SubPackages};
					my @new_subpackages;
					foreach my $subpackage (@subpackages){
						unless($subpackage->[0] eq $regpackobj->Name and
						   $subpackage->[1] eq $regpackobj->Path){
							push @new_subpackages,$subpackage;							
						}	
					}
					$parent->SubPackages(\@new_subpackages);
					$parent->setValues();
					$SAPDB::Install::InstallRegistry::Package::SELFOBJ[$parent->ID] = undef; 	
				}
			}
		}
		
		# remove package from install registy
		$instRegistry->removePackage($regpackobj->Name,$regpackobj->Path);
		
		# try to delete directories
		my $main_path_inside = 0; 
		my $drive = '';
		
		if(defined $regpackobj->Pathes){
			foreach my $dirref (@{$regpackobj->Pathes}){	
				if(normalizePath($dirref->{'value'}) eq normalizePath($regpackobj->Path)){
  					$main_path_inside = 1;
				}
				if($^O =~ /mswin/i){
					($drive) = ($dirref->{'value'} =~ /^([a-z]:)/i); 
					`$drive`;
				}
				chdir($drive.'/'); # change into root directory
				my ($rc,$ref_removed,$ref_unremoved) = removeEmptyDirs( $dirref->{'value'});
				@unremoved_dirs = union (\@unremoved_dirs,$ref_unremoved);  
				@removed_dirs = union (\@removed_dirs,$ref_removed);  

				my $subpath = $dirref->{'value'};

				while(1){
					$subpath =~ s/\/[^\/]+$//;
					if ($^O =~ /mswin/i){
						if ($subpath !~ /[\\\/]/){
							last;
						}
						opendir(SDH,$subpath);
						my @entries = grep {not /^\.{1,2}$/} readdir (SDH);
						closedir(SDH);
						last if $#entries > 3;
					}
					if(-d $subpath){
						my ($lrc,$lref_removed,$lref_unremoved) = 
						removeEmptyDirs($subpath,{'uid' => $sdb_uid, 'gid' => $sdba_gid});
 						@removed_dirs = union (\@removed_dirs,$lref_removed);
					}
					else{
						last;
					}
				}
			}
		}
		else{
			TraceMsg("member pathes not defined\n",4,\$DEBUG);
		}
		unless($main_path_inside){
			TraceMsg("main path not part of pathes\n",4,\$DEBUG);
			if($^O =~ /mswin/i){
					($drive) = ($regpackobj->Path =~ /^([a-z]:)/i); 
					`$drive`;
			}
			chdir($drive.'/'); # change into root directory
			my ($rc,$ref_removed,$ref_unremoved) = removeEmptyDirs($regpackobj->Path);
			
			my $subpath = $regpackobj->Path;
			
			while(1){
				$subpath =~ s/\/[^\/]+$//;
				if ($^O =~ /mswin/i){
						if ($subpath !~ /[\\\/]/){
							last;
						}
						opendir(SDH,$subpath);
						my @entries = grep {not /^\.{1,2}$/} readdir (SDH);
						closedir(SDH);
						last if $#entries > 3;
					}
				if(-d $subpath){
					my ($lrc,$lref_removed,$lref_unremoved) = 
						removeEmptyDirs($subpath,{'uid' => $sdb_uid, 'gid' => $sdba_gid});
 						@removed_dirs = union (\@removed_dirs,$lref_removed);  
				}
				else{
					last;
				}
			}

			@unremoved_dirs = union (\@unremoved_dirs,$ref_unremoved);  
			@removed_dirs = union (\@removed_dirs,$ref_removed);  
		}
	}
	
	
	# normalize pathes

	if($^O =~ /mswin/i){
		my @tmp;
		foreach my $path (@unremoved_dirs){
			my $tmp = $path;
			normalizePath(\$tmp);
			push @tmp,$tmp;
		}
		@unremoved_dirs=@tmp;
		
		@tmp=();
		foreach my $path (@removed_dirs){
			my $tmp = $path;
			normalizePath(\$tmp);
			push @tmp,$tmp;
		}
		@removed_dirs=@tmp;	
	}
	
	
	@unremoved_dirs = removeDoublesFromList(\@unremoved_dirs);
	@removed_dirs = removeDoublesFromList(\@removed_dirs);
	
	@unremoved_dirs = minus(\@unremoved_dirs,\@removed_dirs);
	
	$instRegistry->RefreshAllFilesList;
	my %existing_files = %{$instRegistry->AllFiles};
		
	foreach my $dir (sort @unremoved_dirs){
		my $found = 0;
		my $pattern = $dir;
		$pattern =~ s/\//\\\//g; # mask slashes
		$pattern = '^'.$pattern;
		foreach my $file (keys(%existing_files)){
			$file =~ /$pattern/ and $found = 1 and last;
		}
		print2stderr("cannot remove directory \"$dir\"\n") unless $found;
	}
	print2stdout("done\n");
	
	if($SAPDB::Install::Config{'HasOwnConsole'}){
		print2stdout("\npress any key...");
		SAPDB::Install::Term::ReadKey::ReadMode('raw');
		SAPDB::Install::Term::ReadKey::ReadKey();
	}

	return 0;
}

1;